@@ -7,29 +7,35 @@ from .dates import __all__ as dates_all
 from .series import *
 from .series import __all__ as series_all
 
-from .plotly_wrap import *
-from .plotly_wrap import __all__ as plotly_wrap_all
+from .ez_plotly import *
+from .ez_plotly import __all__ as ez_plotly_all
 
 from .rephrases import *
 from .rephrases import __all__ as rephrases_all
 
-from .databoxes.main import *
-from .databoxes.main import __all__ as databoxes_all
+from .databoxes import *
+from .databoxes import __all__ as databoxes_all
+
+from .dataslates import *
+from .dataslates import __all__ as dataslates_all
 
 from .chartpacks.main import *
 from .chartpacks.main import __all__ as chartpacks_all
 
-from .dataslates.main import *
-from .dataslates.main import __all__ as dataslates_all
-
 from .sources import *
 from .sources import __all__ as sources_all
 
 from .simultaneous import *
 from .simultaneous import __all__ as simultaneous_all
 
-from .vector_autoregressions import *
-from .vector_autoregressions import __all__ as vector_autoregressions_all
+from .stackers import *
+from .stackers import __all__ as stackers_all
+
+from .fords import *
+from .fords import __all__ as fords_all
+
+from .vars import *
+from .vars import __all__ as vars_all
 
 from .quantities import *
 from .quantities import __all__ as quantities_all
@@ -52,6 +58,12 @@ from .namings import __all__ as namings_all
 from .file_io import *
 from .file_io import __all__ as file_io_all
 
+from .portables import *
+from .portables import __all__ as portables_all
+
+from .progress_bars import *
+from .progress_bars import __all__ as progress_bars_all
+
 
 __version__ = _md.version(__name__)
 __doc__ = _md.metadata(__name__).json["description"]
@@ -83,12 +95,15 @@ def _convert_version(version_str: str) -> tuple[int, ...]:
 __all__ = (
     *dates_all,
     *series_all,
-    *plotly_wrap_all,
+    *ez_plotly_all,
     *rephrases_all,
     *databoxes_all,
+    *dataslates_all,
     *sources_all,
     *simultaneous_all,
-    *vector_autoregressions_all,
+    *stackers_all,
+    *fords_all,
+    *vars_all,
     *quantities_all,
     *equations_all,
     *sequentials_all,
@@ -96,8 +111,9 @@ __all__ = (
     *plans_all,
     *namings_all,
     *file_io_all,
+    *portables_all,
+    *progress_bars_all,
     "min_irispie_version_required",
     "__version__",
 )
 
-
