@@ -20,16 +20,9 @@ class AttributesProtocol(Protocol, ):
     #]
 
 
-class AttributesMixin:
-    """
-    """
-    #[
-
-    def has_attributes(self, *attributes) -> bool:
-        attributes = (set((a, )) if isinstance(a, str) else set(a) for a in attributes)
-        return all(self.attributes.intersection(a, ) for a in attributes)
-
-    #]
+def has_attributes(self, *attributes, ) -> bool:
+    attributes = (set((a, )) if isinstance(a, str) else set(a) for a in attributes)
+    return all(self.attributes.intersection(a, ) for a in attributes)
 
 
 def generate_by_attributes(
