@@ -4,25 +4,28 @@ Time series chartpacks
 
 
 #[
+
 from __future__ import annotations
 
 import functools as _ft
 import re as _re
 import math as _ma
+import documark as _dm
 import plotly.graph_objects as _pg
+from collections.abc import Sequence
 
 from ..conveniences import descriptions as _descriptions
 from ..conveniences import copies as _copies
-from .. import plotly_wrap as _plotly_wrap
+from .. import ez_plotly as _ez_plotly
 from .. import dates as _dates
-from .. import pages as _pages
 from ..databoxes import main as _databoxes
 
-from typing import (TYPE_CHECKING, )
+from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import (Self, Callable, )
-    from collections.abc import (Iterator, Iterable, Sequence, )
-    from types import (EllipsisType, )
+    from typing import Self, Callable
+    from collections.abc import Iterator, Iterable
+    from types import EllipsisType
+
 #]
 
 
@@ -36,6 +39,8 @@ _FIGURE_SETTINGS = {
     "highlight": None,
     "tiles": None,
     "shared_xaxes": False,
+    "figure_height": None,
+    "vertical_spacing": None,
 }
 _FIGURE_SETTINGS_KEYS = tuple(_FIGURE_SETTINGS.keys(), )
 
@@ -46,6 +51,7 @@ _CHART_SETTINGS = {
     "reverse_plot_order": False,
     "span": ...,
     "chart_type": "line",
+    "update_traces": None,
 }
 _CHART_SETTINGS_KEYS = tuple(_CHART_SETTINGS.keys(), )
 
@@ -55,10 +61,11 @@ _CHART_INPUT_STRING_PATTERN = _re.compile(
 )
 
 
-@_pages.reference(
+@_dm.reference(
     path=("visualization_reporting", "chartpacks.md", ),
     categories={
         "constructor": "Creating new chartpacks",
+        "information": "Getting information about chartpacks",
         "property": None,
         "plot": "Plotting chartpacks",
         "add": "Adding figures and charts to chartpacks",
@@ -81,9 +88,9 @@ Chartpacks
     __slots__ = (
         "title",
         "_figures",
-        "_description",
         "_figure_settings",
         "_chart_settings",
+        "__description__",
     )
 
     def __init__(
@@ -95,7 +102,7 @@ Chartpacks
         """
         self.title = title
         self._figures = None
-        self._description = None
+        self.__description__ = None
         #
         self._figure_settings = {
             n: kwargs[n]
@@ -110,7 +117,7 @@ Chartpacks
         }
 
     @classmethod
-    @_pages.reference(category="constructor", call_name="Chartpack", )
+    @_dm.reference(category="constructor", call_name="Chartpack", )
     def _constructor_doc():
         """
 ················································································
@@ -169,7 +176,7 @@ self = Chartpack(
         """
         pass
 
-    @_pages.reference(category="plot", )
+    @_dm.reference(category="plot", )
     def plot(
         self,
         input_db: _databoxes.Databox,
@@ -205,7 +212,7 @@ self = Chartpack(
             figure.format_figure_title(**kwargs, )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def num_figures(self, ) -> int:
         """==Total number of figures in the chartpack=="""
         return len(self._figures, ) if self._figures else 0
@@ -217,7 +224,7 @@ self = Chartpack(
             self._figures = []
         self._figures.append(figure, )
 
-    @_pages.reference(category="add", )
+    @_dm.reference(category="add", )
     def add_figure(self, figure_string: str, **kwargs, ) -> None:
         """
 ················································································
@@ -236,7 +243,7 @@ self = Chartpack(
         return new_figure
 
     @staticmethod
-    @_pages.reference(category="add", call_name="add_chart", )
+    @_dm.reference(category="add", call_name="add_chart", )
     def _add_chart_doc():
         """
 ················································································
@@ -340,18 +347,18 @@ class _Figure:
     def plot(
         self,
         input_db: _databoxes.Databox,
-        /,
         shared_xaxes: bool = False,
         **kwargs,
-    ) -> None:
-        """
+    ) -> _pg.Figure:
+        r"""
         """
         tiles = _resolve_tiles(self.tiles, self.num_charts, )
-        figure = _plotly_wrap.make_subplots(
-            rows=tiles[0],
-            columns=tiles[1],
+        figure = _ez_plotly.make_subplots(
+            rows_columns=(tiles[0], tiles[1], ),
             subplot_titles=tuple(chart.caption for chart in self),
             shared_xaxes=self.shared_xaxes,
+            figure_height=self.figure_height,
+            vertical_spacing=self.vertical_spacing,
         )
         for i, chart in enumerate(self, ):
             chart.plot(
@@ -359,7 +366,8 @@ class _Figure:
                 legend=self.legend if i == 0 else None,
             )
             if self.highlight is not None:
-                _plotly_wrap.highlight(figure, self.highlight, subplot=i, )
+                _ez_plotly.highlight(figure, self.highlight, subplot=i, )
+        #
         figure.update_layout(title={"text": self.title, }, )
         return figure
 
@@ -496,6 +504,7 @@ class _Chart:
             legend=legend,
             reverse_plot_order=self.reverse_plot_order,
             chart_type=self.chart_type,
+            update_traces=self.update_traces,
         )
 
     def _apply_transform(self, x, ):
@@ -522,7 +531,7 @@ class _Chart:
     def _one_liner(self, /, ) -> str:
         return f"Chart({self.title!r}, {self.expression!r}, {self.transform!r}, )"
 
-    def __iter__(self, ) -> NoReturn:
+    def __iter__(self, ) -> Iterable:
         yield from ()
 
     #]
