@@ -4,6 +4,7 @@ Exporting data to CSV sheets
 
 
 #[
+
 from __future__ import annotations
 
 from typing import (TYPE_CHECKING, )
@@ -13,15 +14,16 @@ import numpy as _np
 import itertools as _it
 import dataclasses as _dc
 import functools as _ft
+import documark as _dm
 
 from ..dates import (Frequency, Period, EmptySpan, )
 from .. import wrongdoings as _wrongdoings
-from .. import pages as _pages
 
 if TYPE_CHECKING:
     from collections.abc import (Iterable, )
     from typing import (Any, Callable, )
     from ..databoxes.main import (Databox, )
+
 #]
 
 
@@ -100,10 +102,9 @@ class Inlay:
     Databox inlay for writing databox time series to CSV file
     """
     #[
-    @_pages.reference(
-        category="import_export",
-    )
-    def to_csv(
+
+    @_dm.reference(category="import_export", )
+    def to_csv_file(
         self,
         file_name: str,
         *,
@@ -127,7 +128,7 @@ class Inlay:
 ==Write Databox time series to a CSV file==
 
 
-    self.to_csv(
+    self.to_csv_file(
         file_name,
         *,
         frequency_span=None,
@@ -241,19 +242,49 @@ class Inlay:
         else:
             return
 
+    @_dm.no_reference
     def to_sheet(self, *args, **kwargs, ):
+        r"""
+        """
         return self.to_csv(*args, **kwargs, )
 
+    # Legacy alias
+    to_csv = to_csv_file
+
+    @_dm.reference(category="import_export", )
     def to_pickle(
         self,
         file_name: str,
         /,
         **kwargs,
     ) -> None:
-        """
+        r"""
+................................................................................
+
+==Write Databox to a pickle file==
+
+    self.to_pickle(
+        file_name,
+        **kwargs, 
+    )
+
+### Input arguments ###
+
+???+ input "file_name"
+    Path to the pickle file where the data will be written.
+
+???+ input "kwargs"
+    Additional keyword arguments for the pickle writer.
+
+### Returns ###
+
+This method returns `None`.
+
+................................................................................
         """
         with open(file_name, "wb+") as fid:
             _pk.dump(self, fid, **kwargs, )
+
     #]
 
 
