@@ -4,6 +4,7 @@ Databox imports
 
 
 #[
+
 from __future__ import annotations
 
 from typing import (Self, )
@@ -13,10 +14,11 @@ import numpy as _np
 import dataclasses as _dc
 import pickle as _pickle
 import warnings as _wa
+import documark as _dm
 
 from ..dates import (Period, Frequency, Span, )
 from ..series.main import (Series, )
-from .. import pages as _pages
+
 #]
 
 
@@ -63,11 +65,11 @@ class Inlay:
     """
     #[
     @classmethod
-    @_pages.reference(
-        category="import_export",
-        call_name="Databox.from_csv",
+    @_dm.reference(
+        category="constructor",
+        call_name="Databox.from_csv_file",
     )
-    def from_csv(
+    def from_csv_file(
         klass,
         file_name: str,
         *,
@@ -89,7 +91,7 @@ class Inlay:
 ==Create a new Databox by reading time series from a CSV file==
 
 
-    self = Databox.from_csv(
+    self = Databox.from_csv_file(
         file_name,
         *,
         period_from_string=None,
@@ -110,7 +112,7 @@ class Inlay:
 
 ???+ input "period_from_string"
     A callable for creating date objects from string representations. If `None`,
-    a default method based on the SDMX string format is used.
+    a default method expecting the SDMX string format is used.
 
 ???+ input "start_period_only"
     If `True`, only the start date of each time series is parsed from the CSV;
@@ -179,19 +181,47 @@ class Inlay:
         return self
 
     @classmethod
+    @_dm.no_reference
     def from_sheet(klass, *args, **kwargs, ):
+        r"""
         """
-        """
-        return klass.from_csv(*args, **kwargs, )
+        return klass.from_csv_file(*args, **kwargs, )
+
+    # Legacy alias
+    from_csv = from_csv_file
 
     @classmethod
+    @_dm.reference(category="import_export", )
     def from_pickle(
         klass,
         file_name: str,
         /,
         **kwargs,
     ) -> Self:
-        """
+        r"""
+................................................................................
+
+==Read a Databox from a pickled file==
+
+    self = Databox.from_pickle(
+        file_name,
+        **kwargs,
+    )
+
+### Input arguments ###
+
+???+ input "file_name"
+    Path to the pickled file to be read.
+
+???+ input "kwargs"
+    Additional keyword arguments to pass to the `pickle.load` function.
+
+### Returns ###
+
+???+ returns "self"
+    A `Databox` object read from the pickled file.
+
+................................................................................
         """
         with open(file_name, "rb") as fid:
             return _pickle.load(fid, **kwargs, )
