@@ -4,49 +4,32 @@ Data arrays with row and column names
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (Self, Protocol, )
-from numbers import (Number, )
-from collections.abc import (Iterable, Iterator, )
+from typing import Self, Protocol
+from numbers import Real
+from collections.abc import Iterable, Iterator
 import numpy as _np
 import numpy as _np
 import functools as _ft
 import itertools as _it
 
-from ..series.main import (Series, )
-from ..databoxes.main import (Databox, )
+from ..series.main import Series
+from ..databoxes.main import Databox
 from ..incidences import main as _incidences
 from ..conveniences import iterators as _iterators
-from ..dates import (Period, Span, )
+from ..dates import Period, Span
 from .. import dates as _dates
 from .. import has_variants as _has_variants
 
-from . import _invariants as _invariants
-from . import _variants as _variants
-#]
-
-
-__all__ = (
-    "Dataslate",
-)
-
-
-class SlatableProtocol(Protocol, ):
-    """
-    """
-    max_lag: int
-    max_lead: int
-    databox_names: Iterable[str]
-    databox_validators: dict[str, Callable]
-    fallbacks: dict[str, Real]
-    overwrites: dict[str, Real]
-    output_names: Iterable[str]
-    qid_to_logly: dict[int, bool | None]
+from ._invariants import Invariant
+from ._variants import Variant
+from ._slatables import Slatable
 
 
 class Dataslate(
-    _has_variants.HasVariantsMixin,
+    _has_variants.Mixin,
 ):
     """
     """
@@ -57,21 +40,15 @@ class Dataslate(
         "_variants",
     )
 
-    def __init__(self, /, ) -> None:
-        self._invariant = None
-        self._variants = []
-
-    @classmethod
-    def skeleton(
-        klass,
-        other: Self,
-    ) -> Self:
+    def __init__(
+        self, 
+        invariant: Invariant | None = None,
+        variants: Iterable[Variant] | None = None,
+    ) -> None:
+        r"""
         """
-        """
-        self = klass()
-        self._invariant = other._invariant
-        self._variants = []
-        return self
+        self._invariant = invariant
+        self._variants = list(variants) if variants else []
 
     @classmethod
     def nan_from_names_periods(
@@ -89,9 +66,9 @@ class Dataslate(
         num_names = len(names)
         num_periods = len(periods)
         self = klass()
-        self._invariant = _invariants.Invariant(names, periods, **kwargs, )
+        self._invariant = Invariant(names, periods, **kwargs, )
         self._variants = [
-            _variants.Variant.nan_data_array(self._invariant, )
+            Variant.nan_data_array(self._invariant, )
             for _ in range(num_variants, )
         ]
         return self
@@ -107,7 +84,7 @@ class Dataslate(
         """
         self = klass.skeleton(other, )
         self._variants = [
-            _variants.Variant.nan_data_array(self._invariant, )
+            Variant.nan_data_array(self._invariant, )
             for _ in range(num_variants, )
         ]
         return self
@@ -116,19 +93,21 @@ class Dataslate(
     def from_databox(
         klass,
         databox: Databox | dict,
-        names: Iterable[str],
+        names: Iterable[str] | None,
         periods: Iterable[Period] | string,
         /,
         num_variants: int = 1,
-        fallbacks: dict[str, Number] | None = None,
-        overwrites: dict[str, Number] | None = None,
+        fallbacks: dict[str, Real] | None = None,
+        overwrites: dict[str, Real] | None = None,
         clip_data_to_base_span: bool = False,
         validators: dict[str, Callable] | None = None,
         **kwargs,
     ) -> Self:
         """
         """
-        names = tuple(names or databox.keys())
+        if names is None:
+            names = databox.keys()
+        names = tuple(names)
         periods = _dates.ensure_period_tuple(periods, )
         #
         if validators:
@@ -139,15 +118,15 @@ class Dataslate(
             _ft.partial(_slate_value_variant_iterator, from_until=from_until, )
         #
         databox_variant_iterator = \
-            Databox.iter_variants(databox, item_iterator=item_iterator, names=names, )
+            Databox.iter_variants(databox, item_iterator=item_iterator, keys=names, )
         #
         fallbacks_variant_iterator = (
-            Databox.iter_variants(fallbacks, item_iterator=item_iterator, names=names, )
+            Databox.iter_variants(fallbacks, item_iterator=item_iterator, )
             if fallbacks else _it.repeat(None, )
         )
         #
         overwrites_variant_iterator = (
-            Databox.iter_variants(overwrites, item_iterator=item_iterator, names=names, )
+            Databox.iter_variants(overwrites, item_iterator=item_iterator, )
             if overwrites else _it.repeat(None, )
         )
         #
@@ -159,9 +138,9 @@ class Dataslate(
         )
         #
         self = klass()
-        self._invariant = _invariants.Invariant(names, periods, **kwargs, )
+        self._invariant = Invariant(names, periods, **kwargs, )
         self._variants = [
-            _variants.Variant.from_databox_variant(
+            Variant.from_databox_variant(
                 databox_v, self._invariant,
                 fallbacks=fallbacks_v,
                 overwrites=overwrites_v,
@@ -174,7 +153,7 @@ class Dataslate(
     @classmethod
     def from_databox_for_slatable(
         klass,
-        slatable: SlatableProtocol,
+        slatable: Slatable,
         databox: Databox | dict,
         base_span: Iterable[Period],
         /,
@@ -262,6 +241,12 @@ class Dataslate(
         """
         self._invariant.base_periods = base_periods
 
+    @property
+    def base_start(self, /, ) -> Period:
+        """
+        """
+        return self._invariant.base_periods[0]
+
     @property
     def names(self, /, ) -> tuple[str]:
         """
@@ -356,6 +341,7 @@ class Dataslate(
         self,
         /,
         target_db: Databox | None = None,
+        span: Literal["base", "full", ] = "full",
     ) -> Databox:
         """
         Add data from a dataslate to a new or existing databox
@@ -363,14 +349,19 @@ class Dataslate(
         #[
         if target_db is None:
             target_db = Databox()
+        if span == "full":
+            start = self.start
+            column_slice = slice(None, None, )
+        elif span == "base":
+            start = self.base_start
+            column_slice = self.base_slice
         num_names = self.num_names
         num_variants = self.num_variants
-        start = self._invariant.periods[0]
         for qid in self._invariant.output_qids:
             name = self._invariant.names[qid]
             description = self._invariant.descriptions[qid]
             values = _np.vstack([
-                v.data[qid, :] for v in self._variants
+                v.data[qid, column_slice] for v in self._variants
             ]).T
             target_db[name] = Series._guaranteed(
                 start=start,
@@ -478,7 +469,7 @@ class Dataslate(
 
 
 def _get_extended_span(
-    slatable: SlatableProtocol,
+    slatable: Slatable,
     base_span: Iterable[Period],
     /,
     prepend_initial: bool,
@@ -494,7 +485,7 @@ def _get_extended_span(
     max_base_date = max(base_span)
     start_date = min_base_date + min_shift
     end_date = max_base_date + max_shift
-    base_columns = tuple(_dates.date_index(base_span, start_date))
+    base_columns = tuple(_dates.period_indexes(base_span, start_date, ))
     extended_dates = tuple(Span(start_date, end_date))
     return extended_dates, base_columns, min_shift, max_shift
 
