@@ -4,22 +4,24 @@ Model equations
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (Self, Iterable, Callable, )
+from typing import Self, Callable
+from collections.abc import Iterable, Sequence
 import enum as _en
 import re as _re
 import numpy as _np
-import dataclasses as _dc
 import itertools as _it
 import operator as _op
-import copy as _cp
+from dataclasses import dataclass
 
-from .incidences.main import (Token, )
-from .incidences import main as _incidence
+from .incidences.main import Token
+from .incidences import main as _incidences
 from . import quantities as _quantities
 from . import attributes as _attributes
 from . import wrongdoings as _wrongdoings
+
 #]
 
 
@@ -28,6 +30,9 @@ _EVALUATOR_FORMAT = EVALUATOR_PREAMBLE + "[{joined_xtrings}]"
 _REPLACE_UKNOWN = "?"
 
 
+_PortableType = tuple[str, str | None, str, str, str]
+
+
 class EquationKind(_en.Flag):
     """
     Classification of model equations
@@ -40,6 +45,16 @@ class EquationKind(_en.Flag):
 
     ENDOGENOUS_EQUATION = TRANSITION_EQUATION | MEASUREMENT_EQUATION
 
+    @classmethod
+    def from_portable(klass, portable: str, /, ) -> Self:
+        return _FROM_PORTABLES[portable]
+
+    def to_keyword(self, /, ) -> str:
+        return "!" + self.name.lower() + "s"
+
+    def to_portable(self, /, ) -> str:
+        return _TO_PORTABLES[self]
+
     @property
     def human(self, /, ) -> str:
         return self.name.replace("_", " ").title()
@@ -47,6 +62,18 @@ class EquationKind(_en.Flag):
     #]
 
 
+_TO_PORTABLES = {
+    EquationKind.TRANSITION_EQUATION: "#T",
+    EquationKind.MEASUREMENT_EQUATION: "#M",
+    EquationKind.STEADY_AUTOVALUES: "#A",
+}
+
+
+_FROM_PORTABLES = {
+    v: k for k, v in _TO_PORTABLES.items()
+}
+
+
 __all__  = (
     "TRANSITION_EQUATION",
     "MEASUREMENT_EQUATION",
@@ -57,10 +84,8 @@ for n in __all__:
     exec(f"{n} = EquationKind.{n}")
 
 
-@_dc.dataclass(slots=True, )
-class Equation(
-    _attributes.AttributesMixin,
-):
+@dataclass
+class Equation:
     """
     """
     #[
@@ -72,7 +97,7 @@ class Equation(
     xtring: str | None = None
     incidence: Iterable[Token] | None = None
     entry: int | None = None
-    attributes: set[str] = ()
+    attributes: set[str] | None = None
 
     def finalize(self, name_to_id: dict[str, int], ) -> None:
         """
@@ -82,7 +107,7 @@ class Equation(
         self.xtring, self.incidence, *_ = xtring_from_human(self.human, name_to_id, )
 
     def create_equator_func(self, /, *, context: dict[str, Callable]) -> Callable:
-        context = dict(context) if context else {}
+        context = (dict(context) if context else {}) | {"__builtins__": {}}
         equator_func_string = create_equator_func_string([self.xtring], context, )
         eval(equator_func_string, context, )
 
@@ -90,10 +115,40 @@ class Equation(
         """
         Shallow copy of the equation
         """
-        return _cp.copy(self, )
+        return type(self)(**self.__dict__, )
+
+    def to_portable(self, complement: Self, /, ) -> _PortableType:
+        """
+        """
+        return (
+            self.kind.to_portable(),
+            self.human,
+            complement.human if complement.human != self.human else None,
+            self.description,
+            " ".join(self.attributes, ),
+        )
+
+    @classmethod
+    def from_portable(klass, portable: _PortableType, /, ) -> Self:
+        kind, human, complement_human, description, attributes = portable
+        self = klass(
+            human=human,
+            kind=EquationKind.from_portable(kind),
+            description=description,
+            attributes=set(attributes),
+        )
+        complement = klass(
+            human=complement_human or human,
+            kind=EquationKind.from_portable(kind),
+            description=description,
+            attributes=set(attributes),
+        )
+        return self, complement,
 
     def __hash__(self, /, ) -> int:
-        return hash(self.__repr__)
+        return hash(self.__repr__, )
+
+    has_attributes = _attributes.has_attributes
 
     #]
 
@@ -150,7 +205,7 @@ def create_eid_to_wrt_tokens(
     #[
     eid_to_wrt_tokens = {}
     for eqn in equations:
-        eid_to_wrt_tokens[eqn.id] = _incidence.sort_tokens(
+        eid_to_wrt_tokens[eqn.id] = _incidences.sort_tokens(
             wrt for wrt in all_wrt_tokens
             if wrt in eqn.incidence
         )
@@ -183,7 +238,9 @@ def xtring_from_human(
         tokens_list.append(new_token)
         return new_token.print_xtring()
 
-    xtring = _quantities.QUANTITY_OCCURRENCE_PATTERN.sub(_replace_human_with_x, human)
+    xtring = human
+    xtring = xtring.replace(":=", "=")
+    xtring = _quantities.QUANTITY_OCCURRENCE_PATTERN.sub(_replace_human_with_x, xtring, )
     xtring = _postprocess_xtring(xtring)
     return xtring, set(tokens_list), tokens_list
     #]
@@ -235,14 +292,14 @@ def sort_equations(equations: Iterable[Equation], ) -> Iterable[Equation]:
 
 
 def get_min_shift_from_equations(equations: Iterable[Equation], ) -> int:
-    return _incidence.get_min_shift(generate_all_tokens_from_equations(equations))
+    return _incidences.get_min_shift(generate_all_tokens_from_equations(equations))
 
 
 def get_max_shift_from_equations(
     equations: Iterable[Equation],
     /,
 ) -> int:
-    return _incidence.get_max_shift(generate_all_tokens_from_equations(equations))
+    return _incidences.get_max_shift(generate_all_tokens_from_equations(equations))
 
 
 def validate_selection_of_equations(
@@ -320,6 +377,31 @@ def stamp_id(equations: Iterable[Equation], /) -> None:
         e.id = i
 
 
+def to_portable(
+    dynamic_equations: Iterable[Equation],
+    steady_equations: Iterable[Equation],
+) -> tuple[_PortableType]:
+    """
+    """
+    #[
+    portable = []
+    for kind in _TO_PORTABLES.keys():
+        portable += [
+            d.to_portable(s, )
+            for d, s in zip(dynamic_equations, steady_equations, )
+            if d.kind == kind
+        ]
+    return tuple(portable)
+    #]
+
+
+def from_portable(portable: tuple[_PortableType], /) -> tuple[Equation]:
+    """
+    """
+    dynamic_steady = ( Equation.from_portable(p, ) for p in portable )
+    return tuple(zip(*dynamic_steady))
+
+
 def create_human_to_description(equations: Iterable[Equation]) -> dict[str, str]:
     return { eqn.human: eqn.description for eqn in equations }
 
