@@ -7,11 +7,14 @@ Plain equator
 from __future__ import annotations
 
 import numpy as _np
-from typing import (Protocol, Callable, Self, )
-from collections.abc import (Iterable, )
 
 from .. import equations as _equations
 from .. import makers as _makers
+
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from typing import Any, Callable
+    from collections.abc import Iterable
 #]
 
 
@@ -23,15 +26,21 @@ class PlainEquator:
     """
     #[
 
-    __slots__ = (
+    _state_slots = (
+        "_context",
         "_equations",
-        "_func",
         "_func_str",
         "_columns",
         "min_shift",
         "max_shift",
     )
 
+    _nonstate_slots = (
+        "_func",
+    )
+
+    __slots__ = _state_slots + _nonstate_slots
+
     def __init__(
         self,
         equations: Iterable[_equations.Equation],
@@ -41,9 +50,10 @@ class PlainEquator:
         """
         """
         self._equations = tuple(equations)
-        self._create_function(context, )
         self._populate_min_max_shifts()
         self._populate_columns(columns, )
+        self._context = context
+        self._create_function()
 
     @property
     def humans(self, /, ) -> tuple[str]:
@@ -55,17 +65,13 @@ class PlainEquator:
         """
         return len(self._equations)
 
-    def _create_function(
-        self,
-        context: dict | None = None,
-        /,
-    ) -> None:
+    def _create_function(self, /, ) -> None:
         """
         """
         joined_xtrings = "  ,  ".join(i.xtring for i in self._equations)
         expression = "(" + joined_xtrings + " , )"
-        self._func, self._func_str, *_ = \
-            _makers.make_lambda(EQUATOR_ARGS, expression, context, )
+        self._func, self._func_str, *_ \
+            = _makers.make_function("__equator", EQUATOR_ARGS, expression, self._context, )
 
     def _populate_min_max_shifts(self, /, ) -> None:
         """
@@ -108,5 +114,20 @@ class PlainEquator:
         value = self.eval(*args, **kwargs, )
         return _np.array(value, dtype=_np.float64, ).reshape(self.num_equations, -1, )
 
+    def __getstate__(self, ) -> dict[str, Any]:
+        """
+        """
+        return {
+            k: getattr(self, k)
+            for k in self._state_slots
+        }
+
+    def __setstate__(self, state: dict[str, Any], ) -> None:
+        """
+        """
+        for n in self._state_slots:
+            setattr(self, n, state[n], )
+        self._create_function()
+
     #]
 
