@@ -57,6 +57,7 @@ import numpy as _np
 from ..incidences.main import (Token, )
 from ..incidences import main as _incidence
 from .. import equations as _equations
+from ..equations import Equation, EquationKind
 from .. import quantities as _quantities
 from ..aldi.differentiators import (AtomFactoryProtocol, Context, )
 from ..aldi import maps as _maps
@@ -66,27 +67,34 @@ from .. import sources as _sources
 
 # Implement AtomFactoryProtocol
 
-def _create_diff_for_token(
-    token: Token,
-    wrt_tokens: tuple[Token, ...],
-    /,
-) -> _np.ndarray:
+
+class _AtomFactory:
     """
     """
-    try:
-        index = wrt_tokens.index(token, )
-    except ValueError:
-        return 0
-    diff = _np.zeros((len(wrt_tokens), 1, ), dtype=float, )
-    diff[index] = 1
-    return diff
+    #[
 
+    @staticmethod
+    def create_data_index_for_token(token: Token, ) -> tuple[int, int]:
+        return token.qid, token.shift,
 
-_ATOM_FACTORY = SimpleNamespace(
-    create_data_index_for_token=lambda token: (token.qid, token.shift, ),
-    create_diff_for_token=_create_diff_for_token,
-    get_diff_shape=lambda wrt_tokens: (len(wrt_tokens), 1, ),
-)
+    @staticmethod
+    def create_diff_for_token(
+        token: Token,
+        wrt_tokens: tuple[Token, ...],
+    ) -> _np.ndarray:
+        try:
+            index = wrt_tokens.index(token, )
+        except ValueError:
+            return 0
+        diff = _np.zeros((len(wrt_tokens), 1, ), dtype=float, )
+        diff[index] = 1
+        return diff
+
+    @staticmethod
+    def get_diff_shape(wrt_tokens: tuple[Token, ...]) -> tuple[int, int]:
+        return len(wrt_tokens), 1,
+
+    #]
 
 
 class Descriptor:
@@ -111,15 +119,18 @@ class Descriptor:
 
     def __init__(
         self,
-        equations: Iterable[_equations.Equation],
+        equations: Iterable[Equation],
         quantities: Iterable[_quantities.Quantity],
         context: dict[str, Any] | None,
-        /,
     ) -> None:
         """
         """
+        equations = tuple(
+            i for i in equations
+            if i.kind == EquationKind.TRANSITION_EQUATION or i.kind == EquationKind.MEASUREMENT_EQUATION
+        )
         self.system_vectors = SystemVectors(equations, quantities)
-        self.solution_vectors = SolutionVectors(self.system_vectors)
+        self.solution_vectors = SolutionVectors.from_system_vectors(self.system_vectors)
         self.system_map = SystemMap(self.system_vectors)
         system_equations = _custom_order_equations_by_eids(
             equations,
@@ -129,7 +140,7 @@ class Descriptor:
         #
         # Create the evaluation context for the algorithmic differentiator
         self.aldi_context = Context(
-            _ATOM_FACTORY,
+            _AtomFactory,
             system_equations,
             eid_to_wrts=self.system_vectors.eid_to_wrt_tokens,
             qid_to_logly=_quantities.create_qid_to_logly(quantities),
@@ -192,6 +203,8 @@ class SystemVectors:
     measurement_variables: Iterable[Token] | None = None
     measurement_variables_are_logly: list[bool] | None = None
     measurement_shocks: tuple[Token, ...] | None = None
+    std_unanticipated_shocks: tuple[Token, ...] | None = None
+    std_measurement_shocks: tuple[Token, ...] | None = None
     #
     shape_A_excl_dynid: tuple[int, int] | None = None
     shape_B_excl_dynid: tuple[int, int] | None = None
@@ -205,9 +218,8 @@ class SystemVectors:
 
     def __init__(
         self,
-        equations: Iterable[_equations.Equation],
+        equations: Iterable[Equation],
         quantities: Iterable[_quantities.Quantity],
-        /,
     ) -> None:
         """
         Construct system vectors from a list of equations and a list of quantities
@@ -215,9 +227,9 @@ class SystemVectors:
         qid_to_logly = _quantities.create_qid_to_logly(quantities, )
         #
         self.transition_eids \
-            = sorted([eqn.id for eqn in equations if eqn.kind in _equations.EquationKind.TRANSITION_EQUATION])
+            = sorted([eqn.id for eqn in equations if eqn.kind in EquationKind.TRANSITION_EQUATION])
         self.measurement_eids \
-            = sorted([eqn.id for eqn in equations if eqn.kind in _equations.EquationKind.MEASUREMENT_EQUATION])
+            = sorted([eqn.id for eqn in equations if eqn.kind in EquationKind.MEASUREMENT_EQUATION])
         qid_to_kind = _quantities.create_qid_to_kind(quantities, )
         #
         # Collect all tokens from equations but also add zero-shifted tokens
@@ -245,7 +257,7 @@ class SystemVectors:
             = [ qid_to_logly[tok.qid] for tok in self.transition_variables ]
         self.populate_true_initials(actual_transition_variable_tokens, )
         #
-        # Unanticipated shocks
+        # Unanticipated shocks and stds
         #
         self.unanticipated_shocks \
             = tuple(_incidence.sort_tokens(_incidence.generate_tokens_of_kinds(
@@ -268,7 +280,7 @@ class SystemVectors:
         self.measurement_variables_are_logly \
             = [ qid_to_logly[tok.qid] for tok in self.measurement_variables ]
         #
-        # Measurement shocks
+        # Measurement shocks and stds
         #
         self.measurement_shocks \
             = tuple(_incidence.sort_tokens(_incidence.generate_tokens_of_kinds(
@@ -307,48 +319,56 @@ class SystemVectors:
             min_shift = actual_min_shifts[token.qid]
             return min_shift <= shift < 0
         self.true_initials = tuple(
-            is_true_initial(t) for t in self.transition_variables
+            is_true_initial(i) for i in self.transition_variables
         )
     #]
 
 
-@_dc.dataclass(slots=True, )
 class SolutionVectors:
     """
     Vectors of quantities in first-order solution matrices
     """
     #[
-    transition_variables: tuple[Token, ...] | None = None
-    true_initials: list[bool, ...] | None = None
-    unanticipated_shocks: tuple[Token, ...] | None = None
-    anticipated_shocks: tuple[Token, ...] | None = None
-    measurement_variables: tuple[Token, ...] | None = None
-    measurement_shocks: tuple[Token, ...] | None = None
 
-    def __init__(self, system_vectors: SystemVectors, /, ) -> None:
+    __slots__ = (
+        "transition_variables",
+        "unanticipated_shocks",
+        "anticipated_shocks",
+        "measurement_variables",
+        "measurement_shocks",
+        "true_initials",
+    )
+
+    def __init__(self, **kwargs, ) -> None:
+        """
+        """
+        for slot in self.__slots__:
+            setattr(self, slot, kwargs.get(slot, None, ), )
+
+    @classmethod
+    def from_system_vectors(
+        klass,
+        system_vectors: SystemVectors,
+    ) -> None:
         """
         Construct solution vectors and initial conditions indicator
         """
+        self = klass()
         self.transition_variables, self.true_initials = \
-            _solution_vector_from_system_vector(system_vectors.transition_variables, system_vectors.true_initials)
+            _solution_vector_from_system_vector(system_vectors.transition_variables, system_vectors.true_initials, )
         self.unanticipated_shocks = tuple(system_vectors.unanticipated_shocks)
         self.anticipated_shocks = tuple(system_vectors.anticipated_shocks)
         self.measurement_variables = tuple(system_vectors.measurement_variables)
         self.measurement_shocks = tuple(system_vectors.measurement_shocks)
+        return self
 
-    def get_initials(
-        self,
-        /,
-    ) -> list[Token]:
+    def get_initials(self, ) -> list[Token]:
         """
         Get tokens representing required initial conditions
         """
         return list(_it.compress(self.transition_variables, self.true_initials))
 
-    def get_curr_transition_indexes(
-        self,
-        /,
-    ) -> tuple[tuple[int, ...], tuple[int, ...]]:
+    def get_curr_transition_indexes(self, ) -> tuple[tuple[int, ...], tuple[int, ...]]:
         """
         """
         lhs_rhs_tuples = [
@@ -378,7 +398,6 @@ class HumanSolutionVectors:
         solution_vectors: SolutionVectors,
         qid_to_name: dict[int, str],
         qid_to_logly: dict[int, bool],
-        /,
     ) -> None:
         """
         """
@@ -391,7 +410,6 @@ class HumanSolutionVectors:
 
 def _create_system_transition_vector(
     transition_variable_tokens: Iterable[Token],
-    /,
 ) -> Iterable[Token]:
     """
     From tokens of transition variables, create vector of transition variables
@@ -402,16 +420,16 @@ def _create_system_transition_vector(
     min_shifts = _incidence.get_some_shift_by_quantities(transition_variable_tokens, lambda x: min(min(x), -1))
     max_shifts = _incidence.get_some_shift_by_quantities(transition_variable_tokens, max)
     #
-    vector_for_qid = lambda qid: [Token(qid, sh) for sh in range(min_shifts[qid]+1, max_shifts[qid]+1)]
+    def list_for_qid(qid: int, ):
+        return [Token(qid, sh) for sh in range(min_shifts[qid]+1, max_shifts[qid]+1)]
     unique_qids = set(t.qid for t in transition_variable_tokens)
-    return _it.chain.from_iterable(vector_for_qid(i) for i in unique_qids)
+    return _it.chain.from_iterable(list_for_qid(i) for i in unique_qids)
     #]
 
 
 def _solution_vector_from_system_vector(
     system_transition_vector: Iterable[Token],
     true_initials: Iterable[bool],
-    /,
 ) -> Iterable[Token]:
     """
     From sorted system vector, get vector of transition variables in solved
@@ -595,14 +613,13 @@ def _create_dynid_matrices(system_transition_vector: Iterable[Token]):
 
 def _adjust_for_measurement_equations(
     transition_variable_tokens: Iterable[_quantities.Quantity],
-    equations: Iterable[_equations.Equation],
+    equations: Iterable[Equation],
     qid_to_kind: dict[int, _quantities.QuantityKind],
-    /,
 ) -> Iterable[_quantities.Quantity]:
     """
     """
     #[
-    tokens_in_measurement_equations = _it.chain.from_iterable(e.incidence for e in equations if e.kind is _equations.EquationKind.MEASUREMENT_EQUATION)
+    tokens_in_measurement_equations = _it.chain.from_iterable(e.incidence for e in equations if e.kind is EquationKind.MEASUREMENT_EQUATION)
     pretend_needed = [
         Token(t.qid, t.shift-1) for t in tokens_in_measurement_equations
         if qid_to_kind[t.qid] in _quantities.QuantityKind.TRANSITION_VARIABLE
@@ -612,10 +629,9 @@ def _adjust_for_measurement_equations(
 
 
 def _custom_order_equations_by_eids(
-    equations: Iterable[_equations.Equation],
+    equations: Iterable[Equation],
     eids: list[int],
-    /,
-) -> tuple[_equations.Equation, ...]:
+) -> tuple[Equation, ...]:
     """
     """
     #[
@@ -655,13 +671,12 @@ class Squid:
 
     def __init__(
         self,
-        model: SquidableProtocol,
-        /,
+        solution_vectors: SolutionVectors,
+        shock_qid_to_std_qid: dict[int, int] | None = None,
     ) -> None:
         """
         """
-        vec = model.solution_vectors
-        shock_qid_to_std_qid = model.shock_qid_to_std_qid
+        vec = solution_vectors
         #
         self.num_xi = len(vec.transition_variables)
         self.curr_xi_qids, self.curr_xi_indexes = vec.get_curr_transition_indexes()
@@ -677,7 +692,7 @@ class Squid:
         self.std_v_qids = None
         self.std_w_qids = None
         #
-        if not model.is_deterministic:
+        if shock_qid_to_std_qid:
             self.std_u_qids = tuple(
                 shock_qid_to_std_qid[t.qid]
                 for t in vec.unanticipated_shocks
@@ -691,6 +706,17 @@ class Squid:
                 for t in vec.measurement_shocks
             )
 
+    @classmethod
+    def from_squidable(
+        klass,
+        squidable: SquidableProtocol,
+    ) -> Self:
+        """
+        """
+        solution_vectors = squidable.solution_vectors
+        shock_qid_to_std_qid = squidable.shock_qid_to_std_qid if squidable.shock_qid_to_std_qid else None
+        return klass(solution_vectors, shock_qid_to_std_qid, )
+
     @property
     def num_curr_xi(self) -> int:
         """==Number of current-dated transition variables in transition vector=="""
@@ -715,9 +741,8 @@ class Squid:
 
 
 def _collect_all_tokens(
-    equations: Iterable[_equations.Equation],
+    equations: Iterable[Equation],
     quantities: Iterable[_quantities.Quantity],
-    /,
 ) -> set[Token]:
     """
     Combine all tokens from equations and zero-shifted tokens created from
