@@ -3,30 +3,36 @@
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (Any, Iterable, Sequence, Literal, Self, Protocol, NoReturn, )
-from types import (EllipsisType, )
-from numbers import (Real, )
-from collections import (namedtuple, )
+from typing import Protocol
+from typing import Literal
 import functools as _ft
-import dataclasses as _dc
 import numpy as _np
-import scipy as _sp
-import wlogging as _wl
+import documark as _dm
 
 from .. import has_variants as _has_variants
 from .. import quantities as _quantities
-from .. import pages as _pages
-from ..frames import (Frame, )
-from ..dataslates.main import (Dataslate, )
-from ..databoxes.main import (Databox, )
-from .solutions import (Solution, right_div, left_div, )
-from ..dates import (Dater, )
+from ..frames import Frame
+from ..dataslates.main import Dataslate
+from ..databoxes.main import Databox
+from ..series.main import Series
+from .solutions import Solution, right_div, left_div
 
 from . import initializers as _initializers
 from . import shock_simulators as _shock_simulators
-from .descriptors import (Squid, )
+from . import covariances as _covariances
+from .descriptors import Squid
+from .. import wrongdoings as _wd
+
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from typing import Any, Iterable, Sequence, Self, NoReturn
+    from types import EllipsisType
+    from numbers import Real
+    from ..dates import Period
+
 #]
 
 
@@ -35,7 +41,13 @@ _simulate_anticipated_shocks \
 
 
 _DEFAULT_DELTA_TOLERANCE = 1e-12
-_LOGGER = _wl.get_colored_logger(__name__, level=_wl.INFO, )
+
+
+DiffuseMethodType = Literal[
+    "approx_diffuse",
+    "fixed_unknown",
+    "fixed_zero",
+]
 
 
 class SingularMatrixError(ValueError, ):
@@ -49,9 +61,7 @@ class KalmanFilterableProtocol(Protocol, ):
 
     is_singleton: bool
 
-    def get_variant(self, vid: int, /) -> Self: ...
-
-    def get_singleton_solution(self, deviation: bool = False, ) -> Solution: ...
+    def _get_singleton_solution(self, deviation: bool = False, ) -> Solution: ...
 
     def get_cov_unanticipated_shocks(self, ) -> _np.ndarray: ...
 
@@ -116,19 +126,22 @@ class Needs:
         "return_update",
         "return_smooth",
         "return_predict_err",
-        "predict_mse_obs",
+        "return_predict_mse_obs",
         "rescale_variance",
+        "likelihood_contributions",
+        "output_store",
     )
 
     def __init__(
         self: Self,
-        return_: Iterable[str, ...],
-        return_predict: bool = True,
-        return_update: bool = True,
-        return_smooth: bool = True,
-        return_predict_err: bool = True,
-        predict_mse_obs: bool = True,
+        return_: Iterable[str, ...] = (),
+        return_predict: bool = False,
+        return_update: bool = False,
+        return_smooth: bool = False,
+        return_predict_err: bool = False,
+        return_predict_mse_obs: bool = False,
         rescale_variance: bool = False,
+        likelihood_contributions: bool = False,
     ) -> None:
         """
         """
@@ -137,8 +150,15 @@ class Needs:
         self.return_update = return_update and "update" in return_
         self.return_smooth = return_smooth and "smooth" in return_
         self.return_predict_err = return_predict_err and "predict_err" in return_
-        self.predict_mse_obs = predict_mse_obs and "predict_mse_obs" in return_
+        self.return_predict_mse_obs = return_predict_mse_obs and "predict_mse_obs" in return_
         self.rescale_variance = rescale_variance
+        self.likelihood_contributions = likelihood_contributions
+        #
+        self.output_store = any((
+            self.return_predict,
+            self.return_update,
+            self.return_smooth,
+        ))
 
     #]
 
@@ -166,7 +186,7 @@ class _LogDataslate:
     def from_names_dates(
         klass,
         measurement_names: Iterable[str],
-        dates: Iterable[Dater],
+        dates: Iterable[Period],
         num_variants: int,
         name_to_log_name: dict[str, str],
     ) -> Self:
@@ -350,7 +370,7 @@ class _OutputStore:
         self.predict_std = _std_constructor() if needs.return_predict else None
         self.predict_mse_obs = [
             [None]*input_ds.num_periods for _ in range(num_variants, )
-        ] if needs.return_predict else None
+        ] if needs.return_predict and needs.return_predict_mse_obs else None
         #
         # Updating step
         #
@@ -376,7 +396,7 @@ class _OutputStore:
                 getattr(self, s).extend(getattr(other, s), )
 
     def rescale_stds(self, var_scale, /, ) -> None:
-        """
+        r"""
         """
         if var_scale == 1 or var_scale is None:
             return
@@ -389,18 +409,17 @@ class _OutputStore:
                 v.rescale_data(std_scale)
 
     def create_out_data(self: Self, /, ) -> Databox:
-        """
+        r"""
         """
         out_data = Databox()
         for name in self._OUT_NAMES:
             attr = getattr(self, name, )
             if attr is None:
                 continue
-            out_data[name] = (
-                attr.to_output_arg()
-                if hasattr(attr, "to_output_arg", )
-                else attr
-            )
+            if hasattr(attr, "to_output_arg", ):
+                out_data[name] = attr.to_output_arg()
+            else:
+                out_data[name] = attr
         return out_data
 
     def store_predict(
@@ -417,7 +436,7 @@ class _OutputStore:
         cov_u0: _np.ndarray | None = None,
         cov_w0: _np.ndarray | None = None,
     ) -> None:
-        """
+        r"""
         """
         if self.predict_med is not None:
             curr_xi_qids = self.squid.curr_xi_qids
@@ -542,36 +561,49 @@ class Mixin:
     """
     #[
 
-    @_pages.reference(category="filtering", )
+
+    @_dm.reference(category="filtering", )
+    def neg_log_likelihood(self, *args, **kwargs, ) -> Real:
+        """
+        """
+        kwargs["return_"] = ()
+        kwargs["likelihood_contributions"] = False
+        kwargs["return_info"] = True
+        _, info = self.kalman_filter(*args, **kwargs, )
+        return info["neg_log_likelihood"]
+
+    @_dm.reference(category="filtering", )
     def kalman_filter(
         self,
         #
         input_db: Databox,
-        span: Iterable[Dater],
+        span: Iterable[Period],
         #
         diffuse_scale: Real | None = None,
-        diffuse_method: Literal["approx_diffuse", "fixed_unknown", "fixed_zero", ] = "fixed_unknown",
+        diffuse_method: DiffuseMethodType = "fixed_unknown",
         #
         return_: Iterable[str, ...] = ("predict", "update", "smooth", "predict_err", "predict_mse_obs", ),
         return_predict: bool = True,
         return_update: bool = True,
         return_smooth: bool = True,
         return_predict_err: bool = True,
-        predict_mse_obs: bool = True,
+        return_predict_mse_obs: bool = True,
         rescale_variance: bool = False,
+        likelihood_contributions: bool = True,
         #
         shocks_from_data: bool = False,
         stds_from_data: bool = False,
+        initials_from_data: bool = False,
         #
         prepend_initial: bool = False,
         append_terminal: bool = False,
         deviation: bool = False,
         check_singularity: bool = False,
+        when_singularity: Literal["critical", "error", "warning", "silent" ] | None = "critical",
         num_variants: int | None = None,
         #
         unpack_singleton: bool = True,
         return_info: bool = False,
-        logging_level: int = _wl.INFO,
     ) -> Databox | tuple[Databox, _Info]:
         r"""
 ················································································
@@ -592,8 +624,9 @@ the time series data.
         return_update=True,
         return_smooth=True,
         return_predict_err=True,
-        predict_mse_obs=True,
+        return_predict_mse_obs=True,
         rescale_variance=False,
+        likelihood_contributions=True,
         shocks_from_data=False,
         stds_from_data=False,
         prepend_initial=False,
@@ -645,6 +678,10 @@ the time series data.
     If `True`, rescale all variances by the optimal variance scale factor
     estimated using maximum likelihood after the filtering process.
 
+???+ input "likelihood_contributions"
+    If `True`, return the contributions of individual periods to the overall
+    (negative) log likelihood.
+
 ???+ input "shocks_from_data"
     If `True`, use possibly time-varying shock values from the data; these
     values are interpreted as the medians (means) of the shocks. If `False`,
@@ -720,13 +757,9 @@ the time series data.
         #    std_names = self.get_names(kind=_quantities.ANY_STD, )
         #    work_db.remove(std_names, strict_names=False, )
 
-        _LOGGER.set_level(logging_level, )
+        num_variants = self.resolve_num_variants_in_context(num_variants, )
 
-        num_variants \
-            = self.resolve_num_variants_in_context(num_variants, )
-        _LOGGER.debug(f"Running {num_variants} variants")
-
-        slatable = self.get_slatable_for_kalman_filter(
+        slatable = self.slatable_for_kalman_filter(
             shocks_from_data=shocks_from_data,
             stds_from_data=stds_from_data,
         )
@@ -751,11 +784,12 @@ the time series data.
             return_update=return_update,
             return_smooth=return_smooth,
             return_predict_err=return_predict_err,
-            predict_mse_obs=predict_mse_obs,
+            return_predict_mse_obs=return_predict_mse_obs,
             rescale_variance=rescale_variance,
+            likelihood_contributions=likelihood_contributions,
         )
 
-        squid = Squid(self, )
+        squid = Squid.from_squidable(self, )
         qid_to_logly = self.create_qid_to_logly()
 
         logly_within_y = tuple(
@@ -777,7 +811,7 @@ the time series data.
             measurement_names=y_names,
             needs=needs,
             name_to_log_name=name_to_log_name,
-        )
+        ) if needs.output_store else None
 
         initialize = _ft.partial(
             _initializers.initialize,
@@ -785,7 +819,10 @@ the time series data.
             diffuse_scale=diffuse_scale,
         )
 
-        zipped = zip(
+        #
+        # Include the variants even though they are not used because otherwise
+        # the number of loops is undetermined (infite iterators)
+        main_iter = zip(
             range(num_variants, ),
             self.iter_variants(),
             input_ds.iter_variants(),
@@ -793,22 +830,31 @@ the time series data.
 
         out_info = []
 
-        for vid, self_v, input_ds_v in zipped:
+        for vid, self_v, input_ds_v in main_iter:
 
-            solution_v = self_v.get_singleton_solution(deviation=deviation, )
+            variant_header = f"[Variant {vid}]"
+            when_singularity = _wd.create_stream(
+                when_singularity,
+                f"Singularity in prediction MSE matrix in {variant_header}",
+            ) if check_singularity else None
+
+            solution_v = self_v._get_singleton_solution(deviation=deviation, )
             data_array = input_ds_v.get_data_variant()
+
             #
-            # Initialize medians and stds
+            # Initialize median and MSE of the alpha state vector
             #
             init_cov_u = self_v.get_cov_unanticipated_shocks()
-            init_med, init_mse, unknown_init_impact \
-                = initialize(solution_v, init_cov_u, )
+            init_med, init_mse, unknown_init_impact = initialize(solution_v, init_cov_u, )
+
             #
             # Presimulate the impact of anticipated shocks
             #
-            all_v_impact \
-                = _simulate_anticipated_shocks \
-                (self_v, input_ds_v, frame, )
+            all_v_impact = (
+                _simulate_anticipated_shocks(self_v, input_ds_v, frame, )
+                if shocks_from_data else None
+            )
+
             #
             # Get values of observables
             #
@@ -827,7 +873,7 @@ the time series data.
                 transform=solution_v.Ua,
                 squid=squid,
                 measurement_incidence=y1_incidence_array,
-            )
+            ) if needs.output_store else None
 
             generate_period_system = _ft.partial(
                 _generate_period_system,
@@ -847,9 +893,20 @@ the time series data.
                 w_array=data_array[squid.w_qids, :],
             )
 
-            store_predict = _ft.partial(_OutputStore.store_predict, self=output_store_v, )
-            store_update = _ft.partial(_OutputStore.store_update, self=output_store_v, )
-            store_smooth = _ft.partial(_OutputStore.store_smooth, self=output_store_v, )
+            store_predict = (
+                _ft.partial(_OutputStore.store_predict, self=output_store_v, )
+                if needs.return_predict else None
+            )
+
+            store_update = (
+                _ft.partial(_OutputStore.store_update, self=output_store_v, )
+                if needs.return_update else None
+            )
+
+            store_smooth = (
+                _ft.partial(_OutputStore.store_smooth, self=output_store_v, )
+                if needs.return_smooth else None
+            )
 
             cache = predict(
                 num_periods=input_ds_v.num_periods,
@@ -859,8 +916,12 @@ the time series data.
                 store_update=store_update,
                 store_smooth=store_smooth,
                 check_singularity=check_singularity,
+                when_singularity=when_singularity,
             )
 
+            if when_singularity is not None:
+                when_singularity._raise()
+
             if cache.needs_estimate_unknown_init:
                 estimate_unknown_init(
                     cache=cache,
@@ -871,34 +932,38 @@ the time series data.
                 )
 
             if needs.return_update:
-                update(
-                    cache=cache,
-                    store_update=store_update,
-                )
+                update(cache=cache, store_update=store_update, )
 
             if needs.return_smooth:
-                smooth(
-                    cache=cache,
-                    store_smooth=store_smooth,
-                )
+                smooth(cache=cache, store_smooth=store_smooth, )
 
             cache.calculate_likelihood(rescale_variance=needs.rescale_variance, )
-            output_store_v.rescale_stds(cache.var_scale, )
-            out_info_v = cache.create_out_info()
+            if needs.likelihood_contributions:
+                cache.calculate_likelihood_contributions()
+
+            if needs.output_store:
+                output_store_v.rescale_stds(cache.var_scale, )
+
+            out_info_v = cache.create_out_info(span, )
+
+            if needs.output_store:
+                output_store.extend(output_store_v, )
 
-            output_store.extend(output_store_v, )
             out_info.append(out_info_v, )
 
-        out_data = output_store.create_out_data()
+        out_data = (
+            output_store.create_out_data()
+            if output_store is not None else None
+        )
         #
         if return_info:
             out_info = _has_variants.unpack_singleton(
                 out_info, self.is_singleton,
                 unpack_singleton=True,
             )
-            return out_data, out_info
-        #
-        return out_data
+            return out_data, out_info,
+        else:
+            return out_data
 
     #]
 
@@ -908,7 +973,7 @@ class Cache:
     """
     #[
 
-    LOG_2_PI = _np.log(2 * _np.pi)
+    _LOG_2_PI = _np.log(2 * _np.pi)
 
     _slots_to_input = (
         "num_periods",
@@ -945,6 +1010,8 @@ class Cache:
         "var_scale",
         "neg_log_likelihood",
         "neg_log_likelihood_contributions",
+        "all_log_det_F",
+        "all_pe_Fi_pe",
     )
 
     # __slots__ = _slots_to_input + _slots_to_preallocate + _other_slots
@@ -965,36 +1032,43 @@ class Cache:
     def calculate_likelihood(self: Self, rescale_variance: bool, ) -> None:
         """
         """
-        all_pe_Fi_pe = tuple(
+        self.all_pe_Fi_pe = tuple(
             pe @ Fi @ pe if pe is not None and Fi is not None else None
-            for pe, Fi in zip(self.all_pe, self.all_Fi, )
+            for pe, Fi, in zip(self.all_pe, self.all_Fi, )
         )
-        all_log_det_F = tuple(
-            -_np.log(_np.linalg.det(Fi)) if Fi is not None else None
+        all_det_Fi = tuple(
+            float(_np.linalg.det(Fi)) if Fi is not None else None
             for Fi in self.all_Fi
         )
+        self.all_log_det_F = tuple(
+            float(-_np.log(det_Fi)) if det_Fi is not None else None
+            for det_Fi in all_det_Fi
+        )
         self.sum_num_obs = sum(i for i in self.all_num_obs if i is not None)
-        self.sum_log_det_F = sum(i for i in all_log_det_F if i is not None)
-        self.sum_pe_Fi_pe = sum(i for i in all_pe_Fi_pe if i is not None)
+        self.sum_log_det_F = sum(i for i in self.all_log_det_F if i is not None)
+        self.sum_pe_Fi_pe = sum(i for i in self.all_pe_Fi_pe if i is not None)
         self.var_scale = 1
         #
         if rescale_variance:
             self._calculate_variance_scale()
         #
-        self.neg_log_likelihood_contributions = [
-            (log_det_F + pe_Fi_pe + num_obs * self.LOG_2_PI)/2 if num_obs else 0
-            for log_det_F, pe_Fi_pe, num_obs in zip(
-                all_log_det_F, all_pe_Fi_pe, self.all_num_obs,
-            )
-            if num_obs is not None
-        ]
-        #
         self.neg_log_likelihood = (
-            + self.sum_num_obs*self.LOG_2_PI
+            + self.sum_num_obs*self._LOG_2_PI
             + self.sum_log_det_F
             + self.sum_pe_Fi_pe
         ) / 2;
 
+    def calculate_likelihood_contributions(self: Self, ) -> None:
+        """
+        """
+        self.neg_log_likelihood_contributions = tuple(
+            (log_det_F + pe_Fi_pe + num_obs * self._LOG_2_PI)/2 if num_obs else 0
+            for log_det_F, pe_Fi_pe, num_obs in zip(
+                self.all_log_det_F, self.all_pe_Fi_pe, self.all_num_obs,
+            )
+            # if num_obs is not None
+        )
+
     def _calculate_variance_scale(self: Self, /, ) -> None:
         """
         """
@@ -1006,14 +1080,21 @@ class Cache:
         self.sum_log_det_F += self.sum_num_obs * _np.log(self.var_scale)
         self.sum_pe_Fi_pe = self.sum_pe_Fi_pe / self.var_scale
 
-    def create_out_info(self: Self, /, ) -> dict[str, Any]:
+    def create_out_info(self: Self, span: Iterable[Period], ) -> dict[str, Any]:
         """
         """
-        return {
-            "neg_log_likelihood": self.neg_log_likelihood,
-            "neg_log_likelihood_contributions": self.neg_log_likelihood_contributions,
-            "var_scale": self.var_scale,
+        out_info = {
+            "neg_log_likelihood": float(self.neg_log_likelihood),
+            "log_det_F": Series(periods=span, values=self.all_log_det_F, ),
+            "var_scale": float(self.var_scale),
+            "std_scale": float(_sqrt_positive(self.var_scale)),
         }
+        if self.neg_log_likelihood_contributions is not None:
+            out_info["neg_log_likelihood_contributions"] = Series(
+                periods=span,
+                values=self.neg_log_likelihood_contributions,
+            )
+        return out_info
 
     #]
 
@@ -1026,6 +1107,7 @@ def predict(
     store_update: Callable | None = None,
     store_smooth: Callable | None = None,
     check_singularity: bool = False,
+    when_singularity: _wd.Stream | None = None,
 ) -> None:
     """
     """
@@ -1035,11 +1117,12 @@ def predict(
     needs_estimate_unknown_init = Xi_prev is not None
     cache.needs_estimate_unknown_init = needs_estimate_unknown_init
     cache.last_period_of_observations = -1
+    create_empty = _ft.partial(_np.empty, shape=(0, 0), dtype=_np.float64, )
     #
     for t in range(num_periods, ):
         #
-        T, P, K, Z, H, D, cov_u, cov_w, v_impact, *_ = generate_period_system(t, )
-        y1, u0, v0, w0, *_ = generate_period_data(t, )
+        T, P, K, Z, H, D, cov_u, cov_w, v_impact, U, *_ = generate_period_system(t, )
+        y1, u0, v0, w0, inx_y, *_ = generate_period_data(t, )
         #
         cache.all_y[t] = y1
         cache.all_num_obs[t] = y1.size
@@ -1056,21 +1139,25 @@ def predict(
         #
         P_cov_u = P @ cov_u
         Q0 = T @ Q1_prev @ T.T + P_cov_u @ P.T
-        Q0 = _make_symmetric(Q0)
+        Q0 = _covariances.symmetrize(Q0)
         H_cov_w = H @ cov_w
         #
-        F = Z @ Q0 @ Z.T + H_cov_w @ H.T \
-            if any_y else _np.zeros((0, 0), dtype=_np.float64, )
-        F = _make_symmetric(F)
+        F = (
+            Z @ Q0 @ Z.T + H_cov_w @ H.T if any_y
+            else create_empty()
+        )
+        F = _covariances.symmetrize(F, )
         #
         # sing_values = _np.linalg.svd(F, compute_uv=False, )
         # relative_cond = sing_values[-1] / sing_values[0]
         # print(t, F.size, relative_cond, sing_values[0], )
         # TODO: Check if F singularity if requested by user
-        if check_singularity and any_y:
-            _check_singularity(F, t, )
+        inv = _INVERSE_FUNCTION["regular"]
+        if when_singularity and any_y:
+            inv = _check_singularity(F, t, inx_y, when_singularity, )
 
-        Fi = _np.linalg.inv(F) if any_y else _np.zeros((0, 0), )
+        Fi = inv(F) if any_y else create_empty()
+        Fi = _covariances.symmetrize(Fi, )
         #
         # Median prediction step
         #
@@ -1085,7 +1172,7 @@ def predict(
         Zt_Fi = Z.T @ Fi
         G = Q0 @ Zt_Fi
         Q1 = Q0 - G @ Z @ Q0
-        Q1 = _make_symmetric(Q1)
+        Q1 = _covariances.symmetrize(Q1)
         if store_update:
             store_update(t=t, Q=Q1, )
         #
@@ -1093,6 +1180,7 @@ def predict(
         #
         pe = y1 - y0
         a1 = a0 + G @ pe
+        x1 = U @ a1
         #
         cache.all_a0[t] = a0
         cache.all_y0[t] = y0
@@ -1148,7 +1236,7 @@ def estimate_unknown_init(
         for Mt_Fi, pe in zip(all_Mt_Fi, cache.all_pe, )
     )
     #
-    sum_Mt_Fi_M = _make_symmetric(sum_Mt_Fi_M)
+    sum_Mt_Fi_M = _covariances.symmetrize(sum_Mt_Fi_M)
     sum_Mt_Fi_M[_np.abs(sum_Mt_Fi_M) < delta_tolerance] = 0
     #
     delta, *_ = _np.linalg.lstsq(sum_Mt_Fi_M, sum_Mt_Fi_pe, rcond=None, )
@@ -1246,7 +1334,7 @@ def one_step_back(
         else:
             N = Zt_Fi_Z + L.T @ N @ L
         Qk = Qk - Q0 @ N @ Q0
-        Qk = _make_symmetric(Qk)
+        Qk = _covariances.symmetrize(Qk)
         #
         if r is None:
             wk = wk + H_cov_w.T @ Fi_pe
@@ -1264,12 +1352,12 @@ def one_step_back(
 
 def _generate_period_system(
     t: int,
-
+    #
     solution_v: Solution,
     y1_array: _np.ndarray,
     std_u_array: _np.ndarray,
     std_w_array: _np.ndarray,
-    all_v_impact: Sequence[_np.ndarray | None],
+    all_v_impact: Sequence[_np.ndarray | None] | None,
     initials: tuple[_np.ndarray, _np.ndarray, int, ],
 ) -> tuple[_np.ndarray, ...]:
     """
@@ -1285,10 +1373,11 @@ def _generate_period_system(
     Z = solution_v.Za[inx_y, :]
     H = solution_v.H[inx_y, :]
     D = solution_v.D[inx_y]
+    U = solution_v.Ua
     cov_u = _np.diag(std_u_array[:, t]**2, )
     cov_w = _np.diag(std_w_array[:, t]**2, )
-    v_impact = all_v_impact[t]
-    return T, P, K, Z, H, D, cov_u, cov_w, v_impact,
+    v_impact = all_v_impact[t] if all_v_impact is not None else None
+    return T, P, K, Z, H, D, cov_u, cov_w, v_impact, U
     #]
 
 
@@ -1307,18 +1396,30 @@ def _generate_period_data(
     u = u_array[:, t]
     v = v_array[:, t]
     w = w_array[:, t]
-    return y, u, v, w,
+    return y, u, v, w, inx_y.tolist(),
     #]
 
 
-def _make_symmetric(matrix, ):
-    return (matrix + matrix.T) / 2
+_INVERSE_FUNCTION = {
+    "regular": _np.linalg.inv,
+    "singular": _np.linalg.pinv,
+}
 
 
-def _check_singularity(F: _np.ndarray, t: int, ) -> None | NoReturn:
-    rank_F = _np.linalg.matrix_rank(F)
-    if rank_F < F.shape[0]:
-        svd = _np.linalg.svd(F, )
-        message = f"Singularity in prediction MSE matrix in period {t}, rank {rank_F} of {F.shape[0]}"
-        raise SingularMatrixError(message, )
+def _check_singularity(
+    F: _np.ndarray,
+    t: int,
+    boolex_y: list[bool],
+    when_singularity: _wd.Stream | None,
+) -> Callable | NoReturn:
+    """
+    """
+    rank_F = _np.linalg.matrix_rank(F, )
+    if rank_F == F.shape[0]:
+        return _INVERSE_FUNCTION["regular"]
+    index_y = _np.where(boolex_y, )[0].tolist()
+    message = f"Singular prediction MSE matrix in period {t}, rank {rank_F} of {F.shape[0]}"
+    # u, s, v, = _np.linalg.svd(F, )
+    when_singularity.add(message, )
+    return _INVERSE_FUNCTION["singular"]
 
