@@ -5,10 +5,10 @@ First-order system simulators
 Table of contents
 ------------------
 
-* `simulate`
+* `simulate_frame`
 * `simulate_flat`
-* `simulate_conditional`
-* `simulate_measurement`
+* `_simulate_conditional`
+* `_simulate_measurement`
 * `_generate_period_system`
 * `_store_smooth`
 * `get_init_xi`
@@ -17,39 +17,52 @@ Table of contents
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (Protocol, )
+from typing import Protocol
 import numpy as _np
 import functools as _ft
-import wlogging as _wl
+import neqs as _nq
 
-from ..dates import (Period, )
-from ..frames import (Frame, )
-from ..dataslates.main import (Dataslate, )
-from ..plans.simulation_plans import (SimulationPlan, )
+from ..dates import Period
+from ..dataslates.main import Dataslate
+from ..plans.simulation_plans import SimulationPlan
 from .. import frames as _frames
 from ..incidences import main as _incidences
-from ..incidences.main import (Token, )
-from .solutions import (Solution, )
-from .descriptors import (SolutionVectors, Squid, )
-from .kalmans import (Cache, )
+from ..incidences.main import Token
+from .descriptors import Squid
 from . import kalmans as _kalmans
 from . import shock_simulators as _shock_simulators
 
-from typing import (TYPE_CHECKING, )
+from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import (Any, )
-    from collections.abc import (Iterable, )
+    from ..frames import Frame
+    from typing import Any
+    from collections.abc import Iterable
+    from .solutions import Solution
+    from .descriptors import SolutionVectors
+
 #]
 
 
-_extract_shock_values \
-    = _shock_simulators.extract_shock_values
+# Simulator protocol requirements
+METHOD_NAME = "first_order"
+def create_frames(): ...
+def simulate_initial_guess(): ...
+def simulate_frame(): ...
 
 
-_simulate_anticipated_shocks \
-    = _shock_simulators.simulate_square_anticipated_shocks
+_extract_shock_values = _shock_simulators.extract_shock_values
+_simulate_anticipated_shocks = _shock_simulators.simulate_square_anticipated_shocks
+
+
+_RELEVANT_REGISTER_NAMES = (
+    "exogenized_anticipated",
+    "endogenized_anticipated",
+    "exogenized_unanticipated",
+    "endogenized_unanticipated",
+)
 
 
 class FordSimulatableProtocol(Protocol, ):
@@ -58,183 +71,158 @@ class FordSimulatableProtocol(Protocol, ):
     #[
 
     num_variants: int
-    solution_vectors: SolutionVectors
-    def get_singleton_solution(self, deviation: bool, ) -> Solution: ...
+    is_singleton: bool
+    def _get_dynamic_solution_vectors(self, ) -> SolutionVectors: ...
+    def _get_singleton_solution(self, deviation: bool, ) -> Solution: ...
 
     #]
 
 
-def simulate(
+def create_frames(
     model_v: FordSimulatableProtocol,
     dataslate_v: Dataslate,
     plan: SimulationPlan | None,
-    vid: int,
     *,
-    logger: _wl.Logger,
-    #
-    deviation: bool = False,
-    check_singularity: bool = False,
     force_split_frames: bool = False,
-    return_info: bool = False,
-) -> dict[str, Any]:
+) -> tuple[Frame, ...]:
     """
     """
     #[
-    periods = dataslate_v.periods
-    num_periods = dataslate_v.num_periods
-    data_array = dataslate_v.get_data_variant(0, )
-    squid = Squid(model_v, )
-    #
-    # In first-order simulation, splitting into frames is necessary only if
-    # there are some endogenized anticipated shocks and, at the same time, some
-    # nonzero unanticipated shocks (not counting the first period in either case)
-    #
     is_plan_empty = plan is None or plan.is_empty
-    needs_splitting = _check_needs_splitting(
-        plan=plan,
-        dataslate=dataslate_v,
-        squid=squid,
-        is_plan_empty=is_plan_empty,
-        force_split_frames=force_split_frames,
-    )
-    logger.debug(f"Needs splitting: {needs_splitting}")
-    #
+    if force_split_frames:
+        needs_splitting = True
+    elif is_plan_empty:
+        needs_splitting = False
+    else:
+        needs_splitting = plan.any_endogenized_anticipated_except_start
+    # TODO: Optimize the simulation end of each frame based on the presence of
+    # endogenized anticipated shocks and nonzero anticipated shocks
     if needs_splitting:
-        create_frames = _frames.split_into_frames
+        base_end = dataslate_v.base_periods[-1]
+        frames = _frames.split_into_frames(
+            model_v, dataslate_v, plan,
+            get_simulation_end=lambda *_, : base_end,
+        )
     else:
-        create_frames = _frames.setup_single_frame
-    #
+        frames = _frames.setup_single_frame(
+            model_v, dataslate_v, plan,
+        )
+    return frames
+    #]
+
+
+def simulate_initial_guess(*args, **kwargs, ) -> None:
+    """
+    """
+    pass
+
+
+def simulate_frame(
+    model_v: FordSimulatableProtocol,
+    frame_ds: Dataslate,
+    *,
+    frame: Frame,
+    input_data_array: _np.ndarray,
+    plan: SimulationPlan | None,
+    simulation_header: str,
+    return_info: bool = False,
+    # Method specific settings
+    deviation: bool = False,
+    check_singularity: bool = False,
+) -> bool:
+    """
+    """
+    #[
+    is_plan_empty = plan is None or plan.is_empty
     if is_plan_empty:
-        simulate_frame = simulate_flat
+        simulate_transition = simulate_flat
     else:
-        plan_registers = _get_plan_registers_as_bool_arrays(plan, periods, )
-        simulate_frame = _ft.partial(
-            simulate_conditional,
+        squid = Squid.from_squidable(model_v, )
+        plan_registers = plan.get_registers_as_bool_arrays(
+            periods=frame_ds.periods,
+            register_names=_RELEVANT_REGISTER_NAMES,
+        )
+        simulate_transition = _ft.partial(
+            _simulate_conditional,
             check_singularity=check_singularity,
-            input_data_array=data_array.copy(),
+            input_data_array=input_data_array,
             plan_registers=plan_registers,
             Z_xi=_create_Z_xi(squid, )
         )
-    #
-    # Split the base span into frames
-    # Each frame is simulated until the end of the dataslate base span
-    #
-    # TODO: Optimize the simulation end of each frame based on the presence of
-    # endogenized anticipated shocks and nonzero anticipated shocks
-    #
-    dataslate_end = dataslate_v.base_periods[-1]
-    frames = create_frames(
-        model_v, dataslate_v, plan,
-        get_simulation_end=lambda *_, : dataslate_end,
+    xi_array_in_frame = simulate_transition(
+        model_v, frame_ds, frame,
+        deviation=deviation,
     )
-    logger.debug(f"Splitting simulation into {len(frames)} frame(s)")
-    #
-    # Resolve columns within each frame based on the start date of the dataslate
-    for frame in frames:
-        frame.resolve_columns(periods[0], )
-    #
-    #
-    #===========================================================================
-    xi_array = _np.full((squid.num_xi, num_periods, ), _np.nan, )
-    frame_db = ()
-    for frame in frames:
-        #
-        if needs_splitting:
-            frame_ds = dataslate_v.copy(invariant=False, )
-            frame.reset_unanticipated(frame_ds, model_v, )
-        else:
-            frame_ds = dataslate_v
-        #
-        xi_array_in_frame = simulate_frame(
-            model_v, frame_ds, frame, vid,
-            logger=logger,
-            deviation=deviation,
-        )
-        xi_array[:, frame.slice] = xi_array_in_frame[:, frame.slice]
-        #
-        simulate_measurement(
-            model_v, dataslate_v, frame, xi_array, vid,
-            logger=logger,
-            deviation=deviation,
-        )
-        #
-        # Copy frame data back to the main dataslate if there are more than one
-        # frame; otherwise the main dataslate is the same as the frame dataslate
-        if needs_splitting:
-            frame.copy_frame_data(dataslate_v, frame_ds, )
-        #
-        # Create a frame databox only if output info is requested
-        if return_info:
-            frame_db = frame_db + (frame_ds.to_databox(), )
-    #===========================================================================
-    #
-    #
-    info = {
-        "method": "first_order",
-        "frames": frames,
-        "frame_db": frame_db,
-    }
-    #
-    return info
+    _simulate_measurement(
+        model_v, frame_ds, frame, xi_array_in_frame,
+        deviation=deviation,
+    )
+
+    return _nq.ExitStatus.SUCCESS
     #]
 
 
 def simulate_flat(
     model_v: FordSimulatableProtocol,
-    dataslate_v: Dataslate,
+    frame_ds: Dataslate,
     frame: Frame,
-    vid: int,
-    *,
-    logger: _wl.Logger,
     deviation: bool,
+    ignore_shocks: bool = False,
+    exogenous_impact: _np.ndarray | None = None,
 ) -> _np.ndarray:
     """
     """
     #[
-    logger.debug(f"Running flat simulation {frame.start}>>{frame.end}>>{frame.simulation_end}")
-    solution = model_v.get_singleton_solution(deviation=deviation, )
-    vec = model_v.solution_vectors
-    num_periods = dataslate_v.num_periods
-    simulation_columns = tuple(range(*frame.simulation_slice.indices(num_periods, ), ), )
+    solution = model_v._get_singleton_solution(deviation=deviation, )
+    vec = model_v._get_dynamic_solution_vectors()
+    num_periods = frame_ds.num_periods
+    simulation_columns = list(range(*frame.simulation_slice.indices(num_periods, ), ), )
     curr_xi_qids, curr_xi_indexes = vec.get_curr_transition_indexes()
+    curr_xi_qids = list(curr_xi_qids)
+    curr_xi_indexes = list(curr_xi_indexes)
     #
     T = solution.T
     P = solution.P
     K = solution.K
     #
-    dataslate_v.logarithmize()
-    data_array = dataslate_v.get_data_variant(0, )
+    frame_ds.logarithmize()
+    data_array = frame_ds.get_data_variant(0, )
     num_xi = len(vec.transition_variables)
     xi_array = _np.full((num_xi, num_periods, ), _np.nan, )
-    u_array = _extract_shock_values(data_array, vec.unanticipated_shocks, )
-    all_v_impact = _simulate_anticipated_shocks(model_v, dataslate_v, frame, )
+    # TODO: precalculate all_v_impact as exogenous_impact
+    all_v_impact = None
+    Pu = None
+    if not ignore_shocks:
+        u_array = _extract_shock_values(data_array, vec.unanticipated_shocks, )
+        Pu = P @ u_array
+        all_v_impact = _simulate_anticipated_shocks(model_v, frame_ds, frame, )
     #
     # Simulate one period at a time
     # Store results in data_array
-    # Capture xi_array for later use
-    #
+    # Capture xi_array for later use in measurement equations
     xi = get_init_xi(data_array, vec.transition_variables, simulation_columns[0], )
     zero_false_init_xi(xi, vec.true_initials, )
     for t in simulation_columns:
-        xi = T @ xi + P @ u_array[:, t] + K
-        if all_v_impact[t] is not None:
+        xi = T @ xi + K
+        if Pu is not None:
+            xi += Pu[:, t]
+        if all_v_impact is not None and all_v_impact[t] is not None:
             xi += all_v_impact[t]
+        if exogenous_impact is not None:
+            xi += exogenous_impact[:, t]
         xi_array[:, t] = xi
-        data_array[curr_xi_qids, t] = xi[curr_xi_indexes, ...]
-    #
-    dataslate_v.delogarithmize()
+        data_array[curr_xi_qids, t] = xi[curr_xi_indexes]
     #
+    frame_ds.delogarithmize()
     return xi_array
     #]
 
 
-def simulate_conditional(
+def _simulate_conditional(
     model_v: FordSimulatableProtocol,
-    dataslate_v: Dataslate,
+    frame_ds: Dataslate,
     frame: Frame,
-    vid: int,
     *,
-    logger: _wl.Logger,
     deviation: bool,
     check_singularity: bool,
     #
@@ -245,15 +233,14 @@ def simulate_conditional(
     """
     """
     #[
-    logger.debug(f"Running conditional simulation {frame.start}>>{frame.end}>>{frame.simulation_end}")
-    solution = model_v.get_singleton_solution(deviation=deviation, )
-    vec = model_v.solution_vectors
-    periods = dataslate_v.periods
-    num_periods = dataslate_v.num_periods
-    squid = Squid(model_v, )
+    solution = model_v._get_singleton_solution(deviation=deviation, )
+    vec = model_v._get_dynamic_solution_vectors()
+    periods = frame_ds.periods
+    num_periods = frame_ds.num_periods
+    squid = Squid.from_squidable(model_v, )
     #
-    dataslate_v.logarithmize()
-    data_array = dataslate_v.get_data_variant()
+    frame_ds.logarithmize()
+    data_array = frame_ds.get_data_variant()
     #
     # Prepare array of all xi values (needed for measurement simulation)
     xi_array = _np.full((squid.num_xi, num_periods, ), _np.nan, )
@@ -281,7 +268,7 @@ def simulate_conditional(
         std_v_array = data_array[squid.std_v_qids, frame.simulation_slice]
         std_w_array = data_array[squid.std_w_qids, frame.simulation_slice]
     #
-    all_v_impact = _simulate_anticipated_shocks(model_v, dataslate_v, frame, )
+    all_v_impact = _simulate_anticipated_shocks(model_v, frame_ds, frame, )
     #
     # Preallocate default data
     curr_xi_exogenized = _np.full((squid.num_curr_xi, num_periods, ), _np.nan, )
@@ -291,10 +278,10 @@ def simulate_conditional(
     # Insert SimulationPlan information
     incidence_v, forward, Rx, std_v_endogenized = None, None, None, None
     if plan_registers:
-        packed = (plan_registers, input_data_array, squid, frame.slice, logger, )
+        packed = (plan_registers, input_data_array, squid, frame.simulation_slice, )
         _insert_exogenized_unanticipated(curr_xi_exogenized, packed, )
         _insert_exogenized_anticipated(curr_xi_exogenized, packed, )
-        _insert_endogenized_unanticipated(std_u_endogenized, packed, )
+        _insert_std_endogenized_unanticipated(std_u_endogenized, packed, )
         #
         incidence_v = plan_registers["endogenized_anticipated"][:, frame.simulation_slice]
         sum_incidence_v = incidence_v.sum()
@@ -303,7 +290,6 @@ def simulate_conditional(
             forward = incidence_v.any(axis=0, ).nonzero()[0].max()
             Rx = solution.expand_square_solution(forward, )
             std_v_endogenized = std_v_array[incidence_v, ]
-        logger.debug(f"Endogenized anticipated: {sum_incidence_v}")
     #
     curr_xi_exogenized = curr_xi_exogenized[:, frame.simulation_slice]
     std_u_endogenized = std_u_endogenized[:, frame.simulation_slice]
@@ -351,33 +337,30 @@ def simulate_conditional(
         cache=cache,
         store_smooth=store_smooth,
     )
-    dataslate_v.delogarithmize()
+    frame_ds.delogarithmize()
     #
     return xi_array
     #]
 
 
-def simulate_measurement(
+def _simulate_measurement(
     model_v: FordSimulatableProtocol,
-    dataslate_v: Dataslate,
+    frame_ds: Dataslate,
     frame: Frame,
     xi_array: _np.ndarray,
-    vid: int,
     *,
-    logger: _wl.Logger,
     deviation: bool = False,
     **kwargs,
 ) -> None:
     """
     """
     #[
-    logger.debug("Running measurement simulation")
-    solution = model_v.get_singleton_solution(deviation=deviation, )
-    vec = model_v.solution_vectors
-    num_periods = dataslate_v.num_periods
+    solution = model_v._get_singleton_solution(deviation=deviation, )
+    vec = model_v._get_dynamic_solution_vectors()
+    num_periods = frame_ds.num_periods
     #
-    dataslate_v.logarithmize()
-    data_array = dataslate_v.get_data_variant(0, )
+    frame_ds.logarithmize()
+    data_array = frame_ds.get_data_variant(0, )
     #
     Z = solution.Z
     H = solution.H
@@ -389,7 +372,7 @@ def simulate_measurement(
     for t in range(*frame.simulation_slice.indices(num_periods), ):
         data_array[y_qids, t] = Z @ xi_array[:, t] + H @ w_array[:, t] + D
     #
-    dataslate_v.delogarithmize()
+    frame_ds.delogarithmize()
     #]
 
 
@@ -465,8 +448,8 @@ def _generate_initials(
         init_mse = initials[1]
         uknown_init_impact = None
         initials = (
-            _np.pad(initials[0], (0, num_v_endogenized), ),
-            _block_diag(initials[1], _np.diag(std_v_endogenized**2), ),
+            _np.pad(init_med, (0, num_v_endogenized), ),
+            _sp.linalg.block_diag(init_mse, _np.diag(std_v_endogenized**2), ),
             uknown_init_impact,
         )
     #
@@ -512,7 +495,6 @@ def _generate_R(
 
 def _generate_period_data(
     t: int,
-
     curr_xi_exogenized: _np.ndarray | None,
     u_array: _np.ndarray,
     v_array: _np.ndarray,
@@ -525,11 +507,13 @@ def _generate_period_data(
         inx_y = ~_np.isnan(curr_xi_exogenized[:, t], )
         y = curr_xi_exogenized[inx_y, t]
     else:
+        inx_y = None
         y = _np.zeros((0, ), )
     u = u_array[:, t]
     v = v_array[:, t]
     w = w_array[:, t]
-    return y, u, v, w,
+    inx_y = inx_y.tolist() if inx_y is not None else None
+    return y, u, v, w, inx_y,
     #]
 
 
@@ -603,12 +587,6 @@ def _get_plan_registers_as_bool_arrays(
     """
     """
     #[
-    register_names = (
-        "exogenized_anticipated",
-        "endogenized_anticipated",
-        "exogenized_unanticipated",
-        "endogenized_unanticipated",
-    )
     get_register = _ft.partial(
         plan.get_register_as_bool_array,
         names=...,
@@ -638,13 +616,10 @@ def _insert_exogenized_unanticipated(
     """
     """
     #[
-    plan_registers, input_data_array, squid, frame_slice, logger = packed
-    #
-    ___ = plan_registers["exogenized_unanticipated"][:, frame_slice]
-    curr_xi_exogenized[:, frame_slice][___] \
-        = input_data_array[squid.curr_xi_qids, frame_slice][___]
-    #
-    logger.debug(f"Exogenized unanticipated: {___.sum()}")
+    plan_registers, input_data_array, squid, simulation_slice, = packed
+    incidence = plan_registers["exogenized_unanticipated"][:, simulation_slice]
+    curr_xi_exogenized[:, simulation_slice][incidence] \
+        = input_data_array[squid.curr_xi_qids, simulation_slice][incidence]
     #]
 
 
@@ -655,80 +630,35 @@ def _insert_exogenized_anticipated(
     """
     """
     #[
-    plan_registers, input_data_array, squid, frame_slice, logger = packed
-    #
-    ___ = plan_registers["exogenized_anticipated"][:, frame_slice]
-    curr_xi_exogenized[:, frame_slice][___] \
-        = input_data_array[squid.curr_xi_qids, frame_slice][___]
-    #
-    logger.debug(f"Exogenized anticipated: {___.sum()}")
+    plan_registers, input_data_array, squid, simulation_slice, = packed
+    incidence = plan_registers["exogenized_anticipated"][:, simulation_slice]
+    curr_xi_exogenized[:, simulation_slice][incidence] \
+        = input_data_array[squid.curr_xi_qids, simulation_slice][incidence]
     #]
 
 
-def _insert_endogenized_unanticipated(
+def _insert_std_endogenized_unanticipated(
     std_u_endogenized: _np.ndarray,
     packed: tuple,
 ) -> None:
     """
     """
     #[
-    plan_registers, input_data_array, squid, frame_slice, logger = packed
+    plan_registers, input_data_array, squid, simulation_slice, = packed
+    incidence = plan_registers["endogenized_unanticipated"][:, simulation_slice]
+    std_u_endogenized[:, simulation_slice][incidence] \
+        = input_data_array[squid.std_u_qids, simulation_slice][incidence]
     #
-    ___ = plan_registers["endogenized_unanticipated"][:, frame_slice]
-    std_u_endogenized[:, frame_slice][___] \
-        = input_data_array[squid.std_u_qids, frame_slice][___]
-    #
-    logger.debug(f"Endogenized unanticipated: {___.sum()}")
     #]
 
 
 
-def _block_diag(A, B):
-    """
-    Concatenate two square matrices into a block diagonal matrix
-    """
-    #[
-    return _np.block([
-        [A, _np.zeros((A.shape[0], B.shape[1]), )],
-        [_np.zeros((B.shape[0], A.shape[1]), ), B],
-    ])
-    #]
-
-
 def _check_needs_splitting(
     plan: SimulationPlan | None,
-    dataslate: Dataslate,
-    squid: Squid,
     is_plan_empty: bool,
-    force_split_frames: bool,
 ) -> bool:
     """
     """
     #[
-    if force_split_frames:
-        return True
-    #
-    if is_plan_empty and not force_split_frames:
-        return False
-    #
-    any_endogenized_anticipated = (
-        plan is not None
-        and plan.any_endogenized_anticipated_except_start
-    )
-    if not any_endogenized_anticipated:
-        return False
-    #
-    base_columns_except_start = dataslate.base_columns[1:]
-    data_array = dataslate.get_data_variant(0, )
-    any_nonzero_unanticipated_shocks \
-            = data_array \
-            [squid.v_qids, :] \
-            [:, base_columns_except_start] \
-            .any()
-    if not any_nonzero_unanticipated_shocks:
-        return False
-    #
-    return True
     #]
 
-
