@@ -35,6 +35,7 @@ $$
 
 
 #[
+
 from __future__ import annotations
 
 import warnings as _wa
@@ -43,19 +44,31 @@ import numpy as _np
 import scipy as _sp
 import copy as _co
 
-from typing import (TYPE_CHECKING, )
+from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import (Self, Callable, )
-    from numbers import (Real, )
+    from typing import Self, Callable
+    from numbers import Real
     from ..fords import descriptors as _descriptors
     from ..fords import systems as _systems
+
 #]
 
 
+__all__ = (
+    "STABLE", "UNIT_ROOT", "UNSTABLE", "UNIT",
+)
+
+
+class UnitRootException(Exception):
+    pass
+
+
 class EigenvalueKind(_en.Flag, ):
     STABLE = _en.auto()
-    UNIT = _en.auto()
+    UNIT_ROOT = _en.auto()
     UNSTABLE = _en.auto()
+    UNIT = UNIT_ROOT
+    ALL = STABLE | UNIT_ROOT | UNSTABLE
 
 
 class SystemStabilityKind(_en.Flag, ):
@@ -64,11 +77,6 @@ class SystemStabilityKind(_en.Flag, ):
     NO_STABLE = _en.auto()
 
 
-class VariableStability(_en.Flag, ):
-    STABLE = _en.auto()
-    UNIT_ROOT = _en.auto()
-
-
 class Solution:
     """
     ## Square solution:
@@ -105,20 +113,69 @@ class Solution:
         "T", "P", "R", "K", "Z", "H", "D",
         "Ta", "Pa", "Ra", "Ka", "Za", "Ua",
         "J", "Ru", "X", "Xa",
+        "square_expansion",
+        "triangular_expansion",
         "eigenvalues",
+
         "eigenvalues_stability",
         "system_stability",
         "transition_vector_stability",
         "measurement_vector_stability",
-        "square_expansion",
-        "triangular_expansion",
     )
 
-    def __init__(self, ) -> None:
+    def __init__(
+        self,
+        T: _np.ndarray | None = None,
+        P: _np.ndarray | None = None,
+        R: _np.ndarray | None = None,
+        K: _np.ndarray | None = None,
+        Z: _np.ndarray | None = None,
+        H: _np.ndarray | None = None,
+        D: _np.ndarray | None = None,
+        Ta: _np.ndarray | None = None,
+        Pa: _np.ndarray | None = None,
+        Ra: _np.ndarray | None = None,
+        Ka: _np.ndarray | None = None,
+        Za: _np.ndarray | None = None,
+        Ua: _np.ndarray | None = None,
+        J: _np.ndarray | None = None,
+        Ru: _np.ndarray | None = None,
+        X: _np.ndarray | None = None,
+        Xa: _np.ndarray | None = None,
+        square_expansion: list[_np.ndarray] | None = None,
+        triangular_expansion: list[_np.ndarray] | None = None,
+        eigenvalues: tuple[Real, ...] | None = None,
+        eigenvalues_stability: tuple[EigenvalueKind, ...] | None = None,
+        system_stability: SystemStabilityKind | None = None,
+        transition_vector_stability: tuple[EigenvalueKind, ...] | None = None,
+        measurement_vector_stability: tuple[EigenvalueKind, ...] | None = None,
+    ) -> None:
+        r"""
         """
-        """
-        for name in self.__slots__:
-            setattr(self, name, None, )
+        self.T = T
+        self.P = P
+        self.R = R
+        self.K = K
+        self.Z = Z
+        self.H = H
+        self.D = D
+        self.Ta = Ta
+        self.Pa = Pa
+        self.Ra = Ra
+        self.Ka = Ka
+        self.Za = Za
+        self.Ua = Ua
+        self.J = J
+        self.Ru = Ru
+        self.X = X
+        self.Xa = Xa
+        self.square_expansion = square_expansion
+        self.triangular_expansion = triangular_expansion
+        self.eigenvalues = eigenvalues
+        self.eigenvalues_stability = eigenvalues_stability
+        self.system_stability = system_stability
+        self.transition_vector_stability = transition_vector_stability
+        self.measurement_vector_stability = measurement_vector_stability
 
     @classmethod
     def from_system(
@@ -126,7 +183,7 @@ class Solution:
         descriptor: _descriptors.Descriptor,
         system: _systems.System,
         *,
-        tolerance: float = 1e-12,
+        tolerance: float,
         clip_small: bool = False,
     ) -> Self:
         """
@@ -148,16 +205,20 @@ class Solution:
         # not detached yet, the system is called "preliminary"
         qz_matrixes, self.eigenvalues = \
             _solve_ordqz(system, is_alpha_beta_stable_or_unit_root, is_stable_root, is_unit_root, )
+        self._classify_eigenvalues_stability(is_stable_root, is_unit_root, )
+        #
         triangular_solution_prelim = \
             _solve_transition_equations(descriptor, system, qz_matrixes, )
         #
         # Detach unit from stable roots to create the final triangular solution
         # From the final triangular solution, calculate the square solution
-        triangular_solution = detach_stable_from_unit_roots(
+        *triangular_solution, check_num_unit_roots = detach_stable_from_unit_roots(
             triangular_solution_prelim,
             is_unit_root,
             clip=clip,
         )
+        if check_num_unit_roots != self.num_unit_roots:
+            raise UnitRootException
         #
         square_solution = _square_from_triangular(triangular_solution, )
         #
@@ -171,7 +232,6 @@ class Solution:
             self.Ua,
             clip=clip,
         )
-        self._classify_eigenvalues_stability(is_stable_root, is_unit_root, )
         self._classify_system_stability(descriptor.get_num_forwards(), )
         self._classify_transition_vector_stability(tolerance=tolerance, )
         self._classify_measurement_vector_stability(tolerance=tolerance, )
@@ -185,8 +245,7 @@ class Solution:
     def deviation_solution(
         klass,
         other: Self,
-        /,
-    ) -> Solution:
+    ) -> Self:
         """
         Create a shallow copy of the solution, and replace constant vectors with
         zeros
@@ -194,9 +253,12 @@ class Solution:
         self = klass()
         for n in self.__slots__:
             setattr(self, n, getattr(other, n, None), )
-        self.K = _np.zeros_like(other.K, )
-        self.Ka = _np.zeros_like(other.Ka, )
-        self.D = _np.zeros_like(other.D, )
+        if self.K is not None:
+            self.K = _np.zeros_like(other.K, )
+        if self.Ka is not None:
+            self.Ka = _np.zeros_like(other.Ka, )
+        if self.D is not None:
+            self.D = _np.zeros_like(other.D, )
         return self
 
     @property
@@ -232,7 +294,7 @@ class Solution:
     @property
     def num_unit_roots(self, /, ) -> int:
         """==Number of unit roots=="""
-        return self.eigenvalues_stability.count(EigenvalueKind.UNIT)
+        return self.eigenvalues_stability.count(EigenvalueKind.UNIT_ROOT)
 
     @property
     def num_stable(self, /, ) -> int:
@@ -271,27 +333,51 @@ class Solution:
 
     @property
     def boolex_stable_transition_vector(self, /, ) -> tuple[int, ...]:
-        """==Index of stable transition vector elements=="""
+        r"""==Index of stable transition vector elements=="""
         return _np.array(tuple(
-            i == VariableStability.STABLE
+            i == EigenvalueKind.STABLE
             for i in self.transition_vector_stability
         ), dtype=bool, )
 
     @property
     def boolex_stable_measurement_vector(self, /, ) -> tuple[int, ...]:
-        """==Index of stable measurement vector elements=="""
+        r"""==Index of stable measurement vector elements=="""
         return _np.array(tuple(
-            i == VariableStability.STABLE
+            i == EigenvalueKind.STABLE
             for i in self.measurement_vector_stability
         ), dtype=bool, )
 
-    def copy(self, ) -> Self:
+    def unpack_square_solution(
+        self, 
+        forward: int = 0,
+    ) -> tuple[_np.ndarray, ...]:
+        r"""
+        Return square solution matrices in the following order:
+        T, P, R, K, Z, H, D, None
+        where R is expanded until t+forward
         """
+        R = self.expand_square_solution(forward, )
+        return self.T, self.P, R, self.K, self.Z, self.H, self.D, None,
+
+    def unpack_triangular_solution(
+        self,
+        forward: int = 0,
+    ) -> tuple[_np.ndarray, ...]:
+        r"""
+        Return triangular solution matrices in the following order:
+        Ta, Pa, Ra, Ka, Za, H, D, Ua
+        where Ra is expanded until t+forward
+        """
+        Ra = self.expand_triangular_solution(forward=0, )
+        return self.Ta, self.Pa, Ra, self.Ka, self.Za, self.H, self.D, self.Ua,
+
+    def copy(self, ) -> Self:
+        r"""
         """
         return _co.deepcopy(self, )
 
-    def expand_square_solution(self, forward: int, /, ) -> list[_np.ndarray]:
-        """
+    def expand_square_solution(self, forward: int, ) -> list[_np.ndarray]:
+        r"""
         Expand R matrices of square solution for t+1...t+forward
         """
         return _get_solution_expansion(
@@ -300,7 +386,7 @@ class Solution:
             forward,
         )
 
-    def expand_triangular_solution(self, forward: int, /, ) -> list[_np.ndarray]:
+    def expand_triangular_solution(self, forward: int, ) -> list[_np.ndarray]:
         """
         Expand Ra matrices of square solution for t+1...t+forward
         """
@@ -314,7 +400,6 @@ class Solution:
         self,
         is_stable_root: Callable[[Real], bool],
         is_unit_root: Callable[[Real], bool],
-        /,
     ) -> None:
         self.eigenvalues_stability = tuple(
             _classify_eigenvalue_stability(v, is_stable_root, is_unit_root, )
@@ -324,7 +409,6 @@ class Solution:
     def _classify_system_stability(
         self,
         num_forwards: int,
-        /,
     ) -> None:
         num_unstable = self.eigenvalues_stability.count(EigenvalueKind.UNSTABLE)
         if num_unstable == num_forwards:
@@ -336,7 +420,6 @@ class Solution:
 
     def _classify_transition_vector_stability(
         self,
-        /,
         tolerance: float = 1e-12,
     ) -> None:
         self.transition_vector_stability \
@@ -348,7 +431,6 @@ class Solution:
 
     def _classify_measurement_vector_stability(
         self,
-        /,
         tolerance: float = 1e-12,
     ) -> None:
         self.measurement_vector_stability \
@@ -360,14 +442,14 @@ class Solution:
     #]
 
 
-def left_div(A, B):
+def left_div(A: _np.ndarray, B: _np.ndarray, ) -> _np.ndarray:
     r"""
     Solve A \ B = pinv(A) @ B or inv(A) @ B
     """
     return _np.linalg.lstsq(A, B, rcond=None)[0]
 
 
-def right_div(B, A):
+def right_div(B: _np.ndarray, A: _np.ndarray, ) -> _np.ndarray:
     r"""
     Solve B / A which is (A' \ B')'
     """
@@ -416,7 +498,7 @@ def detach_stable_from_unit_roots(
     Ra = u.T @ Rg
     Ka = u.T @ Kg
     Xa = u.T @ Xg if Xg is not None else None
-    return Ua, Ta, Pa, Ra, Ka, Xa, J, Ru
+    return Ua, Ta, Pa, Ra, Ka, Xa, J, Ru, check_num_unit_roots
     #]
 
 
@@ -544,7 +626,7 @@ def _classify_eigenvalue_stability(
     if is_stable_root(abs_eigenvalue):
         return EigenvalueKind.STABLE
     elif is_unit_root(abs_eigenvalue):
-        return EigenvalueKind.UNIT
+        return EigenvalueKind.UNIT_ROOT
     else:
         return EigenvalueKind.UNSTABLE
     #]
@@ -562,8 +644,8 @@ def _classify_solution_vector_stability(
     test_matrix = _np.abs(transform_matrix[:, :num_unit_roots], )
     index = _np.any(test_matrix > tolerance, axis=1, )
     return tuple(
-        VariableStability.UNIT_ROOT if i
-        else VariableStability.STABLE
+        EigenvalueKind.UNIT_ROOT if i
+        else EigenvalueKind.STABLE
         for i in index
     )
     #]
@@ -599,3 +681,8 @@ def _get_solution_expansion(
     # ]
 
 
+STABLE = EigenvalueKind.STABLE
+UNIT_ROOT = EigenvalueKind.UNIT_ROOT
+UNSTABLE = EigenvalueKind.UNSTABLE
+UNIT = UNIT_ROOT
+
