@@ -1,32 +1,35 @@
 """
-# First-order unsolved system matrices
 
-## Unsolved system
+# First-order dynamic system with expectations unsolved
+
+## Unsolved-expectations system
 
 $$
-A E[x_t] + B E[x_{t-1}] + C + D u_t + E v_t = 0 \\
+A E[x_t] + B E[x_{t-1}] + C + D u_t + E v_t = 0
 $$
 
 $$
 F y_t + G x_t + H + J w_t = 0
 $$
+
 """
 
 
 #[
+
 from __future__ import annotations
 
-import dataclasses as _dc
 import numpy as _np
 
 from . import descriptors as _descriptors
 from ..simultaneous import _flags as _flags
+
 #]
 
 
 class System:
     """
-    Unsolved system matrices
+    Unsolved-expectations system
     """
     #[
 
@@ -57,9 +60,9 @@ class System:
         smap = descriptor.system_map
         svec = descriptor.system_vectors
 
-        self.A = _np.zeros(svec.shape_A_excl_dynid, dtype=float)
+        self.A = _np.zeros(svec.shape_A_excl_dynid, dtype=float, )
         self.A[smap.A.lhs] = td[smap.A.rhs]
-        self.A = _np.vstack((self.A, smap.dynid_A))
+        self.A = _np.vstack((self.A, smap.dynid_A, ), )
 
         self.B = _np.zeros(svec.shape_B_excl_dynid, dtype=float)
         self.B[smap.B.lhs] = td[smap.B.rhs]
@@ -126,3 +129,4 @@ def _get_vector(
     return x
     #]
 
+
