@@ -27,7 +27,7 @@ class HumanBlock:
 
     def __init__(
         self,
-        block: _Block,
+        block: Block,
         equations: Iterable[Equation],
         quantities: Iterable[Quantity],
     ) -> None:
@@ -37,10 +37,27 @@ class HumanBlock:
         self.equations = tuple(equations[eid].human for eid in sorted(block.eids))
         self.quantities = tuple(quantities[qid].human for qid in sorted(block.qids))
 
+    @property
+    def num_equations(self, ) -> int:
+        r"""
+        """
+        return len(self.equations)
+
+    @property
+    def num_quantities(self, ) -> int:
+        r"""
+        """
+        return len(self.quantities)
+
+    def __repr__(self) -> str:
+        """
+        """
+        return f"HumanBlock[{self.num_equations}×{self.num_quantities}]"
+
     #]
 
 
-class _Block:
+class Block:
     """
     """
     #[
@@ -85,7 +102,7 @@ def blaze(
     qids: Iterable[int] = None,
     #
     return_info: bool = False,
-) -> tuple[tuple[_Block, ...], dict[str, Any]]:
+) -> tuple[Block, ...] | tuple[tuple[Block, ...], dict[str, Any]]:
     """
     Complete sequential block analysis of incidence matrix
     """
@@ -109,8 +126,30 @@ def blaze(
     #
     if im_inner.size:
         eids_inner, qids_inner, im_inner, *_ \
-            = triangularize_inner_block \
-            (im_inner, eids=eids_inner, qids=qids_inner, )
+            = triangularize_inner_block(im_inner, eids=eids_inner, qids=qids_inner, )
+
+        # from pyomo.contrib.incidence_analysis.dulmage_mendelsohn import dulmage_mendelsohn as dm
+        # import scipy as _sp
+        # im_inner = im
+        # rr, cc = im_inner.nonzero()
+        # z = _sp.sparse.coo_matrix((im_inner[rr, cc], (rr, cc, )), shape=im_inner.shape, )
+        # row_part, col_part, = dm(z)
+        # row_perm = _np.concatenate((
+        #     row_part.unmatched,
+        #     row_part.underconstrained,
+        #     row_part.square,
+        #     row_part.overconstrained
+        #     )).astype(int)
+        # col_perm = _np.concatenate((
+        #     col_part.unmatched,
+        #     col_part.underconstrained,
+        #     col_part.square,
+        #     col_part.overconstrained
+        #     )).astype(int)
+        # im_inner = im_inner[row_perm, :]
+        # im_inner = im_inner[:, col_perm]
+        # eids_inner = tuple(_np.array(eids_inner, dtype=int)[row_perm])
+        # qids_inner = tuple(_np.array(qids_inner, dtype=int)[col_perm])
     #
     # Combine the results
     #
@@ -120,9 +159,9 @@ def blaze(
     # Step 3: Create a tuple with a Block object for each block of
     # equations and equantities
     #
-    first_blocks = tuple(_Block((eid, ), (qid, ), ) for eid, qid in zip(eids_first, qids_first, ))
+    first_blocks = tuple(Block((eid, ), (qid, ), ) for eid, qid in zip(eids_first, qids_first, ))
     inner_blocks = tuple(_generate_inner_blocks(im_inner, eids=eids_inner, qids=qids_inner, ))
-    last_blocks = tuple(_Block((eid, ), (qid, ), ) for eid, qid in zip(eids_last, qids_last, ))
+    last_blocks = tuple(Block((eid, ), (qid, ), ) for eid, qid in zip(eids_last, qids_last, ))
     #
     out_blocks = first_blocks + inner_blocks + last_blocks
     if return_info:
@@ -145,7 +184,7 @@ def _generate_inner_blocks(
     im: _np.ndarray,
     eids: Iterable[int],
     qids: Iterable[int],
-) -> Generator[_Block]:
+) -> Generator[Block]:
     """
     """
     #[
@@ -153,7 +192,7 @@ def _generate_inner_blocks(
     qids = tuple(qids)
     while im.size:
         block_size = next(i for i in range(1, im.shape[0] + 1) if not im[:i, i:].any())
-        yield _Block(eids[:block_size], qids[:block_size], )
+        yield Block(eids[:block_size], qids[:block_size], )
         eids = eids[block_size:]
         qids = qids[block_size:]
         im = im[block_size:, block_size:]
@@ -361,11 +400,14 @@ def is_sequential(
 
 
 def human_blocks_from_blocks(
-    blocks: Iterable[_Block],
+    blocks: Iterable[Block],
     equations: Iterable[Equation],
     quantities: Iterable[Quantity],
 ) -> tuple[HumanBlock, ...]:
     """
     """
-    return tuple(HumanBlock(block, equations, quantities) for block in blocks)
+    return tuple(
+        HumanBlock(block, equations, quantities)
+        for block in blocks
+    )
 
