@@ -18,8 +18,9 @@ from .. import quantities as _quantities
 
 
 _PRINT_TOKEN = "x[({qid},t{shift:+g})]"
-_PRINT_TOKEN_ZERO_SHIFT = "x[({qid},t)]"
+_PRINT_ZERO_SHIFT_TOKEN = "x[({qid},t)]"
 _PRINT_SHIFT = "[{shift:+g}]"
+ZERO_SHIFT_TOKEN_PATTERN = r"x\[\((\d+),t\)\]"
 
 
 class Token(NamedTuple, ):
@@ -50,14 +51,13 @@ class Token(NamedTuple, ):
 
     def print_xtring(self, /, ) -> str:
         return (
-            _PRINT_TOKEN.format(qid=self.qid, shift=self.shift)
-            if self.shift else _PRINT_TOKEN_ZERO_SHIFT.format(qid=self.qid)
+            _PRINT_TOKEN.format(qid=self.qid, shift=self.shift, )
+            if self.shift else _PRINT_ZERO_SHIFT_TOKEN.format(qid=self.qid, )
         )
 
     #]
 
 
-
 def get_max_shift(tokens: Iterable[Token]) -> int:
     return max(tok.shift for tok in tokens) if tokens else None
 
