@@ -4,9 +4,11 @@ HTML report objects
 
 
 #[
+
 from __future__ import annotations
 
-from .. import pages as _pages
+import documark as _dm
+
 #]
 
 
@@ -15,7 +17,7 @@ __all__ = (
 )
 
 
-@_pages.reference(
+@_dm.reference(
     path=("visualization_reporting", "html_reports.md", ),
     categories={
         "constructor": "Creating new `Rephrase` objects",
@@ -34,7 +36,7 @@ HTML reports are implemented as `Rephrase` objects.
 ················································································
     """
 
-    @_pages.reference(category="constructor", call_name="Rephrase", )
+    @_dm.reference(category="constructor", call_name="Rephrase", )
     def __init__(
         self,
         /,
