@@ -1,39 +1,40 @@
-"""
+r"""
 Sequential models
 """
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (TYPE_CHECKING, )
 import numpy as _np
 import copy as _co
 import os as _os
+import documark as _dm
 
 from .. import equations as _equations
 from .. import quantities as _quantities
 from .. import sources as _sources
-from .. import pages as _pages
-from ..incidences.main import (Token, )
+from ..incidences.main import Token
 from ..incidences import main as _incidences
 from ..incidences import blazer as _blazer
 from ..explanatories import main as _explanatories
 from .. import has_variants as _has_variants
 
-from . import _invariants as _invariants
-from . import _variants as _variants
 from . import _simulate as _simulate
 from . import _assigns as _assigns
 from . import _get as _get
-from . import _slatable_protocol as _slatable_protocol
+from . import _slatable_protocols as _slatable_protocols
 from . import _plannable_protocols as _plannable_protocols
+from ._invariants import Invariant
+from ._variants import Variant
 
+from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import (Self, Any, )
-    from collections.abc import (Iterable, Iterator, )
-    from ..equations import (Equation, )
-    from ..quantities import (Quantity, )
+    from typing import Self, Any
+    from collections.abc import Iterable, Iterator
+    from ..equations import Equation
+    from ..quantities import Quantity
 
 #]
 
@@ -43,25 +44,25 @@ __all__ = (
 )
 
 
-@_pages.reference(
+@_dm.reference(
     path=("structural_models", "sequential.md", ),
     categories={
-        "constructor": "Creating new `Sequential` models",
+        "constructor": "Creating new Sequential models",
         "property": None,
-        "simulation": "Simulating `Sequential` models",
-        "parameters": "Manipulating `Sequential` model parameters",
-        "information": "Information about `Sequential` models",
-        "manipulation": "Manipulating `Sequential` models",
+        "information": "Getting information about Sequential models",
+        "simulation": "Simulating Sequential models",
+        "parameters": "Manipulating Sequential model parameters",
+        "manipulation": "Manipulating Sequential models",
     },
 )
 class Sequential(
     _simulate.Inlay,
     _assigns.Inlay,
     _get.Inlay,
-    _slatable_protocol.Inlay,
+    _slatable_protocols.Inlay,
     _plannable_protocols.Inlay,
     #
-    _has_variants.HasVariantsMixin,
+    _has_variants.Mixin,
 ):
     """
 ......................................................................
@@ -85,14 +86,15 @@ and period-by-period.
 
     def __init__(
         self,
-        /,
+        invariant: Invariant | None = None,
+        variants: Iterable[Variant] | None = None,
     ) -> None:
+        r"""
         """
-        """
-        self._invariant = _invariants.Invariant()
-        self._variants = []
+        self._invariant = Invariant()
+        self._variants = list(variants) if variants else []
 
-    @_pages.reference(category="manipulation", )
+    @_dm.reference(category="manipulation", )
     def copy(self, /, ) -> Self:
         """
 ················································································
@@ -122,19 +124,7 @@ other = self.copy()
         return _co.deepcopy(self, )
 
     @classmethod
-    def skeleton(
-        klass,
-        other,
-        /,
-    ) -> Self:
-        """
-        """
-        self = klass()
-        self._invariant = other._invariant
-        return self
-
-    @classmethod
-    @_pages.reference(category="constructor", call_name="Sequential.from_file", )
+    @_dm.reference(category="constructor", call_name="Sequential.from_file", )
     def from_file(klass, *args, **kwargs, ) -> _sources.SourceMixinProtocol:
         """
 ················································································
@@ -182,7 +172,7 @@ model object.
         return _sources.from_file(klass, *args, **kwargs, )
 
     @classmethod
-    @_pages.reference(category="constructor", call_name="Sequential.from_string",)
+    @_dm.reference(category="constructor", call_name="Sequential.from_string",)
     def from_string(klass, *args, **kwargs, ) -> _sources.SourceMixinProtocol:
         """
 ················································································
@@ -216,7 +206,7 @@ See [`Sequential.from_file`](#sequentialfrom_file) for other input arguments.
 
 ### Returns ###
 
-See [`Sequential.from_file`](sequentialfrom_file) for return values.
+See [`Sequential.from_file`](#sequentialfrom_file) for return values.
 
 ················································································
         """
@@ -233,8 +223,8 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         """
         """
         self = klass()
-        self._invariant = _invariants.Invariant.from_equations(equations, quantities, **kwargs, )
-        initial_variant = _variants.Variant(self._invariant.parameter_names, )
+        self._invariant = Invariant.from_equations(equations, quantities, **kwargs, )
+        initial_variant = Variant(self._invariant.parameter_names, )
         self._variants = [ initial_variant ]
         return self
 
@@ -259,37 +249,37 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
     #
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def all_names(self, /, ) -> tuple[str]:
         """==Names of all variables occurring in the model in order of appearance=="""
         return tuple(self._invariant.all_names)
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def lhs_names(self, /, ) -> tuple[str]:
         """==Unique names of LHS variables in order of their first appearance in equations=="""
         return self._invariant.lhs_names
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def lhs_names_in_equations(self, /, ) -> tuple[str]:
         """==Names of LHS variables in order of their appearance in equations=="""
         return tuple( x.lhs_name for x in self._invariant.explanatories )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def residual_names(self, /, ) -> tuple[str]:
         """==Unique names of residuals in order of their first appearance in  equations=="""
         return self._invariant.residual_names
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def residual_names_in_equations(self, /, ) -> tuple[str]:
         """==Names of residuals in order of their appearance in  equations=="""
         return tuple( x.residual_name for x in self._invariant.explanatories )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def rhs_only_names(self, /, ) -> tuple[str]:
         """==Names of variables appearing only on the RHS of equations=="""
         exclude_names = self._invariant.lhs_names + self._invariant.residual_names + self._invariant.parameter_names
@@ -299,13 +289,13 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def parameter_names(self, /, ) -> tuple[str]:
         """==Names of model parameters=="""
         return tuple(self._invariant.parameter_names)
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def identity_index(self, /, ) -> tuple[int]:
         """==Indexes of identity equations=="""
         return tuple(
@@ -314,7 +304,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def nonidentity_index(self, /, ) -> tuple[int]:
         """==Indexes of nonidentity equations=="""
         return tuple(
@@ -323,7 +313,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def equation_strings(self, /, ) -> tuple[Equation]:
         """==Equation strings in order of appearance=="""
         return tuple(
@@ -332,7 +322,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def lhs_quantities(self, /, ) -> tuple[Quantity]:
         """==LHS quantities in order of appearance=="""
         lhs_names = self._invariant.lhs_names
@@ -345,13 +335,13 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def num_equations(self, /, ) -> int:
         """==Number of equations=="""
         return self._invariant.num_equations
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def num_lhs_names(self, /, ) -> int:
         """==Number of unique LHS names=="""
         return self._invariant.num_lhs_names
@@ -365,7 +355,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def incidence_matrix(self, /, ) -> _np.ndarray:
         """==Incidence matrix with equations in rows and LHS quantities in columns=="""
         num_lhs_names = len(self.lhs_names)
@@ -383,7 +373,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def max_lag(self, /, ) -> int:
         """==Maximum lag occurring on the RHS of equations=="""
         return min(
@@ -396,7 +386,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         return self.max_lag
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def max_lead(self, /, ) -> int:
         """==Maximum lead occurring on the RHS of equations=="""
         return max(
@@ -409,7 +399,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
         return self.max_lead
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def is_sequential(
         self,
         /,
@@ -424,7 +414,7 @@ See [`Sequential.from_file`](sequentialfrom_file) for return values.
     def equations(self, /, ) -> tuple[Equation]:
         return self._invariant.equations
 
-    @_pages.reference(category="manipulation", )
+    @_dm.reference(category="manipulation", )
     def reorder_equations(self, *args, **kwargs, ) -> None:
         """
 ......................................................................
@@ -455,7 +445,7 @@ equation indexes.
         """
         self._invariant.reorder_equations(*args, **kwargs, )
 
-    @_pages.reference(category="manipulation", )
+    @_dm.reference(category="manipulation", )
     def sequentialize(
         self,
         /,
@@ -504,7 +494,7 @@ sequentially. The reordered equation indexes are returned as a tuple.
 
     iter_explanatories = iter_equations
 
-    @_pages.reference(category="information", )
+    @_dm.reference(category="information", )
     def get_description(self, /, ) -> str:
         """
 ················································································
@@ -534,7 +524,7 @@ description = self.get_description()
         """
         return self._invariant.get_description()
 
-    @_pages.reference(category="information", )
+    @_dm.reference(category="information", )
     def set_description(self, *args, **kwargs, ) -> None:
         """
 ················································································
@@ -580,15 +570,13 @@ self.set_description(description, )
         indented = " " * 4
         return "\n".join((
             f"",
-            f"{self.__class__.__name__} model",
-            f"Description: \"{self.get_description()}\"",
-            f"⏐",
+            f"<{self.__class__.__name__} at {id(self):#x}>",
+            f"[Description: \"{self.get_description()}\"]",
             f"[Num variants: {self.num_variants}]",
-            f"[Num equations: {self.num_equations}]",
-            f"[Num (nonidentities, identities): ({len(self.nonidentity_index)}, {len(self.identity_index)})]",
+            f"[Num equations, nonidentities, identities: {self.num_equations}, {len(self.nonidentity_index)}, {len(self.identity_index)}]",
             f"[Num parameters: {len(self.parameter_names)}]",
             f"[Num rhs-only variables: {len(self.rhs_only_names)}]",
-            f"[Max (lag, lead): (-{abs(self.min_shift):g}, +{abs(self.max_shift):g})]",
+            f"[Max lag, lead: t{self.min_shift:+g}, t{self.max_shift:+g}]",
             f"",
         ))
 
