@@ -4,6 +4,7 @@ Frequency conversion of time series
 
 
 #[
+
 from __future__ import annotations
 
 from typing import (Self, Callable, )
@@ -12,18 +13,19 @@ import numpy as _np
 import functools as _ft
 import operator as _op
 import statistics as _st
+import documark as _dm
 
 from .. import dates as _dates
-from .. import pages as _pages
 from . import arip as _arip
-from . import _functionalize
+from ._functionalize import FUNC_STRING
+
 #]
 
 
-__all__ = (
+__all__ = [
     "convert_roc",
     "convert_pct",
-)
+]
 
 
 _builtin_min = min
@@ -32,6 +34,7 @@ _builtin_sum = sum
 
 
 _DEFAULT_METHOD = "mean"
+_DEFAULT_DISCARD_MISSING = False
 
 
 class Inlay:
@@ -39,14 +42,16 @@ class Inlay:
     """
     #[
 
-    @_pages.reference(category="conversion", )
+    @_dm.reference(category="conversion", )
     def aggregate(
         self,
         target_freq: _dates.Frequency,
-        /,
+        #
         method: Literal["mean", "sum", "first", "last", "min", "max"] | Callable | None = None,
-        remove_missing: bool = False,
+        discard_missing: bool | None = None,
         select: list[int] | None = None,
+        # Do not include the following in the docstring
+        remove_missing: bool | None = None, # Legacy
     ) -> None:
         """
 ················································································
@@ -59,20 +64,20 @@ class Inlay:
     new = irispie.aggregate(
         self,
         target_freq,
-        /,
+
         method="mean",
-        remove_missing=False,
+        discard_missing=False,
         select=None,
     )
 
 
-### Class method form for creating new time `Series` objects ###
+### Class method changing an existing Series object in-place ###
 
     self.aggregate(
         target_freq,
-        /,
+
         method="mean",
-        remove_missing=False,
+        discard_missing=False,
         select=None,
     )
 
@@ -97,7 +102,7 @@ class Inlay:
     | "min"     | Minimum of high-frequency values
     | "max"     | Maximum of high-frequency values
 
-???+ input "remove_missing"
+???+ input "discard_missing"
     Remove missing values from the high-frequency data before
     applying the aggregation `method`.
 
@@ -116,6 +121,12 @@ class Inlay:
 
 ················································································
         """
+        # Resolve legacy options
+        if remove_missing is not None and discard_missing is None:
+            discard_missing = remove_missing
+        if discard_missing is None:
+            discard_missing = _DEFAULT_DISCARD_MISSING
+        #
         method = method or _DEFAULT_METHOD
         method_func = (
             _AGGREGATION_METHOD_RESOLUTION[method]
@@ -132,7 +143,7 @@ class Inlay:
         aggregate_within_data_func = _ft.partial(
             _aggregate_within_data,
             select,
-            remove_missing,
+            discard_missing,
             method_func,
         )
         #
@@ -144,11 +155,11 @@ class Inlay:
         new_start_date, new_data = aggregate_func(self, new_dater_class, aggregate_within_data_func, )
         self._replace_start_and_values(new_start_date, new_data, )
 
-    @_pages.reference(category="conversion", )
+    @_dm.reference(category="conversion", )
     def disaggregate(
         self,
         target_freq: _dates.Frequency,
-        /,
+        #
         method: str = "flat",
         **kwargs,
     ) -> Self:
@@ -163,7 +174,7 @@ class Inlay:
     new = irispie.disaggregate(
         self,
         target_freq,
-        /,
+
         method="flat",
     )
 
@@ -172,7 +183,7 @@ class Inlay:
 
     self.disaggregate(
         target_freq,
-        /,
+
         method="flat",
         model=None,
     )
@@ -192,6 +203,7 @@ class Inlay:
     |-----------|-------------
     | "flat"    | Repeat the high-frequency values
     | "first"   | Place the low-frequency value in the first high-frequency period
+    | "middle"  | Place the low-frequency value in the middle high-frequency period
     | "last"    | Place the low-frequency value in the last high-frequency period
     | "arip"    | Interpolate using a smooth autoregressive process
 
@@ -278,9 +290,11 @@ class Inlay:
     #]
 
 
-for n in ("aggregate", "disaggregate", ):
-    exec(_functionalize.FUNC_STRING.format(n=n, ), globals(), locals(), )
-    __all__ += (n, )
+attributes = (n for n in dir(Inlay) if not n.startswith("_"))
+for n in attributes:
+    code = FUNC_STRING.format(n=n, )
+    exec(code, globals(), locals(), )
+    __all__.append(n)
 
 
 def _aggregate_daily_to_regular(
@@ -296,8 +310,8 @@ def _aggregate_daily_to_regular(
     from_until = (start_date, end_date, )
     start_year = self.start_date.get_year()
     end_year = self.end_date.get_year()
-    new_start_date = new_dater_class.from_year_period(start_year, 1)
-    new_end_date = new_dater_class.from_year_period(end_year, "end")
+    new_start_date = new_dater_class.from_year_segment(start_year, 1)
+    new_end_date = new_dater_class.from_year_segment(end_year, "end")
     get_slice_func = lambda t: slice(
         t.to_daily(position="start", ) - start_date,
         t.to_daily(position="end", ) - start_date + 1,
@@ -319,7 +333,6 @@ def _aggregate_regular_to_regular(
     self,
     new_dater_class: type,
     aggregate_within_data_func: Callable,
-    /,
 ) -> tuple[Dater, _np.ndarray]:
     """
     """
@@ -327,7 +340,7 @@ def _aggregate_regular_to_regular(
     start_year = self.start_date.get_year()
     start_date = self.start_date.create_soy()
     end_date = self.end_date.create_eoy()
-    new_start_date = new_dater_class.from_year_period(start_year, 1)
+    new_start_date = new_dater_class.from_year_segment(start_year, 1)
     target_freq = new_dater_class.frequency
     self_data = self.get_data_from_until((start_date, end_date))
     factor = self.frequency.value // target_freq
@@ -349,7 +362,6 @@ def _aggregate_regular_to_regular(
 def _disaggregate_flat(
     self,
     high_dater_class: type,
-    /,
 ) -> tuple[Dater, _np.ndarray]:
     """
     """
@@ -365,39 +377,50 @@ def _disaggregate_flat(
 def _disaggregate_first(
     self,
     high_dater_class: type,
-    /,
 ) -> tuple[Dater, _np.ndarray]:
     """
     """
     #[
-    high_start_date, high_data, factor = _disaggregate_flat(self, high_dater_class, )
-    for i in range(1, factor, ):
-        high_data[i::factor, :] = _np.nan
-    return high_start_date, high_data
+    high_start_date, flat_high_data, factor = _disaggregate_flat(self, high_dater_class, )
+    high_data = _np.full(flat_high_data.shape, None, dtype=self.data_type, )
+    high_data[::factor, :] = flat_high_data[::factor, :]
+    return high_start_date, high_data, factor
+    #]
+
+
+def _disaggregate_middle(
+    self,
+    high_dater_class: type,
+) -> tuple[Dater, _np.ndarray]:
+    """
+    """
+    #[
+    high_start_date, flat_high_data, factor = _disaggregate_flat(self, high_dater_class, )
+    high_data = _np.full(flat_high_data.shape, None, dtype=self.data_type, )
+    high_data[factor//2::factor, :] = flat_high_data[factor//2::factor, :]
+    return high_start_date, high_data, factor,
     #]
 
 
 def _disaggregate_last(
     self,
     high_dater_class: type,
-    /,
 ) -> tuple[Dater, _np.ndarray]:
     """
     """
     #[
-    high_start_date, high_data, factor = _disaggregate_flat(self, high_dater_class, )
-    for i in range(0, factor-1, ):
-        high_data[i::factor, :] = _np.nan
+    high_start_date, flat_high_data, factor = _disaggregate_flat(self, high_dater_class, )
+    high_data = _np.full(flat_high_data.shape, None, dtype=self.data_type, )
+    high_data[factor-1::factor, :] = flat_high_data[factor-1::factor, :]
     return high_start_date, high_data, factor
     #]
 
 
 def _aggregate_within_data(
     select: list[int] | None,
-    remove_missing: bool,
+    discard_missing: bool,
     method_func: Callable,
     within_data: _np.ndarray,
-    /,
 ) -> _np.ndarray:
     """
     """
@@ -405,7 +428,7 @@ def _aggregate_within_data(
     if select is not None:
         select = tuple(select)
         within_data = within_data[select]
-    if remove_missing:
+    if discard_missing:
         within_data = within_data[~_np.isnan(within_data)]
     return method_func(within_data) if within_data.size > 0 else _np.nan
     #]
@@ -426,6 +449,7 @@ _AGGREGATION_METHOD_RESOLUTION = {
 _CHOOSE_DISAGGREGATION_METHOD = {
     "flat": _disaggregate_flat,
     "first": _disaggregate_first,
+    "middle": _disaggregate_middle,
     "last": _disaggregate_last,
     "arip": _arip.disaggregate_arip,
 }
@@ -435,7 +459,6 @@ def convert_roc(
     roc: Real,
     from_freq: _dates.Frequency,
     to_freq: _dates.Frequency,
-    /,
 ) -> Real:
     """
     """
@@ -446,7 +469,6 @@ def convert_pct(
     pct: Real,
     from_freq: _dates.Frequency,
     to_freq: _dates.Frequency,
-    /,
 ) -> Real:
     """
     """
@@ -457,7 +479,6 @@ def convert_diff(
     diff: Real,
     from_freq: _dates.Frequency,
     to_freq: _dates.Frequency,
-    /,
 ) -> Real:
     """
     """
