@@ -3,25 +3,31 @@
 
 
 #[
+
 from __future__ import annotations
 
 import functools as _ft
 import numpy as _np
+import documark as _dm
+from typing import Literal
 
-from .. import pages as _pages
-from . import _functionalize
+from ._functionalize import FUNC_STRING
 
 from typing import (TYPE_CHECKING, )
 if TYPE_CHECKING:
-    from typing import (Any, Literal, Callable, EllipsisType, )
-    from collections.abc import (Iterable, )
-    from numbers import (Real, )
-    from ..dates import (Period, )
-    from ..series import (Series, )
+    from typing import Any, Callable, EllipsisType
+    from collections.abc import Iterable
+    from numbers import Real
+    from ..dates import Period
+    from ..series import Series
+
 #]
 
 
-__all__ = ()
+__all__ = []
+
+
+MethodType = Literal["next", "previous", "nearest", "linear", "log_linear", "constant", "from_series", ]
 
 
 class Inlay:
@@ -29,10 +35,10 @@ class Inlay:
     """
     #[
 
-    @_pages.reference(category="homogenizing", )
+    @_dm.reference(category="homogenizing", )
     def fill_missing(
         self,
-        method: Literal["next", "previous", "nearest", "linear", "log_linear", "constant", "series", ],
+        method: Literal["next", "previous", "nearest", "linear", "log_linear", "constant", "from_series", ],
         method_args: Any | None = None,
         span: Iterable[Period] | EllipsisType | None = None,
     ) -> None:
@@ -52,7 +58,7 @@ class Inlay:
     )
 
 
-### Class method form for changing existing time `Series` objects in-place ###
+### Class method form changing existing `Series` objects in-place ###
 
     self.fill_missing(
         method,
@@ -70,23 +76,23 @@ class Inlay:
 ???+ input "method"
     The method to be used for filling missing observations. The following methods are available:
 
-    | Method        | Description
-    |---------------|-------------
-    | "constant"    | Fill with a constant value
-    | "next"        | Next available observation
-    | "previous"    | Previous available observation
-    | "nearest"     | Nearest available observation
-    | "linear"      | Linear interpolation or extrapolation
-    | "log_linear"  | Log-linear interpolation or extrapolation
-    | "series"      | Fill with values from another time series object
+    | Method         | Description
+    |----------------|-------------
+    | "constant"     | Fill with a constant value
+    | "next"         | Next available observation
+    | "previous"     | Previous available observation
+    | "nearest"      | Nearest available observation
+    | "linear"       | Linear interpolation or extrapolation
+    | "log_linear"   | Log-linear interpolation or extrapolation
+    | "from_series"  | Fill with values from another time series object
 
 ???+ input "*args"
     Additional arguments to be passed to the filling method. The following methods require additional arguments:
 
-    | Method     | Additional argument(s)
-    |------------|-----------------------
-    | "constant" | A single constant value
-    | "series"   | A time `Series` object
+    | Method         | Additional argument(s)
+    |----------------|-----------------------
+    | "constant"     | A single constant value
+    | "from_series"  | A time `Series` object
 
 
 ???+ input "span"
@@ -115,9 +121,11 @@ class Inlay:
     #]
 
 
-for n in ("fill_missing", ):
-    exec(_functionalize.FUNC_STRING.format(n=n, ), globals(), locals(), )
-    __all__ += (n, )
+attributes = (n for n in dir(Inlay) if not n.startswith("_"))
+for n in attributes:
+    code = FUNC_STRING.format(n=n, )
+    exec(code, globals(), locals(), )
+    __all__.append(n)
 
 
 def _fill_neighbor(
@@ -218,7 +226,7 @@ def _fill_constant(
     return data
 
 
-def _fill_series(
+def fill_from_series(
     values: _np.ndarray,
     method_args: Series,
     span: Iterable[Period],
@@ -277,7 +285,10 @@ _FILL_METHOD_DISPATCH = {
     "linear": _ft.partial(_fill_interp, func=_interpolation_linear, ),
     "log_linear": _ft.partial(_fill_interp, func=_interpolation_log_linear, ),
     "constant": _fill_constant,
-    "series": _fill_series,
+    "from_series": fill_from_series,
+    #
+    # Aliases for backward compatibility
+    "series": fill_from_series,
 }
 
 
