@@ -4,6 +4,7 @@ Univariate time series filters
 
 
 #[
+
 from __future__ import annotations
 
 from numbers import (Real, )
@@ -11,16 +12,17 @@ from collections.abc import (Iterable, Callable, )
 from types import (EllipsisType, )
 from typing import (Self, )
 import numpy as _np
+import documark as _dm
 
-from .. import pages as _pages
 from ..dates import (Period, Frequency, )
 from .. import dates as _dates
 from . import main as _series
-from . import _functionalize
+from ._functionalize import FUNC_STRING
+
 #]
 
 
-__all__ = ("hpf", )
+__all__ = ["hpf", ]
 
 
 _AUTO_SMOOTH = {
@@ -163,7 +165,7 @@ class Inlay:
     """
     #[
 
-    @_pages.reference(
+    @_dm.reference(
         category="filtering",
     )
     def hpf(self, /, ):
@@ -173,7 +175,7 @@ class Inlay:
 ==Constrained Hodrick-Prescott filter==
 
 
-### Functional forms creating a new time `Series` object ###
+### Functional form creating a new time `Series` object ###
 
 
     trend, gap = irispie.hpf(
@@ -187,7 +189,7 @@ class Inlay:
     )
 
 
-### Class methods changing an existing time `Series` object in-place ###
+### Class method changing an existing time `Series` objects in-place ###
 
 
     self.hpf_trend(
@@ -372,7 +374,7 @@ def _data_hpf(
     Hodrick-Prescott filter run on a multi-variant data matrix
     """
     #[
-    span = self._resolve_dates(span, )
+    span = self.resolve_periods(span, )
     encompassing_span, *from_until = _dates.get_encompassing_span(self, level, change, span, )
     num_periods = len(encompassing_span, )
     level_data, level_where = _prepare_constraints(level, from_until, )
@@ -431,8 +433,8 @@ def hpf(self, *args, **kwargs, ) -> tuple[_series.Series, _series.Series]:
 
 
 for n in ("hpf_trend", "hpf_gap", ):
-    exec(_functionalize.FUNC_STRING.format(n=n, ), globals(), locals(), )
-    __all__ += (n, )
+    exec(FUNC_STRING.format(n=n, ), globals(), locals(), )
+    __all__.append(n)
 
 
 
