@@ -14,14 +14,14 @@ import platform as _pf
 import tempfile as _tf
 import copy as _co
 import glob as _gl
+import documark as _dm
 
 from .. import executables as _executables
 from .. import wrongdoings as _wrongdoings
 from .. import has_variants as _has_variants
-from .. import pages as _pages
 from ..dates import (Period, Span, Frequency, )
 
-from . import _functionalize
+from ._functionalize import FUNC_STRING
 
 if TYPE_CHECKING:
     from typing import (Any, )
@@ -30,7 +30,7 @@ if TYPE_CHECKING:
 #]
 
 
-__all__ = ()
+__all__ = []
 
 
 Mode = Literal["mult", "add", "pseudoadd", "logadd", ]
@@ -41,7 +41,7 @@ class Inlay:
     """
     #[
 
-    @_pages.reference(category="filtering", )
+    @_dm.reference(category="filtering", )
     def x13(
         self,
         *,
@@ -96,7 +96,7 @@ new, info = irispie.x13(
 ```
 
 
-### Class methods for changing existing Series objects in-place ###
+### Class method for changing existing time `Series` objects in-place ###
 
 
 ```
@@ -257,7 +257,7 @@ info = self.x13(
 
 ................................................................................
         """
-        span = tuple(self._resolve_dates(span))
+        span = self.resolve_periods(span)
         base_start, base_end = span[0], span[-1]
         self.clip(base_start, base_end, )
         #
@@ -314,9 +314,11 @@ info = self.x13(
     #]
 
 
-for n in ("x13", ):
-    exec(_functionalize.FUNC_STRING.format(n=n, ), globals(), locals(), )
-    __all__ += (n, )
+attributes = (n for n in dir(Inlay) if not n.startswith("_"))
+for n in attributes:
+    code = FUNC_STRING.format(n=n, )
+    exec(code, globals(), locals(), )
+    __all__.append(n)
 
 
 def _x13_data(
@@ -334,8 +336,8 @@ def _x13_data(
     #[
     new_data = _np.full(base_variant_data.shape, _np.nan, dtype=float, )
     start_date, variant_data, output_slice = _remove_leading_trailing_nans(base_start, base_variant_data, )
-    year, period = start_date.to_year_period()
-    specs = specs.replace("$(series_start)", _X13_DATE_FORMAT_RESOLUTION[start_date.frequency].format(year=year, period=period, ), )
+    year, segment = start_date.to_year_segment()
+    specs = specs.replace("$(series_start)", _X13_DATE_FORMAT_RESOLUTION[start_date.frequency].format(year=year, segment=segment, ), )
     specs = specs.replace("$(series_data)", _print_series_data(variant_data, ), )
     specs_file_name_without_ext = _write_specs_to_file(specs, )
     system_output = _execute(specs_file_name_without_ext, )
@@ -632,8 +634,8 @@ _X11_OUTPUT_RESOLUTION = {
 }
 
 _X13_DATE_FORMAT_RESOLUTION = {
-    Frequency.MONTHLY: "{year:04d}.{period:02d}",
-    Frequency.QUARTERLY: "{year:04d}.{period:1d}",
+    Frequency.MONTHLY: "{year:04d}.{segment:02d}",
+    Frequency.QUARTERLY: "{year:04d}.{segment:1d}",
 }
 
 _X13_EXECUTABLE_FILE = {
