@@ -4,19 +4,16 @@ Main time series class definition
 
 
 #[
+
 from __future__ import annotations
 
-from numbers import (Real, )
-from collections.abc import (Iterable, Callable, )
-from typing import (Self, Any, TypeAlias, NoReturn, )
-from types import (EllipsisType, )
+from numbers import Real
+from collections.abc import Iterable, Callable
+from typing import Self, Any, TypeAlias, NoReturn, Literal
+from types import EllipsisType
 import numpy as _np
-import scipy as _sp
-import itertools as _it
-import functools as _ft
 import operator as _op
-import copy as _cp
-import textwrap as _tw
+import documark as _dm
 
 from ..conveniences import descriptions as _descriptions
 from ..conveniences import copies as _copies
@@ -25,97 +22,53 @@ from ..dates import (Period, Span, Frequency, EmptyRanger, )
 from .. import dates as _dates
 from .. import wrongdoings as _wrongdoings
 from .. import has_variants as _has_variants
-from .. import pages as _pages
 
-from . import _temporal
-from . import _filling
-from . import _moving
-from . import _conversions
-from . import _hp
-from . import _ar
-from . import _x13
+from ._functionalize import FUNC_STRING
 
 from . import _indexing
 from . import _plotly
 from . import _views
-from . import _functionalize
+from . import _temporal
+from . import _filling
+from . import _conversions
+from . import _hp
+from . import _extrapolate
+from . import _x13
+from . import _moving
+from . import _elementwise
+from . import _statistics
 
-from ._temporal import __all__ as _diffs_cums__all__
-from ._temporal import *
+from ._categories import CATEGORIES
 
-from ._filling import __all__ as _fillings__all__
-from ._filling import *
-
-from ._hp import __all__ as _hp__all__
-from ._hp import *
-
-from ._ar import __all__ as _ar__all__
-from ._ar import *
-
-from ._x13 import __all__ as _x13__all__
-from ._x13 import *
-
-from ._moving import __all__ as _moving__all__
-from ._moving import *
-
-from ._conversions import __all__ as _conversions__all__
-from ._conversions import *
 #]
 
 
-_ELEMENTWISE_FUNCTIONS = {
-    "log": _np.log,
-    "exp": _np.exp,
-    "sqrt": _np.sqrt,
-    "abs": _np.abs,
-    "sign": _np.sign,
-    "sin": _np.sin,
-    "cos": _np.cos,
-    "tan": _np.tan,
-    "round": _np.round,
-    "logistic": _sp.special.expit,
-    "normal_cdf": _sp.stats.norm.cdf,
-    "normal_pdf": _sp.stats.norm.pdf,
-}
-
-FUNCTION_ADAPTATIONS_TIMEWISE_NUMPY = (
-    "sum",
-    "mean",
-)
-
-
-# FIXME
-FUNCTION_ADAPTATIONS_ELEMENTWISE = tuple(_ELEMENTWISE_FUNCTIONS.keys())
-
-FUNCTION_ADAPTATIONS_NUMPY_APPLY = () # ("maximum", "minimum", "mean", "median", "abs", )
-FUNCTION_ADAPTATIONS_BUILTINS = ("max", "min", )
-FUNCTION_ADAPTATIONS = tuple(set(
-    FUNCTION_ADAPTATIONS_ELEMENTWISE
-))
+#
+# The following functions can return either a new time series object if applied
+# across variants (axis=1), or a scalar or a list of values if applied across
+# time (axis=0)
+#
 
 
 __all__ = (
-    ("Series", "shift", "series", )
-    + _conversions__all__
-    + _diffs_cums__all__
-    + _fillings__all__
-    + _hp__all__
-    + _ar__all__
-    + _moving__all__
-    + _x13__all__
-    + FUNCTION_ADAPTATIONS
+    "Series",
+    "shift",
 )
 
-Dates: TypeAlias = Period | Iterable[Period] | Span | EllipsisType | None
-VariantsRequestType: TypeAlias = int | Iterable[int] | slice | None
+
+Dates = Period | Iterable[Period] | Span | EllipsisType | None
+VariantsRequestType = int | Iterable[int] | slice | None
+LayMethodType = Literal["by_span", ]
+ShiftType = int | Literal["yoy", "soy", "eopy", "tty", ]
+AxisType = Literal[0, 1]
 
 
 def _get_date_positions(dates, base, num_periods, /, ):
-    pos = tuple(_dates.date_index(dates, base))
-    min_pos = _builtin_min((x for x in pos if x is not None), default=0)
-    max_pos = _builtin_max((x for x in pos if x is not None), default=0)
-    add_before = _builtin_max(-min_pos, 0)
-    add_after = _builtin_max(max_pos - num_periods + 1, 0)
+    pos = tuple(_dates.period_indexes(dates, base, ))
+    min_pos = min((x for x in pos if x is not None), default=0)
+    max_pos = max((x for x in pos if x is not None), default=0)
+    add_before = max(-min_pos, 0)
+    add_after = max(max_pos - num_periods + 1, 0)
     pos_adjusted = [
         p + add_before if p is not None else None
         for p in pos
@@ -123,18 +76,9 @@ def _get_date_positions(dates, base, num_periods, /, ):
     return pos_adjusted, add_before, add_after
 
 
-@_pages.reference(
+@_dm.reference(
     path=("data_management", "time_series.md", ),
-    categories={
-        "constructor": "Constructing new time series",
-        "conversion": "Converting time series frequency",
-        "manipulation": "Manipulating time series values",
-        "homogenizing": "Homogenizing and extrapolating time series",
-        "filtering": "Filtering time series",
-        "moving": "Applying moving window functions",
-        "temporal_change": "Calculating temporal change",
-        "temporal_cumulation": "Calculating temporal cumulation",
-    },
+    categories=CATEGORIES,
 )
 class Series(
     _indexing.Inlay,
@@ -142,11 +86,13 @@ class Series(
     _temporal.Inlay,
     _filling.Inlay,
     _hp.Inlay,
-    _ar.Inlay,
+    _extrapolate.Inlay,
     _moving.Inlay,
     _x13.Inlay,
     _plotly.Inlay,
     _views.Inlay,
+    _elementwise.Inlay,
+    _statistics.Inlay,
     #
     _descriptions.DescriptionMixin,
     _copies.Mixin,
@@ -171,7 +117,7 @@ variants of the data, stored as mutliple columns.
         "data",
         "data_type",
         "metadata",
-        "_description",
+        "__description__",
     )
 
     _numeric_format: str = "15g"
@@ -181,7 +127,7 @@ variants of the data, stored as mutliple columns.
     _missing_str: str = "⋅"
     _test_missing_period = staticmethod(lambda x: _np.all(_np.isnan(x)))
 
-    @_pages.reference(
+    @_dm.reference(
         category="constructor",
         call_name="Series",
     )
@@ -220,29 +166,37 @@ self = Series(
 )
 ```
 
+```
+self = Series(
+    periods=periods,
+    func=func,
+)
+```
+
 
 ### Input arguments ###
 
-
 ???+ input "start"
-
     The time [`Period`](periods.md) of the first value in the `values`.
 
 ???+ input "periods"
-
     An iterable of time [`Periods`](periods.md) that will be used to time stamp
     the `values`. The iterable can be e.g. a tuple, a list, a time
     [`Span`](spans.md), or a single time [`Period`](periods.md).
 
 ???+ input "values"
-
     Time series values, supplied either as a single values, a tuple of values,
     or a NumPy array.
 
+???+ input "func"
+    A function that will be used to populate the time series; the function
+    should not take any input arguments, and should return a single (scalar)
+    numerical value; the function will called once for each period and each
+    variant.
+
 
 ### Returns ###
 
-
 ???+ returns "None"
     This method modifies `self` in-place and does not return a value.
 
@@ -253,7 +207,7 @@ self = Series(
         self.data_type = data_type
         self.data = _np.full((0, num_variants), _np.nan, dtype=self.data_type, )
         self.metadata = {}
-        self._description = description
+        self.__description__ = description
         #
         start = start_date if start is None else start
         periods = dates if periods is None else periods
@@ -270,6 +224,22 @@ self = Series(
                 func=func,
             )
 
+    @classmethod
+    def as_empty(
+        klass,
+        num_variants: int = 1,
+        data_type: type = _np.float64,
+        description: str = "",
+    ) -> Self:
+        r"""
+        """
+        return klass(
+            num_variants=num_variants,
+            data_type=data_type,
+            description=description,
+            populate=False,
+        )
+
     @classmethod
     def _guaranteed(
         klass,
@@ -282,7 +252,7 @@ self = Series(
         self = klass()
         self.start = start
         self.data = values
-        self._description = description
+        self.__description__ = description
         self.trim()
         return self
 
@@ -302,19 +272,19 @@ self = Series(
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def shape(self, /, ) -> tuple[int, int]:
         """==Shape of time series data=="""
         return self.data.shape
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def num_periods(self, /, ) -> int:
         """==Number of periods from the first to the last observation=="""
         return self.data.shape[0]
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def num_variants(self, /, ) -> int:
         """==Number of variants (columns) within the `Series` object=="""
         return self.data.shape[1]
@@ -327,7 +297,7 @@ self = Series(
         return _has_variants.is_singleton(self.num_variants, )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def span(self, ):
         """==Time span of the time series=="""
         return Span(self.start, self.end, ) if self.start else ()
@@ -335,13 +305,13 @@ self = Series(
     range = span
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def from_until(self, ):
         """==Two-tuple with the start date and end date of the time series=="""
         return self.start, self.end
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def periods(self, /, ) -> tuple[Period, ...]:
         """==N-tuple with the periods from the start period to the end period of the time series=="""
         return tuple(self.range, )
@@ -349,7 +319,7 @@ self = Series(
     dates = periods
 
     @property
-    @_pages.reference(category="property", call_name="start", )
+    @_dm.reference(category="property", call_name="start", )
     def _start(self):
         """==Start date of the time series=="""
         raise NotImplementedError
@@ -361,7 +331,7 @@ self = Series(
     start_date = start_period
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def end(self):
         """==End period of the time series=="""
         return (
@@ -373,7 +343,7 @@ self = Series(
     end_date = end
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def frequency(self, /, ):
         """==Date frequency of the time series=="""
         return (
@@ -383,13 +353,13 @@ self = Series(
         )
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def is_empty(self, ) -> bool:
         """==True if the time series is empty=="""
         return not self.data.size
 
     @property
-    @_pages.reference(category="property", )
+    @_dm.reference(category="property", )
     def has_missing(self, /, ):
         """==True if the time series is non-empty and contains in-sample missing values=="""
         return (not self.is_empty) and _np.isnan(self.data).any()
@@ -424,7 +394,7 @@ self = Series(
     ) -> None:
         """
         """
-        dates = self._resolve_dates(dates, )
+        dates = self.resolve_periods(dates, )
         is_data_ndarray = isinstance(data, _np.ndarray)
         is_empty_data = (
             data is None
@@ -473,7 +443,7 @@ self = Series(
     ) -> tuple[Iterable[Period], Iterable[int], Iterable[int], _np.ndarray]:
         """
         """
-        dates = [ t for t in self._resolve_dates(dates, ) ]
+        dates = self.resolve_periods(dates, )
         variants = self._resolve_variants(variants)
         if not dates:
             dates = ()
@@ -481,7 +451,7 @@ self = Series(
             data = self._create_periods_of_missing_values(num_rows=0, )[:, variants]
             return dates, pos, variants, data
         #
-        base_date = self.start or _builtin_min(dates, )
+        base_date = self.start or min(dates, )
         pos, add_before, add_after = _get_date_positions(dates, base_date, self.shape[0], )
         data = self._create_expanded_data(add_before, add_after, )
         if not isinstance(pos, Iterable):
@@ -531,13 +501,13 @@ self = Series(
         *args,
         unpack_singleton: bool = True,
         **kwargs,
-    ) -> _np.ndarray:
+    ) -> list[tuple[Real, ...]] | tuple[Real, ...]:
         """
         """
         data = self.get_data(*args, **kwargs, )
         values = [ tuple(data_column.tolist()) for data_column in data.T ]
         return _has_variants.unpack_singleton(
-            values, self.is_singleton,
+            values,
             unpack_singleton=unpack_singleton,
         )
 
@@ -605,23 +575,23 @@ self = Series(
         self.start = new_start
         return self
 
-    def _resolve_dates(self, dates, ) -> tuple[Period]:
+    def resolve_periods(self, periods, ) -> tuple[Period]:
         """
         """
-        if dates is None:
-            # dates = []
-            dates = ...
-        if isinstance(dates, slice) and dates == slice(None, ):
-            dates = ...
-        if dates is ... and self.start is not None:
-            dates = Span(None, None, )
-        if dates is ... and self.start is None:
-            dates = EmptyRanger()
-        if hasattr(dates, "needs_resolve") and dates.needs_resolve:
-            dates = dates.resolve(self, )
+        if periods is None:
+            # periods = []
+            periods = ...
+        if isinstance(periods, slice) and periods == slice(None, ):
+            periods = ...
+        if periods is ... and self.start is not None:
+            periods = Span(None, None, )
+        if periods is ... and self.start is None:
+            periods = EmptyRanger()
+        if hasattr(periods, "needs_resolve") and periods.needs_resolve:
+            periods = periods.resolve(self, )
         return tuple(
             d.resolve(self) if hasattr(d, "needs_resolve") and d.needs_resolve else d
-            for d in dates
+            for d in periods
         )
 
     def _resolve_variants(
@@ -642,51 +612,95 @@ self = Series(
 
     def shift(
         self,
-        by: int | str = -1,
-        /,
+        by: ShiftType = -1,
+        **kwargs,
     ) -> None:
-        match by:
-            case "yoy":
-                self._shift_by_number(-self.frequency.value, )
-            case "boy" | "soy":
-                self._shift_to_soy()
-            case "eopy":
-                self._shift_to_eopy()
-            case "tty":
-                self._shift_to_tty()
-            case _:
-                self._shift_by_number(by, )
+        r"""
+................................................................................
 
-    def _shift_by_number(self, by: int, ) -> None:
+==Shift the time series start date==
+
+Shift the start date of the time series by a number of periods or to a specific
+date.
+
+    self.shift(by=-1, )
+
+### Input arguments ###
+
+???+ input "self"
+    The current time series object that will be shifted.
+
+???+ input "by"
+    The number of periods to shift the observations by. If `by` is a string,
+    the observations are manipulated as follows:
+
+    * `"yoy"`: Shift all observations by one year back.
+
+    * `"soy"`: Shift to the start of the year.
+
+    * `"eopy"`: Shift to the end of the previous year.
+
+................................................................................
         """
+        if isinstance(by, int):
+            self._shift_by_number(by, **kwargs, )
+        else:
+            method_name = f"_shift_{by}"
+            getattr(self, method_name)(**kwargs, )
+
+    def _shift_by_number(self, by: int, **kwargs, ) -> None:
+        r"""
         Shift (lag, lead) start date by a number of periods
         """
-        self.start = (
-            self.start - by
-            if self.start else self.start
-        )
+        if self.start is None:
+            return
+        self.start -= by
 
-    def _shift_to_soy(self, ) -> None:
+    def _shift_yoy(self, /, **kwargs, ) -> None:
+        r"""
+        Shift the start date by one year back
+        """
+        self._shift_by_number(-self.frequency.value, )
+
+    def _shift_soy(self, /, **kwargs, ) -> None:
+        r"""
+        Replace each observation by the start of the year observation
+        """
         new_data = self.get_data(
             t.create_soy()
             for t in self.range
         )
         self._replace_data(new_data, )
 
-    def _shift_to_tty(self, ) -> None:
-        new_data = self.get_data(
-            t.create_tty()
-            for t in self.range
-        )
-        self._replace_data(new_data, )
-
-    def _shift_to_eopy(self, ) -> Self:
+    def _shift_eopy(self, /, **kwargs, ) -> Self:
+        r"""
+        Replace each observation by the end of the previous year observation
+        """
         new_data = self.get_data(
             t.create_eopy()
             for t in self.range
         )
         self._replace_data(new_data, )
 
+    def _shift_tty(
+        self,
+        by: Any | None = None,
+        *,
+        neutral_value: int | None = None,
+        **kwargs,
+    ) -> None:
+        r"""
+        Replace each observation by the start of the year observation except for
+        the start-of-year periods which are filled with a neutral value.
+        """
+        zipped = tuple((t, t.create_tty(), ) for t in self.span)
+        neutral_periods = tuple(t for t, tty in zipped if tty is None)
+        periods = (t for t, tty in zipped if tty is not None )
+        dty_periods = (tty for _, tty in zipped if tty is not None)
+        dty_values = self.get_data(dty_periods, )
+        self.set_data(periods, dty_values, )
+        self.set_data(neutral_periods, neutral_value, )
+
     def hstack(self, *args, ):
         """
         """
@@ -709,7 +723,7 @@ self = Series(
         new.set_data(encompassing_span, new_data, )
         return new
 
-    @_pages.reference(category="homogenizing", )
+    @_dm.reference(category="homogenizing", )
     def clip(
         self,
         /,
@@ -762,46 +776,146 @@ self = Series(
         num_variants = num_variants if num_variants is not None else self.num_variants
         self.data = _np.empty((0, num_variants), dtype=self.data_type)
 
-    def overlay_by_range(
+    @_dm.no_reference
+    def overlay_by_span(
         self,
         other: Self,
         /,
-    ) -> Self:
+    ) -> None:
         self.set_data(other.range, other.data, )
         self.trim()
 
+    @_dm.reference(category="multiple", )
     def overlay(
         self,
         other: Self,
         /,
-        method = "by_range",
-    ) -> Self:
-        _broadcast_variants_if_needed(self, other, )
-        self._LAY_METHOD_RESOLUTION[method]["overlay"](self, other, )
+        method: LayMethodType = "by_span",
+    ) -> None:
+        r"""
+................................................................................
 
-    def underlay_by_range(
+==Overlay another time series values onto the current time series==
+
+Overlay the values of another time series onto the current time series on the
+entire span of the other time series, i.e. from the start to the end period
+regardless of missing in-sample values.
+
+    self.overlay(
+        other,
+        method="by_span",
+    )
+
+### Input arguments ###
+
+???+ input "self"
+    The current time series object.
+
+???+ input "other"
+    The time series object whose values will be overlaid onto the current time
+    series.
+
+???+ input "method"
+    The method to use for overlaying the values. The default (and currently the
+    only available) method is `"by_span"`.
+
+### Returns ###
+
+This method modifies `self` in place and returns `None`.
+
+### Details ###
+
+???+ abstract "Algorithm"
+
+    The resulting time series is determined the following way:
+
+    1. The span of the resulting series starts at the earliest start period of the two
+    series and ends at the latest end period of the two series.
+
+    2. The observations from the `self` (current) time series used to fill the
+    resulting time span.
+
+    3. Within the span of the `other` time series (from the first available
+    observation to the last available observation), the observations from this
+    `other` time series are superimposed on the resulting time series, including any
+    in-sample missing observations.
+
+................................................................................
+"""
+        _broadcast_variants_if_needed(self, other, )
+        getattr(type(self), "overlay_" + method)(self, other, )
+
+    @_dm.no_reference
+    def underlay_by_span(
         self,
         other: Self,
         /,
-    ) -> Self:
-        """
+    ) -> None:
+        r"""
         """
         new_self = other.copy()
         new_self.overlay(self, )
         self._shallow_copy_data(new_self, )
 
+    @_dm.reference(category="multiple", )
     def underlay(
         self,
         other: Self,
         /,
-        method = "by_range",
-    ) -> Self:
-        _broadcast_variants_if_needed(self, other, )
-        self._LAY_METHOD_RESOLUTION[method]["underlay"](self, other, )
+        method: LayMethodType = "by_span",
+    ) -> None:
+        r"""
+................................................................................
 
-    _LAY_METHOD_RESOLUTION = {
-        "by_range": {"overlay": overlay_by_range, "underlay": underlay_by_range},
-    }
+==Underlay another time series values beneath the current time series==
+
+Underlay the values of another time series beneath the current time series on
+the entire span of the other time series, i.e. from the start to the end period
+regardless of missing in-sample values.
+
+    self.underlay(
+        other,
+        method="by_span",
+    )
+
+### Input arguments ###
+
+???+ input "self"
+    The current time series object.
+
+???+ input "other"
+    The time series object whose values will be underlaid beneath the current
+    time series.
+
+???+ input "method"
+    The method to use for underlaying the values. The default (and currently the
+    only available) method is `"by_span"`.
+
+### Returns ###
+
+This method modifies `self` in place and returns `None`.
+
+### Details ###
+
+???+ abstract "Algorithm"
+
+    The resulting time series is determined the following way:
+
+    1. The span of the resulting series starts at the earliest start period of the two
+    series and ends at the latest end period of the two series.
+
+    2. The observations from the `other` time series used to fill the
+    resulting time span.
+
+    3. Within the span of the `self` time series (from the first available
+    observation to the last available observation), the observations from this
+    `self` time series are superimposed on the resulting time series, including any
+    in-sample missing observations.
+
+................................................................................
+        """
+        _broadcast_variants_if_needed(self, other, )
+        getattr(type(self), "underlay_" + method)(self, other, )
 
     def redate(
         self,
@@ -815,7 +929,7 @@ self = Series(
         else:
             self.start = new_date - (old_date - self.start)
 
-    @_pages.reference(category="manipulation", )
+    @_dm.reference(category="manipulation", )
     def replace_where(
         self,
         test: Callable,
@@ -879,7 +993,28 @@ self.replace_where(
 
     __or__ = __and__
 
-    def trim(self):
+    @_dm.reference(category="manipulation", )
+    def trim(self, /, ) -> None:
+        r"""
+................................................................................
+
+==Trim time series data==
+
+Trim leading and trailing missing values from the time series data.
+
+    self.trim()
+
+### Input arguments ###
+
+???+ input "self"
+    The time series object to trim.
+
+### Returns ###
+
+This method modifies `self` in place and returns `None`.
+
+................................................................................
+        """
         if self.data.size == 0:
             self.reset()
             return self
@@ -1016,6 +1151,29 @@ self.replace_where(
         """
         return self.apply(lambda data: data.__rmod__(other))
 
+    def __abs__(self, *args, **kwargs, ) -> Self:
+        """
+        abs(self)
+        """
+        new = self.copy()
+        new.abs(*args, **kwargs, )
+        return new
+
+    def abs(self, *args, **kwargs, ) -> None:
+        r"""
+        """
+        self.data = _np.abs(self.data, *args, **kwargs, )
+
+    def __round__(self, *args, **kwargs, ) -> Self:
+        r"""
+        """
+        new = self.copy()
+        new.round(*args, **kwargs, )
+        return new
+
+    def round(self, *args, **kwargs, ) -> None:
+        self.data = _np.round(self.data, *args, **kwargs, )
+
     for n in ["gt", "lt", "ge", "le", "eq", "ne", ]:
         exec(f"def __{n}__(self, other): return self._binop(other, _op.{n}, )", )
 
@@ -1084,7 +1242,7 @@ self.replace_where(
         self.start = new_start
         self._replace_data(new_values, )
 
-    def iter_dates_values(self, /, unpack_singleton=True, ):
+    def iter_dates_values(self, unpack_singleton=True, ):
         """
         Default iterator is line by line, yielding a tuple of (date, values)
         """
@@ -1100,7 +1258,7 @@ self.replace_where(
         for date, data_row in zip(self.range, self.data, ):
             yield date, data_row_func(data_row.tolist(), )
 
-    def iter_variants(self, /, ) -> Iterator[Self]:
+    def iter_variants(self, ) -> Iterator[Self]:
         """
         """
         for data in self.iter_data_variants_from_until(..., ):
@@ -1108,14 +1266,17 @@ self.replace_where(
             new._replace_data(data, )
             yield new
 
-    def iter_own_data_variants_from_until(self, from_until, /, ) -> Iterator[_np.ndarray]:
+    def iter_own_data_variants_from_until(self, from_until, ) -> Iterator[_np.ndarray]:
         """
         Iterates over the data variants from the given start date to the given end date
         """
-        data_from_until = self.data if from_until == ... else self.get_data_from_until(from_until, )
+        if from_until == ...:
+            data_from_until = self.data
+        else:
+            data_from_until = self.get_data_from_until(from_until, )
         return iter(data_from_until.T, )
 
-    def iter_data_variants_from_until(self, from_until, /, ) -> Iterator[_np.ndarray]:
+    def iter_data_variants_from_until(self, from_until, ) -> Iterator[_np.ndarray]:
         """
         Iterates over the data variants from the given start date to the given end date
         """
@@ -1126,12 +1287,6 @@ self.replace_where(
         """
         self.data = 1 / (1 + _np.exp(-self.data))
 
-    for n in FUNCTION_ADAPTATIONS_ELEMENTWISE:
-        exec(f"def {n}(self, *args, **kwargs, ): self.data = _ELEMENTWISE_FUNCTIONS['{n}'](self.data, *args, **kwargs, )")
-
-    for n in FUNCTION_ADAPTATIONS_TIMEWISE_NUMPY:
-        exec(f"def {n}(self, *args, **kwargs, ): self.data = _np.{n}(self.data, axis=1, *args, **kwargs, ).reshape(-1, 1)")
-
     #]
 
 
@@ -1170,56 +1325,6 @@ def _create_data_variant_from_number(
     return _np.full((len(span), 1), number, dtype=data_type)
 
 
-class series:
-    """
-    """
-    #[
-
-    for n in FUNCTION_ADAPTATIONS_TIMEWISE_NUMPY:
-        exec(_tw.dedent(f"""
-            @staticmethod
-            def {n}(self: Series, *args, **kwargs, ) -> None:
-                new = self.copy()
-                new.{n}(*args, **kwargs, )
-                return new
-        """))
-
-    #]
-
-
-for n in FUNCTION_ADAPTATIONS_ELEMENTWISE:
-    exec(
-        f"@_ft.singledispatch\n"
-        f"def {n}(x, *args, **kwargs):\n"
-        f"    return _ELEMENTWISE_FUNCTIONS['{n}'](x, *args, **kwargs)\n"
-    )
-    exec(
-        f"@{n}.register(Series, )\n"
-        f"def _(x, *args, **kwargs):\n"
-        f"    new = x.copy()\n"
-        f"    new.{n}(*args, **kwargs)\n"
-        f"    return new\n"
-    )
-
-
-for n in FUNCTION_ADAPTATIONS_BUILTINS:
-    exec(
-        f"_builtin_{n} = {n}"
-    )
-#     exec(
-#         f"@_ft.singledispatch"
-#         f"\n"
-#         f"def {n}(x, *args, **kwargs): "
-#         f"return _builtin_{n}(x, *args, **kwargs)"
-#     )
-#     exec(
-#         f"@{n}.register(Series, )"
-#         f"\n"
-#         f"def {n}(x, *args, **kwargs): "
-#         f"return x._{n}_(*args, **kwargs)"
-#     )
-
-
 def _from_periods_and_values(
     self,
     periods: Iterable[Period] | str,
@@ -1272,9 +1377,9 @@ def _from_start_and_values(
         values = _reshape_numpy_array(values, )
     else:
         values = _has_variants.iter_variants(values, )
-        values = _np.column_stack(tuple(
+        values = _np.column_stack([
             v for v, _ in zip(values, range(self.num_variants), )
-        ))
+        ])
     values = values.astype(self.data_type, )
     self.data = values
     self.trim()
@@ -1328,20 +1433,8 @@ _SERIES_POPULATOR = {
 }
 
 
-def shift(
-    self,
-    by: int | str = -1,
-) -> _series.Series:
-    """
-    """
-    #[
-    new = self.copy()
-    new.shift(by)
-    return new
-    #]
-
-
-for n in ("underlay", "overlay", "redate", ):
-    exec(_functionalize.FUNC_STRING.format(n=n, ), globals(), locals(), )
+for n in ("shift", "redate", "underlay", "overlay", ):
+    code = FUNC_STRING.format(n=n, )
+    exec(code, globals(), locals(), )
     __all__ += (n, )
 
