@@ -1,9 +1,10 @@
 """
-Simultaneous module
+Simultaneous models
 """
 
 from .main import *
 from .main import __all__ as main_all
 
-__all__ = main_all
+__all__ = []
+__all__.extend(main_all, )
 
