@@ -2,37 +2,34 @@
 """
 
 
-#
+#[
+
 from __future__ import annotations
 
-from typing import (Literal, Callable, )
-from collections.abc import (Iterable, )
-from numbers import (Real, )
+from typing import Literal, Callable
+from collections.abc import Iterable
+from numbers import Real
 import functools as _ft
 import json as _js
 import numpy as _np
 
 from .. import equations as _equations
 from .. import quantities as _quantities
-from .. import sources as _sources
-from .. import dates as _dates
+from ..quantities import QuantityKind
 from .. import has_variants as _has_variants
+from ..has_variants import unpack_singleton_decorator as _unpack_singleton
 from ..series import main as _series
 from ..incidences import main as _incidence
-from ..databoxes.main import (Databox, )
-from ..fords.solutions import (VariableStability, Solution, )
+from ..databoxes.main import Databox
+from ..fords.solutions import EigenvalueKind, Solution
 from ..fords import descriptors as _descriptors
 
+
 from . import _flags
+
 #]
 
 
-"""
-Quantities that are time series in model databoxes
-"""
-_TIME_SERIES_QUANTITY = _quantities.QuantityKind.ANY_VARIABLE | _quantities.QuantityKind.ANY_SHOCK
-
-
 def _cast_as_output_type(func: Callable, ):
     """
     """
@@ -46,17 +43,6 @@ def _cast_as_output_type(func: Callable, ):
     #]
 
 
-def _unpack_singleton(func: Callable, ):
-    #[
-    @_ft.wraps(func)
-    def _wrapper(self, *args, **kwargs, ):
-        unpack_singleton = kwargs.pop("unpack_singleton", True)
-        output = func(self, *args, **kwargs)
-        return self.unpack_singleton(output, unpack_singleton=unpack_singleton, )
-    return _wrapper
-    #]
-
-
 def _unpack_singleton_in_dict(func: Callable, ):
     #[
     @_ft.wraps(func)
@@ -68,7 +54,9 @@ def _unpack_singleton_in_dict(func: Callable, ):
     #]
 
 
-class Inlay:
+class Inlay(
+    _quantities.Mixin,
+):
     """
     Frontend getter methods for Simultaneous objects
     """
@@ -125,7 +113,7 @@ steady_levels = self.get_steady_levels(
         """
         kind = kind if kind is not None else _resolve_steady_kind(**kwargs, )
         qids = _quantities.generate_qids_by_kind(self._invariant.quantities, kind, )
-        return self._get_values("levels", qids, **kwargs, )
+        return self._get_values_as_dict("levels", qids, **kwargs, )
 
     @_cast_as_output_type
     @_unpack_singleton_in_dict
@@ -181,7 +169,7 @@ steady_changes = self.get_steady_changes(
         """
         kind = kind if kind is not None else _resolve_steady_kind(**kwargs, )
         qids = _quantities.generate_qids_by_kind(self._invariant.quantities, kind, )
-        return self._get_values("changes", qids, **kwargs, )
+        return self._get_values_as_dict("changes", qids, **kwargs, )
 
     @_cast_as_output_type
     @_unpack_singleton_in_dict
@@ -196,8 +184,8 @@ steady_changes = self.get_steady_changes(
         """
         kind = kind if kind is not None else _resolve_steady_kind(**kwargs, )
         qids = tuple(_quantities.generate_qids_by_kind(self._invariant.quantities, kind, ))
-        levels = self._get_values("levels", qids, **kwargs, )
-        changes = self._get_values("changes", qids, **kwargs, )
+        levels = self._get_values_as_dict("levels", qids, **kwargs, )
+        changes = self._get_values_as_dict("changes", qids, **kwargs, )
         return {
             k: list(zip(levels[k], changes[k]))
             for k in levels.keys()
@@ -214,7 +202,7 @@ steady_changes = self.get_steady_changes(
         """
         """
         qids = _quantities.generate_qids_by_kind(self._invariant.quantities, _quantities.QuantityKind.PARAMETER, )
-        return self._get_values("levels", qids, **kwargs, )
+        return self._get_values_as_dict("levels", qids, **kwargs, )
 
     @_cast_as_output_type
     @_unpack_singleton_in_dict
@@ -228,7 +216,7 @@ steady_changes = self.get_steady_changes(
         """
         """
         std_qids = self._get_std_qids(kind=kind, )
-        return self._get_values("levels", std_qids, **kwargs, )
+        return self._get_values_as_dict("levels", std_qids, **kwargs, )
 
     def _get_std_qids(
         self,
@@ -252,18 +240,7 @@ steady_changes = self.get_steady_changes(
         """
         """
         qids = _quantities.generate_qids_by_kind(self._invariant.quantities, _quantities.QuantityKind.PARAMETER_OR_STD, )
-        return self._get_values("levels", qids, **kwargs, )
-
-    @_cast_as_output_type
-    def get_log_status(
-        self,
-        /,
-    ) -> dict[str, bool]:
-        return {
-            qty.human: qty.logly
-            for qty in self._invariant.quantities
-            if qty.kind in _sources.LOGLY_VARIABLE
-        }
+        return self._get_values_as_dict("levels", qids, **kwargs, )
 
     def get_initials(
         self,
@@ -283,89 +260,63 @@ steady_changes = self.get_steady_changes(
         return _incidence.print_tokens(initial_tokens, qid_to_name, qid_to_logly, )
         # [^1] Do not wrap initial conditions in log(...)
 
-    # REFACTOR:
-    # Create a SteadyBoxable class
-    def generate_steady_items(
-        self,
-        start_date: _dates.Dater,
-        end_date: _dates.Dater,
-        /,
-        deviation: bool = False,
-    ) -> Iterable[tuple[str, _series.Series | Real]]:
+    def _get_dynamic_solution_vectors(self, ) -> _descriptors.SolutionVectors:
+        r"""
+        FordSimulatableProtocol
         """
-        """
-        num_columns = int(end_date - start_date + 1)
-        shift_in_first_column = start_date.get_distance_from_origin()
-        qid_to_name = self.create_qid_to_name()
-        qid_to_description = self.create_qid_to_description()
-        #
-        array = self.create_some_array(
-            deviation=deviation,
-            num_columns=num_columns,
-            shift_in_first_column=shift_in_first_column,
-        )
-        num_rows = array.shape[0]
-        #
-        qid_to_kind = self.create_qid_to_kind()
-        remove_qids = tuple(
-            qid
-            for qid, kind in qid_to_kind.items()
-            if kind not in _TIME_SERIES_QUANTITY
-        )
-        names = (
-            qid_to_name[qid]
-            for qid in range(num_rows, )
-            if qid in remove_qids
-        )
-        descriptions = (
-            qid_to_description[qid]
-            for qid in range(num_rows, )
-            if qid in remove_qids
-        )
-        for qid in qid_to_name.keys():
-            if qid_to_kind[qid] in _TIME_SERIES_QUANTITY:
-                yield qid_to_name[qid], _series.Series(
-                    start_date=start_date,
-                    values=array[qid, :].reshape(-1, 1),
-                    description=qid_to_description[qid],
-                )
-            else:
-                yield qid_to_name[qid], array[qid, 0]
+        return self._invariant.dynamic_descriptor.solution_vectors
 
-    def get_solution_vectors(self, /, ) -> _descriptors.HumanSolutionVectors:
+    def get_solution_vectors(self, ) -> _descriptors.HumanSolutionVectors:
         """
         Get the solution vectors of the model
         """
         qid_to_name = self.create_qid_to_name()
         qid_to_logly = self.create_qid_to_logly()
         return _descriptors.HumanSolutionVectors(
-            self.solution_vectors,
+            self._dynamic_solution_vectors,
             qid_to_name,
             qid_to_logly,
         )
 
     def get_solution(
         self,
-        /,
+        #
         unpack_singleton: bool = True,
     ) -> Solution | list[Solution]:
         """
         """
-        solution_matrices = [ v.solution for v in self._variants ]
+        solution_matrices = [ i for i in self.iter_solution() ]
         return self.unpack_singleton(solution_matrices, unpack_singleton=unpack_singleton, )
 
-    def get_singleton_solution(
+    def iter_solution(self, /, ) -> Iterable[Solution]:
+        """
+        """
+        for v in self._variants:
+            yield v.solution
+
+    def _get_singleton_solution(
         self,
         deviation: bool = False,
         vid: int = 0,
     ) -> Solution:
+        r"""
+        """
+        return (
+            self._variants[vid].solution if not deviation
+            else self._variants[vid].deviation_solution
+        )
+
+    def iter_std_name_to_value(self, /, ) -> Iterable[dict[str, Real]]:
         """
         """
-        solution = self._variants[vid].solution
-        if not deviation:
-            return solution
-        else:
-            return Solution.deviation_solution(solution)
+        qid_to_name = self.create_qid_to_name()
+        std_qids = _quantities.generate_qids_by_kind(self._invariant.quantities, _quantities.QuantityKind.ANY_STD, )
+        for v in self._variants:
+            std_qid_to_value = v.retrieve_levels_as_dict(std_qids, )
+            yield {
+                qid_to_name[qid]: value
+                for qid, value in std_qid_to_value.items()
+            }
 
     def get_dynamic_equations(
         self,
@@ -415,25 +366,6 @@ steady_changes = self.get_steady_changes(
             if kind else self._invariant.steady_equations
         )
 
-    def get_quantities(
-        self,
-        /,
-        *,
-        kind: _quantities.QuantityKind | None = None,
-    ) -> _quantities.Quantities:
-        return tuple(
-            _quantities.generate_quantities_of_kind(self._invariant.quantities, kind, )
-            if kind else self._invariant.quantities
-        )
-
-    def get_names(
-        self,
-        /,
-        *,
-        kind: _quantities.QuantityKind | None = None,
-    ) -> _quantities.Quantities:
-        return tuple(q.human for q in self.get_quantities(kind=kind, ))
-
     def get_flags(
         self,
         /,
@@ -446,7 +378,7 @@ steady_changes = self.get_steady_changes(
     ) -> dict[str, Callable]:
         return self._invariant._context
 
-    def _get_values(
+    def _get_values_as_dict(
         self,
         variant_attr: Literal["levels", "changes"],
         qids: Iterable[int],
@@ -457,24 +389,21 @@ steady_changes = self.get_steady_changes(
         """
         custom_round = kwargs.pop("round", None, )
         def apply(value: Real) -> Real:
-            value = float(value)
-            return round(value, custom_round) if custom_round is not None else value
+            return (
+                round(value, custom_round)
+                if custom_round is not None and value is not None
+                else value
+            )
         #
-        qids = tuple(qids)
         qid_to_name = self.create_qid_to_name()
         out_dict = {}
-        #
-        for v in self._variants:
-            values = v.retrieve_values(variant_attr, qids)
-            for qid, value in zip(qids, values):
-                out_dict.setdefault(qid_to_name[qid], []).append(apply(value))
+        for qid in qids:
+            out_dict[qid_to_name[qid]] = [
+                apply(getattr(v, variant_attr, )[qid], )
+                for v in self._variants
+            ]
         return out_dict
 
-    def get_quantity_descriptions(self, ) -> dict[str, str]:
-        """
-        """
-        return _quantities.create_name_to_description(self._invariant.quantities, )
-
     def get_equation_descriptions(self, ) -> dict[str, str]:
         """
         """
@@ -485,20 +414,28 @@ steady_changes = self.get_steady_changes(
         self,
         /,
         *,
+        kind: EigenvalueKind = EigenvalueKind.ALL,
         unpack_singleton: bool = True,
     ) -> tuple[Real, ...] | list[tuple[Real, ...]]:
         """
         Eigenvalues
         """
-        return [ v.solution.eigenvalues for v in self._variants ]
+        return [
+            tuple(e for e, s in zip(v.solution.eigenvalues, v.solution.eigenvalues_stability, ) if s in kind)
+            for v in self._variants
+        ]
 
     @_unpack_singleton
     def get_eigenvalues_stability(
         self,
         *,
+        kind: EigenvalueKind = EigenvalueKind.ALL,
         unpack_singleton: bool = True,
     ):
-        return [ v.solution.eigenvalues_stability for v in self._variants ]
+        return [ 
+            tuple(i for i in v.solution.eigenvalues_stability if i in kind)
+            for v in self._variants
+        ]
 
     def _get_variable_stability_for_variant(self, variant, /, ) -> dict[str, bool]:
         """
@@ -507,12 +444,12 @@ steady_changes = self.get_steady_changes(
         qid_to_name = self.create_qid_to_name()
         return {
             qid_to_name[token.qid]: \
-                variant.solution.transition_vector_stability[index] == VariableStability.STABLE
+                variant.solution.transition_vector_stability[index] == EigenvalueKind.STABLE
             for index, token in enumerate(vec.transition_variables)
             if token.shift == 0
         } | {
             qid_to_name[token.qid]: \
-                variant.solution.measurement_vector_stability[index] == VariableStability.STABLE
+                variant.solution.measurement_vector_stability[index] == EigenvalueKind.STABLE
             for index, token in enumerate(vec.measurement_variables)
             if token.shift == 0
         }
@@ -552,6 +489,10 @@ steady_changes = self.get_steady_changes(
     #]
 
 
+# Decorate methods from quantities.Mixin
+Inlay.get_log_status = _cast_as_output_type(Inlay.get_log_status, )
+
+
 def _resolve_steady_kind(
     *,
     include_shocks: bool = False,
@@ -561,8 +502,8 @@ def _resolve_steady_kind(
     """
     #[
     return (
-        _sources.LOGLY_VARIABLE if not include_shocks
-        else _sources.LOGLY_VARIABLE_OR_ANY_SHOCK
+        QuantityKind.LOGGABLE_VARIABLE if not include_shocks
+        else QuantityKind.LOGGABLE_VARIABLE_OR_ANY_SHOCK
     )
     #]
 
