@@ -3,30 +3,38 @@
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (TYPE_CHECKING, )
-from collections import (namedtuple, )
+from collections import namedtuple
 import functools as _ft
 import itertools as _it
 import numpy as _np
 import scipy as _sp
+import neqs as _ne
 
 from .. import equations as _equations
 from ..incidences import blazer as _blazer
+from ..incidences.blazer import Block, HumanBlock
 from .. import has_variants as _has_variants
 from .. import wrongdoings as _wrongdoings
 from ..fords import steadiers as _fs
 from ..steadiers import evaluators as _evaluators
+from ..steadiers import solver_dispatcher as _solver_dispatcher
+from ..equations import ENDOGENOUS_EQUATION
+from ..quantities import QuantityKind
 
 from . import _flags
 
+from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from ..plans.steady_plans import (SteadyPlan, )
-    from ..equations import (Equation, )
-    from ._variants import (Variant, )
-    from collections.abc import (Iterable, Callable, )
-    from typing import (Any, Literal, NoReturn, )
+    from ..plans.steady_plans import SteadyPlan
+    from ..equations import Equation
+    from ._variants import Variant
+    from collections.abc import Iterable, Callable
+    from typing import Any, Literal, NoReturn
+    from ..steadiers.solver_dispatcher import SolverType
+
 #]
 
 
@@ -42,6 +50,11 @@ _Wrt = namedtuple("Wrt", (
 ))
 
 
+_STEADY_EQUATION_SOLVED = ENDOGENOUS_EQUATION
+
+_DEFAULT_SOLVER = "neqs_levenberg"
+
+
 class Inlay:
     """
     """
@@ -142,17 +155,17 @@ class Inlay:
         *,
         plan: SteadyPlan | None = None,
         split_into_blocks: bool | None = None,
-        root_settings: dict[str, Any] | None = None,
+        solver: SolverType = _DEFAULT_SOLVER,
+        solver_settings: dict[str, Any] | None = None,
         iter_printer_settings: dict[str, Any] | None = None,
         **kwargs,
     ) -> dict[str, Any]:
         """
         """
-        root_settings = (
-            _DEFAULT_ROOT_SETTINGS
-            if root_settings is None
-            else _DEFAULT_ROOT_SETTINGS | root_settings
-        )
+        iter_printer_settings = iter_printer_settings or {}
+        #
+        qid_to_kind = self.create_qid_to_kind()
+        qid_to_name = self.create_qid_to_name()
         #
         split_into_blocks = _resolve_split_into_blocks(split_into_blocks, plan, )
         wrt = self._resolve_steady_wrt(plan, is_flat=model_flags.is_flat, )
@@ -160,23 +173,29 @@ class Inlay:
             im = _calculate_steady_incidence_matrix(wrt.equations, wrt.qids, )
             blocks = _blazer.blaze(im, wrt.eids, wrt.qids, )
         else:
-            blocks = (_blazer._Block(wrt.eids, wrt.qids), )
+            blocks = (_blazer.Block(wrt.eids, wrt.qids), )
+        num_blocks = len(blocks)
         #
         all_quantities = self.get_quantities()
-        all_equations = self.get_steady_equations()
+        all_equations = self.get_steady_equations(kind=_STEADY_EQUATION_SOLVED, )
         human_blocks \
             = _blazer.human_blocks_from_blocks \
             (blocks, all_equations, all_quantities, )
         #
-        info = {
-            "variant_success": None,
-            "blocks": human_blocks,
-            "block_success": [],
-            "block_root_final": [],
+        info_v = {
+            "success": None,
+            "blocks": [],
         }
         #=======================================================================
         for bid, (block, human_block) in enumerate(zip(blocks, human_blocks, ), ):
-            header_message = f"[Variant {vid}][Block {bid}]"
+            #
+            info_b = {
+                "success": None,
+                "exit_status": None,
+                "equations": human_block.equations,
+                "quantities": human_block.quantities,
+            }
+            custom_header = f"[Variant {vid}][Block {bid}]"
             #
             block_level_qids = tuple(sorted(set(block.qids) - set(wrt.fixed_level_qids)))
             block_change_qids = tuple(sorted(set(block.qids) - set(wrt.fixed_change_qids)))
@@ -184,10 +203,13 @@ class Inlay:
             #
             has_no_qids = (not block_level_qids) and (not block_change_qids)
             has_no_equations = not block_equations
+            #
+            solve = getattr(_solver_dispatcher, solver)
             if has_no_qids or has_no_equations:
-                info["block_success"].append(True, )
-                info["block_root_final"].append(None, )
-                print(f"\n-Skipping {header_message}")
+                info_b["success"] = True
+                info_b["exit_status"] = _ne.ExitStatus.NO_SOLVER_NEEDED
+                info_v["blocks"].append(info_b, )
+                print(f"\n-Skipping {custom_header}")
                 continue
             #
             steady_evaluator = evaluator_class(
@@ -196,34 +218,31 @@ class Inlay:
                 block_equations,
                 all_quantities,
                 variant,
-                iter_printer_settings=iter_printer_settings,
+                context=self._invariant._context,
+                iter_printer_settings=iter_printer_settings | {"custom_header": custom_header},
             )
-            steady_evaluator.iter_printer.header_message = header_message
-            root_final = _sp.optimize.root(
-               steady_evaluator.eval,
-               steady_evaluator.get_init_guess(),
-               jac=True,
-               **root_settings,
-            )
-            steady_evaluator.iter_printer.print_footer()
-            steady_evaluator.final_guess = root_final.x
             #
-            func_norm = _sp.linalg.norm(root_final.fun, 2, )
-            success = root_final.success and func_norm < root_settings["tol"]
+            maybelog_init_guess = steady_evaluator.get_init_guess()
+            maybelog_final_guess, success, exit_status = solve(
+                steady_evaluator,
+                maybelog_init_guess,
+                solver_settings=solver_settings,
+            )
+            steady_evaluator.final_guess = maybelog_final_guess
             #
             if not success:
-                _throw_block_error(human_block, header_message, )
+                _throw_block_error(human_block, custom_header, )
             #
             # Update variant with steady levels and changes
-            _update_variant_with_final_guess(variant, steady_evaluator, )
+            _update_variant_with_final_guess(variant, steady_evaluator, qid_to_kind, qid_to_name, )
             #
-            info["block_success"].append(success, )
-            info["block_root_final"].append(root_final, )
+            info_b["success"] = success
+            info_b["exit_status"] = exit_status
+            info_v["blocks"].append(info_b, )
         #=======================================================================
         #
-        info["variant_success"] = all(info["block_success"])
-        #
-        return info
+        info_v["success"] = all(i["success"] for i in info_v["blocks"])
+        return info_v
 
     _steady_nonlinear_flat = _ft.partialmethod(
         _steady_nonlinear,
@@ -242,7 +261,7 @@ class Inlay:
     ) -> _Wrt:
         """
         """
-        wrt_equations = self.get_steady_equations()
+        wrt_equations = self.get_steady_equations(kind=_STEADY_EQUATION_SOLVED, )
         wrt_eids = tuple(e.id for e in wrt_equations)
         #
         plannable = self.get_steady_plannable(is_flat=is_flat, )
@@ -256,7 +275,7 @@ class Inlay:
             exogenized_names = plan.get_exogenized_names()
             endogenized_names = plan.get_endogenized_names()
             fixed_level_names = plan.get_fixed_level_names()
-            fixed_change_names = plan.get_fixed_change_names()
+            fixed_change_names = plan.get_fixed_change_names() + endogenized_names
         #
         wrt_names = (wrt_names - set(exogenized_names)) | set(endogenized_names)
         # wrt_level_names = (wrt_names - set(fixed_level_names))
@@ -266,9 +285,11 @@ class Inlay:
         name_to_qid = self.create_name_to_qid()
         # wrt_level_qids = tuple(sorted([name_to_qid[name] for name in wrt_level_names]))
         # wrt_change_qids = tuple(sorted([name_to_qid[name] for name in wrt_change_names]))
-        wrt_qids = tuple(sorted([name_to_qid[name] for name in wrt_names]))
-        wrt_fixed_level_qids = tuple(sorted([name_to_qid[name] for name in fixed_level_names]))
-        wrt_fixed_change_qids = tuple(sorted([name_to_qid[name] for name in fixed_change_names]))
+        def get_sorted_qids(names: Iterable[str], ) -> tuple[int, ...]:
+            return tuple(sorted([name_to_qid[name] for name in names]))
+        wrt_qids = get_sorted_qids(wrt_names, )
+        wrt_fixed_level_qids = get_sorted_qids(fixed_level_names, )
+        wrt_fixed_change_qids = get_sorted_qids(fixed_change_names, )
         #
         # Make order of names consistent with qids
         qid_to_name = self.create_qid_to_name()
@@ -334,13 +355,14 @@ class Inlay:
         # REFACTOR
         #
         t_zero = -equator.min_shift
-        discrepancies = [
-            _np.hstack((
-                _np.array(equator.eval(x, t_zero, )).reshape(-1, 1),
-                _np.array(equator.eval(x, t_zero+1, )).reshape(-1, 1),
-            ))
-            for x in steady_arrays
-        ]
+        discrepancies = []
+        for x in steady_arrays:
+            at_t_zero = equator.eval(x, t_zero, )
+            at_t_zero_plus_one = equator.eval(x, t_zero + 1, )
+            discrepancies.append(_np.hstack((
+                _np.array(at_t_zero).reshape(-1, 1),
+                _np.array(at_t_zero_plus_one).reshape(-1, 1),
+            )))
         #
         # REFACTOR
         #
@@ -357,18 +379,37 @@ class Inlay:
         fail_stream._raise()
         all_status = all(status)
         #
-        if return_info:
-            info = [
-                {"discrepancies": d, }
-                for d in discrepancies
-            ]
-            info = _has_variants.unpack_singleton(
-                info, self.is_singleton,
-                unpack_singleton=unpack_singleton,
-            )
-            return all_status, info
-        else:
+        if not return_info:
             return all_status
+        #
+        info = [
+            {"discrepancies": d, }
+            for d in discrepancies
+        ]
+        info = _has_variants.unpack_singleton(
+            info, self.is_singleton,
+            unpack_singleton=unpack_singleton,
+        )
+        return all_status, info,
+
+
+    def split_into_blocks(
+        self,
+        plan: SteadyPlan,
+        **kwargs,
+    ) -> tuple[HumanBlock, ...]:
+        r"""
+        """
+        model_flags = self.resolve_flags(**kwargs, )
+        wrt = self._resolve_steady_wrt(plan, is_flat=model_flags.is_flat, )
+        im = _calculate_steady_incidence_matrix(wrt.equations, wrt.qids, )
+        blocks = _blazer.blaze(im, wrt.eids, wrt.qids, )
+        equations = self._invariant.steady_equations
+        quantities = self._invariant.quantities
+        return tuple(
+            HumanBlock(i, equations, quantities, )
+            for i in blocks
+        )
 
     #]
 
@@ -424,7 +465,7 @@ def _resolve_split_into_blocks(
     #]
 
 
-def _update_variant_with_final_guess(variant, steady_evaluator, ) -> None:
+def _update_variant_with_final_guess(variant, steady_evaluator, qid_to_kind, qid_to_name, ) -> None:
     """
     """
     #[
@@ -434,22 +475,30 @@ def _update_variant_with_final_guess(variant, steady_evaluator, ) -> None:
     variant.update_levels_from_array(levels, wrt_level_qids, )
     #
     changes, wrt_change_qids = steady_evaluator.extract_changes(final_guess, )
-    variant.update_changes_from_array(changes, wrt_change_qids, )
+    #
+    # Filter out non-loggable quantities, i.e. endogenized parameters (only loggable
+    # quantities can have steady-state change)
+    changes_to_update = []
+    wrt_change_qids_to_update = []
+    for change, qid in zip(changes, wrt_change_qids, ):
+        if qid_to_kind[qid] not in QuantityKind.LOGGABLE_VARIABLE:
+            continue
+        changes_to_update.append(change)
+        wrt_change_qids_to_update.append(qid)
+    variant.update_changes_from_array(changes_to_update, wrt_change_qids_to_update, )
     #]
 
 
-def _throw_block_error(human_block, header_message: str, ) -> NoReturn:
+def _throw_block_error(human_block, custom_header: str, ) -> NoReturn:
     """
     """
     #[
-    message = (f"Steady state calculations failed to converge in {header_message}", )
-    message += human_block.equations
+    message = (
+        f"Steady state calculations failed to converge in {custom_header}",
+        *human_block.equations,
+        "Solved for " + " ".join(human_block.quantities, ),
+    )
     raise _wrongdoings.IrisPieError(message, )
     #]
 
 
-_DEFAULT_ROOT_SETTINGS = {
-    "method": "lm",
-    "tol": 1e-12,
-}
-
