@@ -4,25 +4,28 @@ Simultaneous models
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (Self, Any, TypeAlias, Literal, )
+from typing import Self, Any, TypeAlias, Literal
 from types import EllipsisType
-from collections.abc import (Iterable, Iterator, Callable, )
-from numbers import (Number, )
-import copy as _co
+from collections.abc import Iterable, Iterator, Callable
+from numbers import Number
 import numpy as _np
 import itertools as _it
 import functools as _ft
+import documark as _dm
 
 from .. import has_invariant as _has_invariant
 from .. import has_variants as _has_variants
 from .. import equations as _equations
 from .. import quantities as _quantities
+from .. import portables as _portables
+from ..quantities import QuantityKind, Quantity
+from ..sources import ModelSource
 from .. import sources as _sources
 from .. import dates as _dates
 from .. import wrongdoings as _wrongdoings
-from .. import pages as _pages
 from ..conveniences import iterators as _iterators
 from ..parsers import common as _pc
 from ..databoxes import main as _databoxes
@@ -34,18 +37,21 @@ from ..fords import systems as _systems
 from ..fords import kalmans as _kalmans
 from ..fords import std_simulators as _std_simulators
 
-from ._invariants import (Invariant, )
-from ._variants import (Variant, )
-from . import _covariances as _covariances
-from . import _flags as _flags
-from . import _simulate as _simulate
-from . import _steady as _steady
-from . import _logly as _logly
-from . import _get as _get
-from . import _pretty as _pretty
-from . import _assigns as _assigns
-from . import _slatable_protocols as _slatable_protocols
-from . import _plannable_protocols as _plannable_protocols
+from ._invariants import Invariant
+from ._variants import Variant
+from . import _covariances
+from . import _flags
+from . import _simulate
+from . import _steady
+from . import _logly
+from . import _get
+from . import _pretty
+from . import _assigns
+from . import _slatable_protocols
+from . import _plannable_protocols
+from . import _steady_boxable_protocols
+from . import _io
+
 #]
 
 
@@ -55,19 +61,23 @@ __all__ = [
 ]
 
 
-@_pages.reference(
+_DEFAULT_SOLUTION_TOLERANCE = 1e-12
+
+
+@_dm.reference(
     path=("structural_models", "simultaneous.md", ),
     categories={
-        "constructor": "Creating new simultaneous models",
-        "filtering": "Applying structural filters on models",
-        "information": "Getting information about models",
-        "parameters": "Manipulating model parameters",
+        "constructor": "Creating new Simultaneous models",
+        "information": "Getting information about Simultaneous models",
+        "filtering": "Applying structural filters on Simultaneous models",
+        "parameters": "Manipulating Simultaneous model parameters",
+        "serialize": "Serializing, saving and loading Simultaneous models",
         "property": None,
     },
 )
 class Simultaneous(
-    _has_invariant.HasInvariantMixin,
-    _has_variants.HasVariantsMixin,
+    _has_invariant.Mixin,
+    _has_variants.Mixin,
     _kalmans.Mixin,
     _std_simulators.Mixin,
 
@@ -80,14 +90,17 @@ class Simultaneous(
     _covariances.Inlay,
     _slatable_protocols.Inlay,
     _plannable_protocols.Inlay,
+    _steady_boxable_protocols.Inlay,
+    _io.Inlay,
 ):
     """
-················································································
+................................................................................
 
 `Simultaneous` models
 ======================
 
-················································································
+
+................................................................................
     """
     #[
 
@@ -99,50 +112,37 @@ class Simultaneous(
     def __init__(
         self,
         invariant: Invariant | None = None,
-        variants: list[Variant] | None = None,
+        variants: Iterable[Variant] | None = None,
     ) -> None:
-        """
+        r"""
         """
         self._invariant = invariant
-        self._variants = variants or []
+        self._variants = list(variants) if variants else []
 
     @classmethod
-    def skeleton(
-        klass,
-        other,
-        /,
-    ) -> Self:
-        """
-        """
-        self = klass()
-        self._invariant = other._invariant
-        return self
-
-    @classmethod
-    @_pages.reference(category="constructor", call_name="Simultaneous.from_file", )
+    @_dm.reference(category="constructor", call_name="Simultaneous.from_file", )
     def from_file(klass, *args, **kwargs, ) -> _sources.SourceMixinProtocol:
-        """
+        r"""
 ················································································
 
 ==Create `Simultaneous` model object from source file or files==
 
-```
-self = Simultaneous.from_file(
-    file_names,
-    /,
-    context=None,
-    description="",
-)
-```
-
 Read and parse one or more source files specified by `file_names` (a string
 or a list of strings) with model source code, and create a `Simultaneous`
 model object.
 
+```
+self = Simultaneous.from_file(
+    file_names,
+    #
+    context=None,
+    description="",
+)
+```
+
 
 ### Input arguments ###
 
-
 ???+ input "file_names"
     The name of the model source file from which the `Simultaneous` model object
     will be created, or a list of file names; if multiple file names are
@@ -158,35 +158,34 @@ model object.
 
 ### Returns ###
 
-
 ???+ returns "self"
-`Simultaneous` model object created from the `file_names`.
+    `Simultaneous` model object created from the `file_names`.
 
 ················································································
         """
         return _sources.from_file(klass, *args, **kwargs, )
 
     @classmethod
-    @_pages.reference(category="constructor", call_name="Simultaneous.from_string",)
+    @_dm.reference(category="constructor", call_name="Simultaneous.from_string",)
     def from_string(klass, *args, **kwargs, ) -> _sources.SourceMixinProtocol:
         """
 ················································································
 
 ==Create `Simultaneous` model from string==
 
+Read and parse a text `string` with a model source code, and create a
+`Simultaneous` model object. Otherwise, this function behaves the same way as
+[`Simultaneous.from_file`](#simultaneousfrom_file).
+
 ```
 self = Simultaneous.from_string(
     string,
-    /,
+    #
     context=None,
     description="",
 )
 ```
 
-Read and parse a text `string` with a model source code, and create a
-`Simultaneous` model object. Otherwise, this function behaves the same way as
-[`Simultaneous.from_file`](#simultaneousfrom_file).
-
 
 ### Input arguments ###
 
@@ -206,18 +205,18 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         """
         return _sources.from_string(klass, *args, **kwargs, )
 
-    def copy(self, /, ) -> Self:
+    def copy(self, ) -> Self:
         """
         Create a quasi-deep copy of this model
         """
-        new = type(self).skeleton(self, )
+        new = type(self)()
+        new._invariant = self._invariant.copy()
         new._variants = [ variant.copy() for variant in self._variants ]
         return new
 
     def __getitem__(
         self,
         request: str | int,
-        /,
     ):
         """
         Implement self[i] for variants and self[name] for quantitie values
@@ -231,7 +230,6 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         self,
         reference: str,
         value: Any,
-        /,
     ) -> None:
         """
         """
@@ -240,25 +238,23 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         else:
             raise NotImplementedError("Assigning model variants not implemented yet", )
 
-    def __repr__(self, /, ) -> str:
+    def __repr__(self, ) -> str:
         """
         """
         indented = " " * 4
         return "\n".join((
             f"",
-            f"{self.__class__.__name__} model",
-            f"Description: \"{self.get_description()}\"",
-            f"⏐",
+            f"<{self.__class__.__name__} at {id(self):#x}>",
+            f"[Description: \"{self.get_description()}\"]",
             f"[Num variants: {self.num_variants}]",
-            f"[Num (transition, measurement) equations: ({self.num_transition_equations}, {self.num_measurement_equations})]",
-            f"[Max (lag, lead): ({self.max_lag:+g}, {self.max_lead:+g})]",
+            f"[Num transition, measurement equations: {self.num_transition_equations}, {self.num_measurement_equations}]",
+            f"[Max lag, lead: t{self.max_lag:+g}, t{self.max_lead:+g}]",
             f"",
         ))
 
     def get_value(
         self,
         name: str,
-        /,
     ) -> Any:
         """
         """
@@ -267,7 +263,7 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         if invalid_names:
             raise _wrongdoings.IrisPieCritical(f"Invalid model name \"{invalid_names[0]}\"", )
         return _has_variants.unpack_singleton(
-            self._get_values("levels", qids, )[name],
+            self._get_values_as_dict("levels", qids, )[name],
             self.is_singleton,
         )
 
@@ -289,81 +285,65 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         self._invariant.quantities = _quantities.change_logly(self._invariant.quantities, new_logly, qids)
 
     @property
-    @_pages.reference(category="property", )
-    def is_linear(self, /, ) -> bool:
+    @_dm.reference(category="property", )
+    def is_linear(self, ) -> bool:
         """==True for models declared as linear=="""
         return self._invariant._flags.is_linear
 
     @property
-    @_pages.reference(category="property", )
-    def is_flat(self, /, ) -> bool:
+    @_dm.reference(category="property", )
+    def is_flat(self, ) -> bool:
         """==True for models declared as flat=="""
         return self._invariant._flags.is_flat
 
     @property
-    @_pages.reference(category="property", )
-    def is_deterministic(self, /, ) -> bool:
+    @_dm.reference(category="property", )
+    def is_deterministic(self, ) -> bool:
         """==True for models declared as deterministic=="""
         return self._invariant._flags.is_deterministic
 
     @property
-    @_pages.reference(category="property", )
-    def num_transition_equations(self, /, ) -> int:
+    @_dm.reference(category="property", )
+    def num_transition_equations(self, ) -> int:
         """==Number of transition equations=="""
         return self._invariant.num_transition_equations
 
     @property
-    @_pages.reference(category="property", )
-    def num_measurement_equations(self, /, ) -> int:
+    @_dm.reference(category="property", )
+    def num_measurement_equations(self, ) -> int:
         """==Number of measurement equations=="""
         return self._invariant.num_measurement_equations
 
     @property
-    @_pages.reference(category="property", )
-    def max_lag(self, /, ) -> int:
+    @_dm.reference(category="property", )
+    def max_lag(self, ) -> int:
         """==Maximul lag in the model (negative or zero)=="""
         return self._invariant._min_shift
 
     @property
-    @_pages.reference(category="property", )
-    def max_lead(self, /, ) -> int:
+    @_dm.reference(category="property", )
+    def max_lead(self, ) -> int:
         """==Maximul lead in the model (positive or zero)=="""
         return self._invariant._max_shift
 
     @property
-    def solution_vectors(self, /, ) -> _descriptors.SolutionVectors:
+    def solution_vectors(self, ) -> _descriptors.SolutionVectors:
         """
         """
         return self._invariant.dynamic_descriptor.solution_vectors
 
     @property
-    def quantities(self, /, ) -> tuple[_quantities.Quantity]:
+    def quantities(self, ) -> tuple[Quantity]:
         """==Tuple of model quantities=="""
         return self._invariant.quantities
 
     @property
-    def shock_qid_to_std_qid(self, /, ) -> dict[int, int]:
+    def shock_qid_to_std_qid(self, ) -> dict[int, int]:
         """==Dictionary mapping shock quantity id to standard deviation quantity id=="""
         return self._invariant.shock_qid_to_std_qid
 
-    def create_name_to_qid(self, /, ) -> dict[str, int]:
-        return _quantities.create_name_to_qid(self._invariant.quantities)
-
-    def create_qid_to_name(self, /, ) -> dict[int, str]:
-        return _quantities.create_qid_to_name(self._invariant.quantities)
-
-    def create_qid_to_kind(self, /, ) -> dict[int, str]:
-        return _quantities.create_qid_to_kind(self._invariant.quantities)
-
-    def create_qid_to_description(self, /, ) -> dict[int, str]:
-        """
-        Create a dictionary mapping from quantity id to quantity descriptor
-        """
-        return _quantities.create_qid_to_description(self._invariant.quantities)
-
     def create_steady_array(
         self,
-        /,
         variant: Variant | None = None,
         **kwargs,
     ) -> _np.ndarray:
@@ -376,7 +356,6 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
 
     def create_zero_array(
         self,
-        /,
         variant: Variant | None = None,
         **kwargs,
     ) -> _np.ndarray:
@@ -389,20 +368,21 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
 
     def create_some_array(
         self,
-        /,
         deviation: bool,
         **kwargs,
     ) -> _np.ndarray:
-        return {
-            True: self.create_zero_array, False: self.create_steady_array,
-        }[deviation](**kwargs)
+        """
+        """
+        if not deviation:
+            return self.create_steady_array(**kwargs, )
+        else:
+            return self.create_zero_array(**kwargs, )
 
-    def _enforce_assignment_rules(self, variant, /, ) -> None:
+    def _enforce_assignment_rules(self, variant, ) -> None:
         """
         """
         #
         # Reset levels of shocks to zero
-        #
         name_to_qid = self.create_name_to_qid()
         shock_qids = _quantities.generate_qids_by_kind(
             self._invariant.quantities, _quantities.QuantityKind.ANY_SHOCK,
@@ -410,17 +390,15 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         zero_shocks = { i: 0 for i in shock_qids }
         variant.update_values_from_dict(zero_shocks, )
         #
-        # Remove changes from quantities that are not logly variables
-        #
-        nonloglies = _quantities.generate_qids_by_kind(
-            self._invariant.quantities, ~_sources.LOGLY_VARIABLE,
+        # Remove changes from quantities that are not loggable
+        non_loggables = _quantities.generate_qids_by_kind(
+            self._invariant.quantities, ~QuantityKind.LOGGABLE_VARIABLE,
         )
-        assign_nonloglies = { qid: (..., _np.nan, ) for qid in nonloglies }
-        variant.update_values_from_dict(assign_nonloglies, )
+        assign_non_loggables = { qid: (..., _np.nan, ) for qid in non_loggables }
+        variant.update_values_from_dict(assign_non_loggables, )
 
     def systemize(
         self,
-        /,
         unpack_singleton: bool = True,
         **kwargs,
     ) -> Iterable[_systems.System]:
@@ -442,7 +420,6 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         variant: Variant,
         descriptor: _descriptors.Descriptor,
         model_flags: flags.Flags,
-        /,
     ) -> _systems.System:
         """
         Create unsolved first-order system for one variant
@@ -467,10 +444,10 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
 
     def solve(
         self,
-        /,
         clip_small: bool = False,
         return_info: bool = False,
         unpack_singleton: bool = True,
+        tolerance: Real = _DEFAULT_SOLUTION_TOLERANCE,
         **kwargs,
     ) -> dict[str, Any]:
         """
@@ -478,8 +455,14 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         """
         model_flags = self.resolve_flags(**kwargs, )
         out_info = [
-            self._solve(self_v, model_flags, clip_small=clip_small, )
-            for self_v in self._variants
+            self._solve_variant(
+                self_v,
+                vid,
+                model_flags,
+                clip_small=clip_small,
+                tolerance=tolerance,
+            )
+            for vid, self_v in enumerate(self._variants, )
         ]
         if return_info:
             out_info = _has_variants.unpack_singleton(
@@ -490,18 +473,35 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
         else:
             return
 
-    def _solve(
+    def _solve_variant(
         self,
         variant: Variant,
+        vid: int,
         model_flags: flags.Flags,
-        /,
         clip_small: bool,
+        tolerance: Real,
     ) -> None:
         """
         Calculate first-order solution for one variant of this model
         """
-        system = self._systemize(variant, self._invariant.dynamic_descriptor, model_flags, )
-        variant.solution = _solutions.Solution.from_system(self._invariant.dynamic_descriptor, system, clip_small=clip_small, )
+        variant_header = f"[Variant {vid}]"
+        system = self._systemize(
+            variant,
+            self._invariant.dynamic_descriptor,
+            model_flags,
+        )
+        try:
+            variant.solution = _solutions.Solution.from_system(
+                self._invariant.dynamic_descriptor,
+                system,
+                clip_small=clip_small,
+                tolerance=tolerance,
+            )
+        except _solutions.UnitRootException:
+            raise _wrongdoings.IrisPieCritical(
+                f"{variant_header} Inconsistency in classification of unit roots; "
+                "modify (increase) the tolerance level",
+            )
         info = {}
         #
         return info
@@ -509,7 +509,6 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
     def _choose_plain_equator(
         self,
         equation_switch: Literal["dynamic", "steady", ],
-        /,
     ) -> Callable | None:
         """
         """
@@ -519,7 +518,7 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
             case "steady":
                 return self._invariant._plain_steady_equator
 
-    def _assign_default_stds(self, /, ) -> None:
+    def reset_stds(self, ) -> None:
         """
         Initialize standard deviations of shocks to default values
         """
@@ -536,30 +535,234 @@ See [`Simultaneous.from_file`](simultaneousfrom_file) for return values.
     @classmethod
     def from_source(
         klass,
-        source: _sources.ModelSource,
-        /,
+        source: ModelSource,
         **kwargs,
     ) -> Self:
         """
         """
         self = klass()
-        self._invariant = Invariant(source, **kwargs, )
+        self._invariant = Invariant.from_source(source, **kwargs, )
+        self._initialize_variants_from_invariant()
+        return self
+
+    def _initialize_variants_from_invariant(self, ) -> None:
         initial_variant = Variant.from_source(
             self._invariant.quantities,
             self._invariant._flags.is_flat,
         )
         self._variants = [ initial_variant ]
-        self._assign_default_stds()
+        self.reset_stds()
         self._enforce_assignment_rules(self._variants[0], )
-        return self
 
-    def get_context(self, /, ) -> dict[str, Any]:
+    def get_context(self, ) -> dict[str, Any]:
         """
         """
         return self._invariant._context
 
-    def resolve_flags(self, /, **kwargs, ) -> _flags.Flags:
+    def resolve_flags(self, **kwargs, ) -> _flags.Flags:
         return _flags.Flags.update_from_kwargs(self.get_flags(), **kwargs)
+
+    def __getstate__(self, ):
+        """
+        """
+        return {
+            "_invariant": self._invariant,
+            "_variants": self._variants,
+        }
+
+    def __setstate__(self, state, ):
+        """
+        """
+        self._invariant = state["_invariant"]
+        self._variants = state["_variants"]
+
+    @_dm.reference(category="serialize", )
+    def to_portable(self, ) -> dict[str, Any]:
+        r"""
+................................................................................
+
+==Serialize `Simultaneous` model to portable dictionary==
+
+Convert a `Simultaneous` model object to a dictionary of primitive values that
+can be saved to a JSON file or transmitted over the network. The structure of
+the portable dictionary is described below.
+
+```
+portable = self.to_portable()
+```
+
+
+### Input arguments ###
+
+???+ input "self"
+    `Simultaneous` model object to be serialized.
+
+
+### Returns ###
+
+???+ returns "portable"
+    A JSON-serializable dictionary with the structure described below.
+
+
+### Details ###
+
+
+???+ abstract "Structure of the portable dictionary, format 0.3.0:"
+
+    The portable dictionary has the following structure:
+
+    ```
+        {
+            "portable_format": "0.3.0",
+            "source": {
+                "description": <str>,
+                "flags": {
+                    "is_linear: <bool>,
+                    "is_flat": <bool>,
+                    "is_deterministic": <bool>
+                },
+                "quantities": [ <QUANTITY>, ... ],
+                "equations": [ <EQUATION>, ... ],
+                "context": {},
+            "variants": [ <VARIANT>, ... ],
+        }
+    ```
+
+    The meaning of the `flags` is as follows:
+
+    * `is_linear` is `True` if the model has been created with the
+    `linear=True`, and the calculation of first-order solution matrices is
+    expected to be independent of the model steady state. * `is_flat` is `True`
+
+    * `is_flat` is `True` if the model has been created with the `flat=True`,
+    and the calculation of the model steady state is done assuming that no model
+    variable is growing over time in steady state.
+
+    * `is_deterministic` is `True` if the model has been created with the
+    `deterministic=True`, and the model is expected to have no stochastic
+    shocks; all shocks are assumed to be deterministic add-factors, and there
+    are no `std` parameter created.
+
+    The `quantities` is a list of quantities, with each `<QUANTITY>` described
+    as a five-element tuple:
+
+    ```
+    [ <KIND>, <NAME>, <LOGLY>, <DESCRIPTION>, <ATTRIBUTES> ]
+    ```
+
+    where
+
+    * `<KIND>` is a string representing the kind of the quantity,
+
+    * `<NAME>` is a string with the name of the quantity,
+
+    * `<LOGLY>` is a boolean or `None` indicating whether the quantity is
+    declared as a log variable `None` if irrelevant for the respective kind of variables),
+
+    * `<DESCRIPTION>` is a string with the description of the quantity,
+
+    * `<ATTRIBUTES>` is a string containing all attributes separated by a white space.
+
+
+    The kinds of variables are coded as follows:
+
+    | Kind of variable | Code |
+    |-------------------|------|
+    | Transition variable | `#x` |
+    | Measurement variable | `#y` |
+    | Unanticipated shock | `#u` |
+    | Anticipated shock | `#v` |
+    | Measurement shock | `#w` |
+    | Parameter | `#p` |
+    | Exogenous variable | `#z` |
+
+
+    The `equations` is a list of equations, with each `<EQUATION>` described as
+    a five-tuple:
+
+    ```
+    [ <KIND>, <DYNAMIC>, <STEADY>, <DESCRIPTION>, <ATTRIBUTES> ]
+    ```
+
+    where
+    * `<KIND>` is a string representing the kind of the equation,
+
+    * `<DYNAMIC>` is a string with the dynamic variant of the equation,
+
+    * `<STEADY>` is a string with the steady-state variant of the equation,
+
+    * `<DESCRIPTION>` is a string with the description of the equation,
+
+    * `<ATTRIBUTES>` is a string containing all attributes separated by a white space.
+
+
+    The kinds of equations are coded as follows:
+
+    | Kind of equation | Code |
+    |-------------------|------|
+    | Transition equation | `#T` |
+    | Measurement equation | `#M` |
+    | Steady autovalues | `#A` |
+
+
+    The `variants` is a list of parameter (and steady-state) variants, with each
+    `<VARIANT>` being a dictionary with its keys corresponding to the names of
+    the model quantities (including `std_` names for the standard deviations of
+    shocks), and the values being two-tuples consisting of the level and the
+    changes of the respective quantity; the change is `None` whenever irrelevant
+    for the respective kind of quantity.
+
+................................................................................
+        """
+        qid_to_name = self.create_qid_to_name()
+        return {
+            "portable_format": _portables.CURRENT_PORTABLE_FORMAT,
+            "source": self._invariant.to_portable(),
+            "variants": [ v.to_portable(qid_to_name, ) for v in self._variants ],
+        }
+
+    @classmethod
+    @_dm.reference(category="constructor", )
+    def from_portable(klass, portable, ) -> Self:
+        r"""
+................................................................................
+
+==Create `Simultaneous` model from portable dictionary==
+
+Create a `Simultaneous` model object from a portable dictionary. See
+`Simultaneous.to_portable` for details on the structure of the portable
+dictionary.
+
+```
+self = Simultaneous.from_portable(portable)
+```
+
+
+### Input arguments ###
+
+???+ input "portable"
+    A dictionary with the structure described in the documentation of
+    `Simultaneous.to_portable`.
+
+
+### Returns ###
+
+???+ returns "self"
+    A new `Simultaneous` model object created from the portable dictionary.
+
+................................................................................
+        """
+        _portables.validate_portable_format(portable["portable_format"], )
+        #
+        self = klass()
+        self._invariant = Invariant.from_portable(portable["source"], )
+        self._initialize_variants_from_invariant()
+        num_variants = len(portable["variants"])
+        self.alter_num_variants(num_variants, )
+        for self_v, portable_v in zip(self.iter_own_variants(), portable["variants"], ):
+            self_v.assign_strict(portable_v, )
+        return self
+
     #]
 
 
