@@ -4,17 +4,19 @@ Handle exceptions and warnings
 
 
 #[
+
 from __future__ import annotations
 
-from typing import (TypeAlias, Literal, Callable, NoReturn, )
-from collections.abc import (Iterable, )
+from typing import Literal, Callable, NoReturn
+from collections.abc import Iterable
 import warnings as _wa
 import os as _os
+
 #]
 
 
 _WARN_SKIPS = (_os.path.dirname(__file__), )
-HOW: TypeAlias = Literal["error", "warning", "silent"]
+HOW = Literal["error", "warning", "silent"]
 
 
 _PLAIN_PREFIX = ""
@@ -27,8 +29,8 @@ class IrisPieError(Exception, ):
     """
     #[
     def __init__(self, message, ):
-        message = _prepare_message(message)
-        super().__init__(message)
+        message = _prepare_message(message, )
+        super().__init__(message, )
     #]
 
 
@@ -127,7 +129,7 @@ _RESOLVE_HOW = {
 }
 
 
-class _Stream:
+class Stream:
     """
     """
     #[
@@ -148,6 +150,13 @@ class _Stream:
     ) -> None:
         ...
 
+    def add_from_iterable(
+        self,
+        messages: Iterable[str],
+    ) -> None:
+        for message in messages:
+            self.add(message, )
+
     def _raise(self, /, ) -> None:
         ...
 
@@ -158,7 +167,7 @@ class _Stream:
     #]
 
 
-class CriticalStream(_Stream):
+class CriticalStream(Stream):
     """
     """
     #[
@@ -178,7 +187,7 @@ class CriticalStream(_Stream):
     #]
 
 
-class ErrorStream(_Stream):
+class ErrorStream(Stream):
     """
     """
     #[
@@ -198,7 +207,7 @@ class ErrorStream(_Stream):
     #]
 
 
-class WarningStream(_Stream):
+class WarningStream(Stream):
     """
     """
     #[
@@ -218,7 +227,7 @@ class WarningStream(_Stream):
     #]
 
 
-class SilentStream(_Stream):
+class SilentStream(Stream):
     """
     """
     #[
@@ -240,7 +249,7 @@ STREAM_FACTORY = {
 }
 
 
-def create_stream(kind: str, title: str, /, when_no_stream=None, ) -> _Stream:
+def create_stream(kind: str, title: str, /, when_no_stream=None, ) -> Stream:
     """
     """
     #[
